/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: BuiltinDialog.cc,v 1.20 2001/06/19 02:19:38 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "FWObjectDatabase.hh"
#include "Group.hh"

#include "BuiltinDialog.hh"
#include "main_window.hh"
#include "helpers.hh"

#include "Policy.hh"
#include "NAT.hh"

#include "DialogPlugin.hh"

#include <gnome-wrappers/wrappers.hh>

#include <gtk--.h>

BuiltinDialog::BuiltinDialog(string &id)
{
    set_name("BuiltinDialog");

    data_changed=false;
    original=FWObjectDatabase::db->getById( id , true);

    assert(original!=NULL);

    scratch=FWObjectDatabase::db->getScratchCopy(original);
    assert(scratch!=NULL);

    *scratch = *original;
    scratch->setDirty(false,true);

    dialog_body= manage( scratch->CreateDialog() );
    if (dialog_body!=NULL) {

	vbox = manage(new Gtk::VBox(false,0));
	add(*vbox,
	    GTK_SIDE_TOP,
	    GTK_ANCHOR_CENTER,
	    GTK_PACK_EXPAND | GTK_FILL_X | GTK_FILL_Y
	);
	vbox->show();

	save_btn=NULL;
	undo_btn=NULL;
	separator=NULL;
	action_area=NULL;

	if ( dialog_body->needDialogButtons()
	     &&
	     Resources::global_res->getObjResourceBool(original,"write") ) {

	    action_area = manage(new Gtk::HBox(false,5));
	    action_area->set_border_width(10);
	    action_area->set_usize(-1, 50);
	    action_area->set_name("action_area");
	    action_area->set_border_width(10);
	    
	    GtkButton *gtk_button1;

	    gtk_button1=GTK_BUTTON(
		gnome_stock_button(GNOME_STOCK_BUTTON_APPLY));
	    save_btn = manage(new Button(gtk_button1));
/*
	    gtk_button1=GTK_BUTTON(
		gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL));
	    undo_btn = new Button(gtk_button1);
*/
	    Gtk::Button *widget = new Gtk::Button;
	    GtkWidget   *pixm   = gnome_stock_pixmap_widget(
		GTK_WIDGET(widget->gtkobj()),GNOME_STOCK_PIXMAP_UNDO);
	    undo_btn = manage( Gtk::wrap((GtkButton*)gnome_pixmap_button(
					     pixm,  "Undo")));
	    delete widget;
	    
	    action_area->pack_end(*undo_btn,false,false,0);
	    action_area->pack_end(*save_btn,false,false,0);

	    vbox->pack_end(*action_area,false,true,0);

	    save_btn->set_sensitive(false);
	    undo_btn->set_sensitive(false);

	    save_btn->show();
	    undo_btn->show();
	    action_area->show();

	    separator = manage(new Gtk::HSeparator());
	    vbox->pack_end(*separator,false,true,0);
	    separator->show();

	    save_btn->clicked.connect(SigC::slot((BuiltinDialog*)this, 
					 &BuiltinDialog::on_save_clicked));
      
	    undo_btn->clicked.connect(SigC::slot((BuiltinDialog*)this, 
					 &BuiltinDialog::on_undo_clicked));

	}

	vbox->pack_end( *dialog_body,true,true,0);

	dialog_body->show();
	show_all();

	dialog_body->wrk2dlg(); 

	data_changed_flag(false);

/*
 * If name is empty, this is brand new object
 */
	if (original->getTypeName()!=Policy::TYPENAME &&
	    original->getTypeName()!=NAT::TYPENAME ) {

	    if (original->getName()=="") data_changed_flag(true);
	}

	if (undo_btn)	    undo_btn->set_sensitive(false);

// this should be done after we initialize entry fields
	ConnectSignals();     
    }
}


/*
 *   Note:  dialog_body gets destroyed BEFORE this destructor gets called. 
 *          Go figure
 */
BuiltinDialog::~BuiltinDialog()
{
    if (dialog_body!=NULL) {
#ifdef WITH_SCROLLED_WINDOW
	scrolled_win->remove();
#endif

//	delete dialog_body;

#ifdef WITH_SCROLLED_WINDOW
	delete scrolled_win;
#endif
//	if (separator)   delete separator;
//	if (undo_btn)    delete undo_btn;
//	if (save_btn)    delete save_btn;
//	if (action_area) delete action_area;
//	delete vbox;
    }
}

void BuiltinDialog::set_policy(gint allow_shrink, 
			       gint allow_grow, 
			       gint auto_shrink){}

void BuiltinDialog::set_position(GtkWindowPosition position){}

void BuiltinDialog::set_title(const Gtk::nstring& title){}


  
void BuiltinDialog::dialog_set_changed_callback_helper(Gtk::Widget *wp)
{
  Gtk::Editable     *ed;
  Gtk::ToggleButton *tb;

  if (wp==NULL) return;

  if ( Gtk::Editable::isA(wp) ) {
      ed=(Gtk::Editable*)(wp);
      ed->changed.connect(SigC::slot((BuiltinDialog*)this, 
				     &BuiltinDialog::OnDataChanged));
      return;
  }
  if (Gtk::ToggleButton::isA(wp)) {
      tb=(Gtk::ToggleButton*)(wp);
      tb->toggled.connect(SigC::slot((BuiltinDialog*)this, 
				     &BuiltinDialog::OnDataChanged));
      return;
  }
  if (Gtk::Combo::isA(wp)) {
      ((Gtk::Combo*)wp)->get_entry()->changed.connect(
	  SigC::slot((BuiltinDialog*)this, 
		     &BuiltinDialog::OnDataChanged));
      return;
  }
  if ( Gtk::Box::isA(wp) || Gtk::Fixed::isA(wp) ) {
    Gtk::Box      *bptr=(Gtk::Box*)wp;
    Gtk::Box_Helpers::BoxList   bl= bptr->children();
    Gtk::Box_Helpers::BoxList::iterator bl_i;
    Gtk::Box_Helpers::Child    *ch;
    
    for (bl_i=bl.begin(); bl_i!=bl.end(); bl_i++) {
      ch= *bl_i;
      dialog_set_changed_callback_helper( ch->get_widget() );
    }
  }
  if ( Gtk::Packer::isA(wp) ) {
    Gtk::Packer      *bptr=(Gtk::Packer*)wp;
    Gtk::Packer_Helpers::PackerList   bl= bptr->children();
    Gtk::Packer_Helpers::PackerList::iterator bl_i;
    Gtk::Packer_Helpers::Child    *ch;
    
    for (bl_i=bl.begin(); bl_i!=bl.end(); bl_i++) {
      ch= *bl_i;
      dialog_set_changed_callback_helper( ch->get_widget() );
    }
  }
  if ( Gtk::Notebook::isA(wp) ) {
    for (int i=0; i<20; ++i)
      dialog_set_changed_callback_helper
	(((Gtk::Notebook*)wp)->get_nth_page(i) );
    return;
  }
  if ( Gtk::Bin::isA(wp) ) {
    dialog_set_changed_callback_helper( ((Gtk::Bin*)wp)->get_child() );
    return;
  }

/*
 *   TODO:
 *
 *   This is broken in gtk-- 1.2.3  Since this is what is shipped with
 *   RedHat 6.2, we will provide workarounds until we either completely
 *   switch to RedHat 7.0 (for which gtk-- 1.2.5 is available as rpms) or
 *   newer version of gtk-- will be available as rpm for RH 6.2
 *
 *   As a workaround we are going to have to attach "on_changed" callback
 *   to dialog elements which are positioned inside tables, or we won't use
 *   tables where possible
 *

  if ( Gtk::Table::isA(wp) ) {
      Gtk::Table_Helpers::TableList lst= ((Gtk::Table*)wp)->children();
      Gtk::Table_Helpers::TableList::iterator m;
      for (m=lst.begin(); m!=lst.end(); m++) {
	  if ( (*m)==NULL ) continue;
	  Gtk::Widget *w = (*m);
	  dialog_set_changed_callback_helper(w);
      }
  }
*/
}


void BuiltinDialog::ConnectSignals()
{
  dialog_set_changed_callback_helper( this );
}

void BuiltinDialog::data_changed_flag(bool f)
{
    if (save_btn!=NULL) {

	save_btn->set_sensitive(f);
	undo_btn->set_sensitive(f);

	data_changed=f;

/*
	Gdk_Color    btn_color;
	btn_color.parse((f)?"red":"black");
	
	Gtk::Style       *btn_style;

	btn_style= save_btn->get_child()->get_style () -> copy ();
	btn_style->set_fg( GTK_STATE_NORMAL      , btn_color );
	save_btn->get_child()->set_style( *btn_style );
	btn_style->unref();
	save_btn->show();
*/
    }
}

void BuiltinDialog::LoadData()
{
    dialog_body->wrk2dlg(); 
    data_changed_flag(false);
    scratch->setDirty(false,true);
}

void BuiltinDialog::SaveData()
{
    if (data_changed) {
	string old_name = original->getName();
	if (dialog_body!=NULL && dialog_body->dlg2wrk() ) {

	    *original = *scratch;

	    data_changed_flag(false);
	    if (original->getName()!=old_name) {
		ObjectTree *w=(ObjectTree*)find_widget("ObjectTree",this);
		if (w!=NULL)  w->schedule_rebuild_and_show( original->getId());
	    }
	    scratch->setDirty(false,true);
	}
    }
}

void BuiltinDialog::UndoChanges()
{
    if (dialog_body!=NULL) {
	if (data_changed) {
	    *scratch = *original;
	    LoadData();
	    if (original->getTypeName()!=Policy::TYPENAME &&
		original->getTypeName()!=NAT::TYPENAME ) {
		if (original->getName()=="")  data_changed_flag(true);
	    }
	}
    }
}


void BuiltinDialog::OnDataChanged()
{
    data_changed_flag(true);
}

gint BuiltinDialog::on_focus_out_event(GdkEventFocus *ev)
{
  return(0);
}

void BuiltinDialog::OnHide()
{
}

void  BuiltinDialog::on_save_clicked()
{
  SaveData();
}

void  BuiltinDialog::on_undo_clicked()
{
  UndoChanges();
}

Gtk::Menu* BuiltinDialog::BuildCustomMenu()
{
    if (dialog_body!=NULL)
	return( dialog_body->BuildCustomMenu() );
    return(NULL);
}

