/* 

                          Firewall Builder

                 Copyright (C) 2001 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: DiscoveryDruidP40Dialog.cc,v 1.19 2001/07/19 23:39:34 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"
#include "DiscoveryDruidP40Dialog.hh"

#include "FWObjectDatabase.hh"
#include "Host.hh"
#include "Network.hh"
#include "Firewall.hh"
#include "FWOptions.hh"
#include "Policy.hh"
#include "NAT.hh"
#include "ObjectTree.hh"
#include "Group.hh"

#include "TableOfObjectsWithTypeChoice.hh"
#include "FilterDialog.hh"

#include "helpers.hh"

#include "main_window.hh"

#include <string>

DiscoveryDruidP40Dialog::DiscoveryDruidP40Dialog()
{
    p40_txt->set_line_wrap(true);
    tbl=NULL;
}

void DiscoveryDruidP40Dialog::prepare()
{
    scrolledwindow->grab_focus();
//    scrolledwindow->grab_default();
}


void DiscoveryDruidP40Dialog::addTypeChoice(const string& _type)
{
    type_choices.push_back(_type);
}

void DiscoveryDruidP40Dialog::on_filter_clicked()
{
    FilterDialog *fd=new FilterDialog( tbl->getAddressFilter(),
				       tbl->getNameFilter() );
    if ( fd->run() ) { 
	    
	tbl->filter( fd->getAddressFilter() , fd->getNameFilter() );
    }
    delete fd;

    if (tbl->isFilterActive())
	tbl_frame->set_label("New Objects (Filter active):");
    else
	tbl_frame->set_label("New Objects: ");
}

void DiscoveryDruidP40Dialog::on_remove_filter_clicked()
{
    tbl->filter("","");
    tbl_frame->set_label("New Objects: ");
}


void  DiscoveryDruidP40Dialog::fillListOfObjects(const vector<ObjectDescriptor>& ah,
						 const string& domain,
						 bool fqdn)
{
    unsigned  row;

    all_hosts=ah;

    if (tbl!=NULL) {
	scrolledwindow->remove();
	tbl=NULL;
    }

    tbl   = manage( new TableOfObjectsWithTypeChoice( ah.size() ) );

    tbl->changed.connect(SigC::slot(this, 
			   &DiscoveryDruidP40Dialog::on_selection_changed));

    number_of_selected_objects->set_text("Selected  0  objects");

    tbl->addTypeChoices(type_choices);

    for (row=0; row<all_hosts.size(); ++row) {
	ObjectDescriptor od = all_hosts[row];
	IPAddress         a = od.addr;

	set<string> names(od.dns_info.aliases);
	string     n;

	if ( ! od.sysname.empty())         names.insert(od.sysname);
	if ( ! od.dns_info.name.empty())   names.insert(od.dns_info.name);

	if ( ! fqdn ) {

	    set<string>::iterator si, so;
	    for (si=names.begin(); si!=names.end(); ) {

		string::size_type i=(*si).rfind(domain);
		if (i!=string::npos) {
		    n= (*si).substr(0,i-1);
		    so=si;
		    ++si;
		    names.erase(so);
		    names.insert(n);
		    continue;
		}
		++si;
	    }
	}
/* 
 * using address as unique identifier to reference objects in the table widget 
 */
	tbl->addObject( row , a.toString() , names , a.toString() );
    }

    scrolledwindow->add_with_viewport( *tbl );
    tbl->show_all();
}

void DiscoveryDruidP40Dialog::on_selection_changed()
{
    int   n=tbl->getNumOfSelected();
    char  s[32];
    sprintf(s,"Selected  %d  objects",n);

    number_of_selected_objects->set_text(s);
}

void DiscoveryDruidP40Dialog::clearList()
{
    scrolledwindow->remove();
    tbl=NULL;

    type_choices.clear();
}

void DiscoveryDruidP40Dialog::on_select_all_clicked()
{   
    tbl->selectAll();
}

void DiscoveryDruidP40Dialog::on_unselect_all_clicked()
{   
    tbl->unselectAll();
}

void DiscoveryDruidP40Dialog::createObjects()
{
    for (unsigned row=0; row<all_hosts.size(); ++row) {
	string a = all_hosts[row].addr.toString();

	if (tbl->isObjectChecked(a)) {
	    string type=tbl->getTypeChoice(a);
	    string name=tbl->getObjectName(a);
	    all_hosts[row].sysname=name;

	    if (type=="Host") 
		newObject(FWObjectDatabase::db->std.HostsId ,
			  Host::TYPENAME,all_hosts[row]);

	    if (type=="Network") 
		newObject(FWObjectDatabase::db->std.NetworksId ,
			  Network::TYPENAME,all_hosts[row]);

	    if (type=="Firewall") 
		newObject(FWObjectDatabase::db->std.FirewallsId ,
			  Firewall::TYPENAME,all_hosts[row]);

	}
    }
}

void DiscoveryDruidP40Dialog::newObject( const string& where_id , 
					 const string& type, 
					 const ObjectDescriptor& od)
{
    IPAddress a=od.addr;
    string address=a.toString();

    FWObject *o=FWObjectDatabase::create(type);

    o->setName(od.sysname);

    if (type==Host::TYPENAME) {
	o->setStr("address", address);
    }

    if (type==Network::TYPENAME) {
	Network *net=static_cast<Network*>(o);
	net->setAddress(IPAddress(address));
	net->setNetmask(Netmask(IPAddress(address)));
    }

    if (type==Firewall::TYPENAME) {
	o->setStr("address", address);
        o->add( new FirewallOptions() );
        o->add( new Policy() );
        o->add( new NAT() );

	vector<Interface>::const_iterator i;
	for (i=od.interfaces.begin(); i!=od.interfaces.end(); ++i) {
	    Interface *itf=new Interface(*i);
	    o->add(itf);
	}
    }

    FWObject *s=FWObjectDatabase::create(type);


    FWObjectDatabase::db->addAt(where_id,o);
    *s = *o;
    FWObjectDatabase::ScratchPad->addAt(where_id,s);


    ObjectTree *w=(ObjectTree*)find_widget("ObjectTree",main_w);
    if (w!=NULL)  w->schedule_rebuild();

}


