/* 

                          Firewall Builder

                 Copyright (C) 2001 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: DiscoveryDruidP70Dialog.cc,v 1.10 2001/07/19 23:39:34 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"
#include "DiscoveryDruidP70Dialog.hh"

#include "FWObjectDatabase.hh"
#include "Network.hh"
#include "Group.hh"

#include "ObjectTree.hh"
#include "main_window.hh"

#include "FilterDialog.hh"

DiscoveryDruidP70Dialog::DiscoveryDruidP70Dialog()
{
    p70_txt_1->set_line_wrap(true);
    tbl=NULL;
}

void DiscoveryDruidP70Dialog::prepare()
{
    scrolledwindow->grab_focus();
//    scrolledwindow->grab_default();
}

void DiscoveryDruidP70Dialog::on_selection_changed()
{
    int   n=tbl->getNumOfSelected();
    char  s[32];
    sprintf(s,"Selected  %d  objects",n);

    number_of_selected_objects->set_text(s);
}


void DiscoveryDruidP70Dialog::clearList()
{
    scrolledwindow->remove();
    tbl=NULL;

    all_nets.clear();
}

void  DiscoveryDruidP70Dialog::fillListOfNetworks( const vector<IPNetwork>& nets )
{
    int               row;

    clearList();

    tbl   = manage( new TableOfObjects( nets.size() ) );

    tbl->changed.connect(SigC::slot(this, 
			   &DiscoveryDruidP70Dialog::on_selection_changed));

    number_of_selected_objects->set_text("Selected  0  objects");

    vector<IPNetwork>::const_iterator m;
    for (row=0,m=nets.begin(); m!=nets.end(); ++row,++m) {

	all_nets.push_back( (*m) );

	IPAddress  a    = (*m).getAddress();
	Netmask    nm   = (*m).getNetmask();
	string     name = "n-"+a.toString();
	string     addr = a.toString() + "/" + nm.toString();

/* using address as unique ID to reference object in the table widget */
	tbl->addObject(row , addr, name , addr );
    }

    scrolledwindow->add_with_viewport( *tbl );
    tbl->show_all();
}


void DiscoveryDruidP70Dialog::on_select_all_clicked()
{
    tbl->selectAll();
}

void DiscoveryDruidP70Dialog::on_unselect_all_clicked()
{
    tbl->unselectAll();
}

void DiscoveryDruidP70Dialog::on_filter_clicked()
{
    FilterDialog *fd=new FilterDialog( tbl->getAddressFilter(),
				       tbl->getNameFilter() );
    if ( fd->run() ) {
	    
	tbl->filter( fd->getAddressFilter() , fd->getNameFilter() );
    }
    delete fd;

    if (tbl->isFilterActive())
	tbl_frame->set_label("New Objects (Filter active):");
    else
	tbl_frame->set_label("New Objects: ");
}

void DiscoveryDruidP70Dialog::on_remove_filter_clicked()
{
    tbl->filter("","");
    tbl_frame->set_label("New Objects: ");
}

void DiscoveryDruidP70Dialog::createObjects()
{
    for (unsigned row=0; row<all_nets.size(); ++row) {

	IPAddress  a    = all_nets[row].getAddress();
	Netmask    nm   = all_nets[row].getNetmask();
	string     addr = a.toString() + "/" + nm.toString();

	if (tbl->isObjectChecked(addr)) 
	    newNet( tbl->getObjectName(addr) , all_nets[row] );
    }
}

void DiscoveryDruidP70Dialog::newNet(string name, IPNetwork& net )
{
    FWObject *o=FWObjectDatabase::create(Network::TYPENAME);

    o->setName(name);
    ((Network*)o)->setData(net);

    FWObjectDatabase::db->addAt(FWObjectDatabase::db->std.NetworksId, o);

    FWObject *s=FWObjectDatabase::create(Network::TYPENAME);
    *s = *o;
    FWObjectDatabase::ScratchPad->addAt(FWObjectDatabase::db->std.NetworksId,
					s);

    ObjectTree *w=(ObjectTree*)find_widget("ObjectTree",main_w);
    if (w!=NULL)  w->schedule_rebuild();
}

