/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: FWObjectDatabase.hh,v 1.25 2001/04/27 01:46:27 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef _FWOBJECTDATABASE_HH
#define _FWOBJECTDATABASE_HH

#include "FWObject.hh"

#include "FWException.hh"

class Network   ;
class IPService ;
class Interval  ;
class Group     ;

/**
 * Database of objects.
 */
class FWObjectDatabase : public FWObject
{
    private:

    static long IDcounter;

    protected:

    string  data_file;

//    virtual void resolveReferences();

    public:

    static const string DB_TEMPLATE_FILE_NAME ;
    static const string DTD_FILE_NAME ;
    

    FWObjectDatabase();
    FWObjectDatabase(const string &filename) throw(FWException);

    virtual ~FWObjectDatabase();

    DECLARE_FWOBJECT_SUBTYPE(FWObjectDatabase);

    // --- XML import/export ---

    virtual xmlNodePtr toXML(xmlNodePtr parent);

    /**
     *   this is the main "Create" method:
     *   it creates instance of FWObject of given type
     */
    static FWObject *create(const string &type);

    /**
     * Creates instance of FWObject using its XML representation
     */
    static FWObject *createFromXML( xmlNodePtr data);

    static string generateUniqueId();

    // --- Load/Save ---

    void save           (                       ) throw(FWException);
    void saveAs         (                       ) throw(FWException);
    void saveFile       ( const string &filename) throw(FWException); 
    void saveXML        ( xmlDocPtr             ) throw(FWException); 
    bool saveIfModified (                       ) throw(FWException);
    void load           ( const string &filename) throw(FWException);

    void setFileName (const string &filename);
    const string& getFileName ();
    const string  getFileDir ();

    void      copyToScratchPad();
    FWObject* getScratchCopy(FWObject *obj);

    /*
     * verify whether given object type is approppriate as a child
     */
    virtual bool  validateChild(FWObject *o);

    // --- Standard objects ---

    struct StandardObjects
    {
        string     AnyNetworkId;
        string     AnyIPServiceId;
        string     AnyIntervalId;
        
        string     ObjectsId;
        string     HostsId;      
        string     NetworksId;    
        string     ObjectGroupsId;
        
        string     ServicesId;
        string     IPServicesId;
        string     ICMPServicesId;
        string     UDPServicesId;
        string     TCPServicesId;
        string     ServiceGroupsId;
        
        string     TimeId;
        
        string     FirewallsId;
    };
    
    struct StandardObjects std;

    static Group     *ScratchPad;
    
    // singelton
    
    static FWObjectDatabase  *db;
};


#endif











