/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: Firewall.cc,v 1.15 2001/07/07 07:19:15 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "FWObject.hh"
#include "FWObjectReference.hh"
#include "FWOptions.hh"

#include "Firewall.hh"
#include "helpers.hh"
#include "Policy.hh"
#include "NAT.hh"
#include "FirewallDialog.hh"

#include "XMLTools.hh"
#include "main_window.hh"


const char *Firewall::TYPENAME={"Firewall"};


Firewall::Firewall()
{
    setStr("platform","");
    setStr("version","");
    setStr("snmp_read_community", "public");
    setStr("snmp_write_community", "");
}


Firewall::~Firewall()  {}

void Firewall::fromXML(xmlNodePtr root)
{
    const char *n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("platform")));
    assert(n!=NULL);
    setStr("platform", n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("version")));
    if (n!=NULL) setStr("version", n);

    Host::fromXML(root);
}

xmlNodePtr Firewall::toXML(xmlNodePtr parent)
{
    xmlNodePtr me = xmlNewChild(parent, NULL, 
				STRTOXMLCAST(getTypeName()), 
				NULL);

    FWObject *o;
    
    map<string, string>::iterator i;
    for(i=data.begin(); i!=data.end(); ++i) 
    {
        string name  = (*i).first;
        string value = (*i).second;

        xmlAttrPtr pr = xmlNewProp(me, 
                                   STRTOXMLCAST(name) , 
                                   STRTOXMLCAST(value));
        if(name=="id")
        {
            xmlAddID(NULL, parent->doc, STRTOXMLCAST(value), pr);
        } else if(name=="ref") 
        {
            xmlAddRef(NULL, parent->doc, STRTOXMLCAST(value), pr);
        }
    }
    
    o=getFirstByType( NAT::TYPENAME );
    if (o) o->toXML(me);

    o=getFirstByType( Policy::TYPENAME );
    if (o) o->toXML(me);

    vector<FWObject*> ol;
    vector<FWObject*>::iterator j;

    ol=getByType( Interface::TYPENAME );
    for(j=ol.begin(); j!=ol.end(); ++j) 
	if ( (o=(*j))!=NULL ) o->toXML(me);

    o=getFirstByType( FirewallOptions::TYPENAME );
    if (o) o->toXML(me);

    return me;
}

FirewallOptions* Firewall::getOptionsObject()
{
    return FirewallOptions::cast( getFirstByType(FirewallOptions::TYPENAME) );
}

DialogPlugin* Firewall::CreateDialog()
{
    return( new FirewallDialog(this) );
}

/*
 *  enable and disable menu items which make sence only for this object
 */
void   Firewall::updateMainMenu()
{
    Gtk::MenuBar *main_menubar=main_w->getMainMenuBar();
    Gtk::Widget *itm;

    FWObject::updateMainMenu();

    itm=find_widget("policy",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("insert_top",main_menubar);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("append_bottom",main_menubar);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("add_after",main_menubar);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("remove_current",main_menubar);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("move_up",main_menubar);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("move_down",main_menubar);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("rules_druid",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("compile_policy",main_menubar);
    if (itm) itm->set_sensitive(true);

    FirewallOptions *fwopt = getOptionsObject();
		
    string inst_script=fwopt->getStr("inst_script");
    itm=find_widget("install_policy",main_menubar);
    if (itm) itm->set_sensitive( inst_script!="" );

}


Policy* Firewall::getPolicy() 
{ 
    return(Policy::cast(getFirstByType(Policy::TYPENAME)));
}

NAT* Firewall::getNAT()    
{ 
    return(NAT::cast(getFirstByType(NAT::TYPENAME)));
}

void Firewall::setDefaults()
{
    setStr("version","");

    if (getStr("platform")=="ipchains") {
//	setStr("tcp_timeout",    "7200");
//	setStr("tcpfin_timeout", "10");
//	setStr("udp_timeout",    "180");
//	setStr("pkt_fwd",       "1");
//	setStr("dyn_addr",      "0");
    }

    if (getStr("platform")=="cisco_acl") {

    }
}


bool  Firewall::validateChild(FWObject *o)
{ 
    string otype=o->getTypeName();
    return (FWObject::validateChild(o) && 
	    (otype==Interface::TYPENAME  ||
	     otype==Policy::TYPENAME     ||
	     otype==NAT::TYPENAME        ||
	     otype==FirewallOptions::TYPENAME ));
}

