/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: FirewallDialog.cc,v 1.75 2001/07/25 01:30:32 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "Firewall.hh"
#include "FWOptions.hh"
#include "FirewallDialog.hh"
#include "InterfaceDialog.hh"
#include "ObjectTree.hh"

#include "GenericBackgroundOpDialog.hh"
#include "MessageDialog.hh"

#include "Policy.hh"
#include "InterfacePolicy.hh"
#include "NAT.hh"

#include "PolicyDialog.hh"

#include "FWObjectDatabase.hh"

#include "IPAddressWidget.hh"

#include "helpers.hh"
#include "snmp.hh"
#include "dns.hh"

#include "FileSel.hh"
#include "Preferences.hh"

#include <gtk--.h>
#include <gdk_imlib.h>

#define GENERAL_NOTEBOOK_PAGE    0
#define INTERFACES_NOTEBOOK_PAGE 1
#define SNMP_NOTEBOOK_PAGE       2
#define COMPILER_NOTEBOOK_PAGE   3
#define IPCHAINS_NOTEBOOK_PAGE   4
#define IPFILTER_NOTEBOOK_PAGE   5
#define IPTABLES_NOTEBOOK_PAGE   6
#define CISCO_NOTEBOOK_PAGE      7

#ifdef HAVE_LIBSNMP

class SNMPSysdescQueryPool: public Pool<SNMP_sysdesc_query>
{
    private:
    
    static SNMPSysdescQueryPool *instance;
    static Mutex                 sysdesc_mutex;
    
    public:
    
    SNMPSysdescQueryPool(size_t initial, size_t max_):Pool<SNMP_sysdesc_query>(initial, max_) {}
    
    static SNMPSysdescQueryPool *getInstance()
    {
        sysdesc_mutex.lock();
        if(instance==NULL)
            instance=new SNMPSysdescQueryPool(0, 32);
        sysdesc_mutex.unlock();
        return instance;
    }
};
 
SNMPSysdescQueryPool *SNMPSysdescQueryPool::instance=NULL;
Mutex SNMPSysdescQueryPool::sysdesc_mutex;

class SNMPInterfaceQueryPool: public Pool<SNMP_interface_query>
{
    private:
    
    static SNMPInterfaceQueryPool *instance;
    static Mutex                   interface_mutex;
    
    public:
    
    SNMPInterfaceQueryPool(size_t initial, size_t max_):Pool<SNMP_interface_query>(initial, max_) {}
    
    static SNMPInterfaceQueryPool *getInstance()
    {
        interface_mutex.lock();
        if(instance==NULL)
            instance=new SNMPInterfaceQueryPool(0, 32);
        interface_mutex.unlock();
        return instance;
    }
};
 
SNMPInterfaceQueryPool *SNMPInterfaceQueryPool::instance=NULL;
Mutex SNMPInterfaceQueryPool::interface_mutex;

#endif


FirewallDialog::FirewallDialog(FWObject *obj) 
{ 
    set_name("firewall_dialog");
     
    interface_policy_list=NULL;

    object=obj;

    options=NULL;

    fw_addr=manage(new IPAddressWidget());
    table16->attach(*fw_addr, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
    fw_addr->changed.connect(SigC::slot(this,&FirewallDialog::on_changed));

    if_list->column(0).set_justification(GTK_JUSTIFY_LEFT);
    if_list->column(1).set_justification(GTK_JUSTIFY_LEFT);
    if_list->column(2).set_justification(GTK_JUSTIFY_LEFT);
    if_list->column(3).set_justification(GTK_JUSTIFY_LEFT);
/*
    if_list->column(1).set_width(90);
    if_list->column(2).set_width(90);
    if_list->column(3).set_width(20);
    if_list->column(0).set_auto_resize(true);
*/

    ((Gtk::Label*)(iptables_firewall_is_in_any->get_child()))->
	set_alignment( 0.0 , 0.0 );
//    ((Gtk::Label*)(iptables_firewall_is_in_any->get_child()))->
//	set_justify(GTK_JUSTIFY_LEFT);


    GdkPixmap *pmap;
    GdkBitmap *mask;

    if ( gdk_imlib_load_file_to_pixmap(
	(char*)(object->getXPMIconFileName("_dlg").c_str()),&pmap, &mask) ) {

	fw_icon->set(pmap,mask);
	gdk_imlib_free_pixmap(pmap);
    }

    ((Gtk::Label*)(ipchains_load_modules->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipchains_debug->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipchains_pkt_fwd->get_child()))->set_alignment(0.0,0.5);

    ((Gtk::Label*)(iptables_log_tcp_seq->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_log_tcp_opt->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_log_ip_opt->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_pkt_fwd->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_proxyarp->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_debug->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_load_modules->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_no_optimisation->get_child()))->set_alignment(0.0,0.5);

    ((Gtk::Label*)(iptables_firewall_is_in_any->get_child()))->set_alignment(0.0,0.0);
    ((Gtk::Label*)(iptables_manage_arp->get_child()))->set_alignment(0.0,0.5);


    action_on_reject_lbl->set_alignment(0.0,0.5);
    action_on_reject_lbl->set_line_wrap(true);

    log_levels.push_back("debug");
    log_levels.push_back("info"   );
    log_levels.push_back("notice" );
    log_levels.push_back("warning" );
    log_levels.push_back("error"  );
    log_levels.push_back("crit"  );
    log_levels.push_back("alert"  );

    limit_suffixes.push_back("/second");
    limit_suffixes.push_back("/minute");
    limit_suffixes.push_back("/hour");
    limit_suffixes.push_back("/day");

    actions_on_reject.push_back("ICMP net unreachable");
    actions_on_reject.push_back("ICMP host unreachable");
    actions_on_reject.push_back("ICMP port unreachable");
    actions_on_reject.push_back("ICMP proto unreachable");
    actions_on_reject.push_back("ICMP net prohibited");
    actions_on_reject.push_back("ICMP host prohibited");
    actions_on_reject.push_back("TCP RST");

    platforms=Resources::global_res->getPlatforms();
    platforms.insert( platforms.begin() , "Undefined" );
}

FirewallDialog::~FirewallDialog() 
{ 
}


/**
 *   this callback is called when user changes firewall platform
 */
void FirewallDialog::on_fw_platform_changed()
{
    firewall_dialog_notebook->get_nth_page(IPCHAINS_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(IPFILTER_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(IPTABLES_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(CISCO_NOTEBOOK_PAGE)->hide();

    string *new_platform=(string*)(fw_platform->get_menu()->get_active()->get_user_data());
    assert(new_platform!=NULL);

    if ( *new_platform=="ipchains") {
	firewall_dialog_notebook->get_nth_page(IPCHAINS_NOTEBOOK_PAGE)->show();
    }
    if ( *new_platform=="ipfilter") {
	firewall_dialog_notebook->get_nth_page(IPFILTER_NOTEBOOK_PAGE)->show();
    }
    if ( *new_platform=="iptables") {
	firewall_dialog_notebook->get_nth_page(IPTABLES_NOTEBOOK_PAGE)->show();
/* 
 *  assign default values to firewall options if this is the first time
 */
	if ( ! options->exists("firewall_is_part_of_any_and_networks") ) {
	    iptables_firewall_is_in_any->set_active( true );
	}
    }
    if ( *new_platform=="cisco") {
	firewall_dialog_notebook->get_nth_page(CISCO_NOTEBOOK_PAGE)->show();
    }

    on_changed();
}

void FirewallDialog::on_ipfilter_log_level_changed()
{
    string *nl=(string*)(ipfilter_log_level->get_menu()->get_active()->get_user_data());
    string  ol=options->getStr("log_level" );
    if (*nl!=ol) on_changed();
}

void FirewallDialog::on_iptables_log_level_changed()
{
    string *nl=(string*)(iptables_log_level->get_menu()->get_active()->get_user_data());
    string  ol=options->getStr("log_level" );
    if (*nl!=ol) on_changed();
}

void FirewallDialog::on_iptables_limit_suffix_changed()
{
    string *nl=(string*)(iptables_limit_suffix->get_menu()->get_active()->get_user_data());
    string  ol=options->getStr("log_limit_suffix" );
    if (*nl!=ol) on_changed();
}

void FirewallDialog::on_iptables_action_on_reject_changed()
{
    string *naor=(string*)(iptables_action_on_reject->get_menu()->get_active()->get_user_data());
    string oaor=options->getStr("action_on_reject" );
    if (*naor!=oaor) on_changed();
}

void FirewallDialog::on_switch_page(Gtk::Notebook_Helpers::Page* p0,guint p1)
{
// do nothing if dialog is not visible
// it turns out, "switch_page" signal gets invoked when notebook pages are
// destroyed

    if (! is_visible() ) return;

    if (p1==INTERFACES_NOTEBOOK_PAGE ) {

	InterfacePolicy *ipol;
	Interface *intf=
	    Interface::cast( object->getFirstByType( Interface::TYPENAME ) );

	if (intf) {
	    ipol=InterfacePolicy::cast(
		intf->getFirstByType( InterfacePolicy::TYPENAME ) );

	    assert (ipol!=NULL);

	    ipol->updateMainMenu();
	}
    }else
	object->updateMainMenu();
}

void FirewallDialog::showInterfaces()
{
    if_list->freeze();
    if_list->rows().clear();

    vector<FWObject*>::iterator m;
    gint row;
    char *crow[4]={"","","",""};
    Interface       *intf;


    for (row=0,m=((Host*)object)->begin(); 
	 m!=((Host*)object)->end(); 
	 ++m) {

	if(Interface::isA((*m))) 
        {
	    intf=Interface::cast(*m);
            
	    if_list->append( (const char**)crow );

	    if_list->cell(row,0).set_spacing(10);
	    if_list->cell(row,0).set_text( intf->getName() );

	    if_list->cell(row,1).set_spacing(10);
	    if_list->cell(row,1).set_text( intf->getStr("address"));

	    if_list->cell(row,2).set_spacing(10);
	    if_list->cell(row,2).set_text( intf->getStr("netmask"));

	    if (if_list->row(0).size()>3) 
            {
		if_list->cell(row,3).set_spacing(10);
		if_list->cell(row,3).set_text( ( intf->isExt())?"ext":"");
	    }

            // We return pointer to actuall string residing
            // within Interface object holding it's ID.
            // If interface object is deleted, this string will
            // become invalid.
            // We assume here that this would not happen, while
            // this dialogue is running.
            // (lord)
            if_list->rows()[row].set_data((void*)&intf->getId());

	    ++row;
	}
    }
    if_list->thaw();

    intf=Interface::cast( object->getFirstByType( Interface::TYPENAME ) );

    if (intf)
	showInterfacePolicy( intf );
}

void FirewallDialog::showInterfacePolicy( Interface* intf )
{
    InterfacePolicy *ipol;

    if (intf) {
	ipol=InterfacePolicy::cast(
	    intf->getFirstByType( InterfacePolicy::TYPENAME ) );
	if (ipol==NULL) {
	    ipol=new InterfacePolicy();
	    intf->add(ipol);
	}

//	DialogPlugin *dlg= manage( ipol->CreateDialog() );

	((Gtk::Bin*)interface_policy_vprt)->remove();

	interface_policy_list=manage( new PolicyList( ipol ) );
	interface_policy_vprt->add( *interface_policy_list );
	interface_policy_list->Build();
	interface_policy_list->show_all();

	interface_policy_frame->set_label( 
	    string("Policy attached to interface ")+intf->getName() );

	ipol->updateMainMenu();
    }
}

/*
 *  Regarding FirewallOptions *options
 *
 *  Unfortunately we have to find this objects every time we load data
 *  into dialog because it may change. In particular, it changes when
 *  user clicks "Undo". This happens because dialog actually works
 *  with a copy of original object in ScratchPad. When user clicks
 *  Undu this copy is overriden with data from original; when this
 *  happens, all child objects are recreated.  (vk)
 */

/**
 *  This method is called to load data from the object into dialog
 *  This happens at the very beginning and when user clicks "Undo"
 *  (vk)
 */
void FirewallDialog::wrk2dlg()
{
    options=(Firewall::cast(object))->getOptionsObject();
    assert(options!=NULL);

//    object->dump();

    fw_name->set_text(object->getName());
    fw_addr->setAddress(object->getStr("address"));
    dyn_addr->set_active( options->getBool("dyn_addr") );

    fw_name->grab_focus();

    comment->set_point(0);
    comment->forward_delete( comment->get_length() );
    comment->insert(object->getComment());

#ifndef HAVE_LIBSNMP
    button4->set_sensitive(false);
    button16->set_sensitive(false);
#endif

    snmp_r_community->set_text( object->getStr("snmp_read_community") );
    snmp_w_community->set_text( object->getStr("snmp_write_community") );

    snmp_description->set_point(0);
    snmp_description->forward_delete( snmp_description->get_length() );
    snmp_description->insert( options->getStr("snmp_description"));


    snmp_location->set_text( options->getStr("snmp_location"));
    snmp_contact->set_text( options->getStr("snmp_contact"));

//    fw_icon->set( object->getXPMIconFileName("_dlg") );

    fw_version->set_text(object->getStr("version"));

    string         pl=object->getStr("platform");

    fw_platform->remove_menu();
    fw_platform->set_menu( createOptionsMenu(platforms,pl,
		  SigC::slot(this,&FirewallDialog::on_fw_platform_changed)) );
    fw_platform->show();




/********************************** Interfaces tab *********************/

    showInterfaces();


/********************************** Platform-specific tabs *********/

    firewall_dialog_notebook->get_nth_page(IPCHAINS_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(IPFILTER_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(IPTABLES_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(CISCO_NOTEBOOK_PAGE)->hide();

/********************/

    ipchains_tcp_timeout->set_value( options->getInt("tcp_timeout") );
    ipchains_tcpfin_timeout->set_value(options->getInt("tcpfin_timeout"));
    ipchains_udp_timeout->set_value( options->getInt("udp_timeout") );

    ipchains_pkt_fwd->set_active( options->getBool("pkt_fwd") );
    ipchains_debug->set_active( options->getBool("debug") );
    ipchains_load_modules->set_active( options->getBool("load_modules") );

/*********************/

//    ipfilter_log_level->get_entry()->set_text(options->getStr("log_level"));
    string ipf_ll=options->getStr("log_level");
    ipfilter_log_level->remove_menu();
    ipfilter_log_level->set_menu( createOptionsMenu(log_levels , ipf_ll,
 SigC::slot(this,&FirewallDialog::on_ipfilter_log_level_changed)) );

    ipfilter_log_or_block->set_active( options->getBool("log_or_block") );
    ipfilter_log_body->set_active( options->getBool("log_packet_body") );

/********************/

    iptables_log_tcp_seq->set_active( options->getBool("log_tcp_seq") );
    iptables_log_tcp_opt->set_active( options->getBool("log_tcp_opt") );
    iptables_log_ip_opt->set_active( options->getBool("log_ip_opt") );


    string ipt_ll=options->getStr("log_level");
    iptables_log_level->remove_menu();
    iptables_log_level->set_menu( createOptionsMenu(log_levels , ipt_ll, 
 SigC::slot(this,&FirewallDialog::on_iptables_log_level_changed)) );

    iptables_log_prefix->set_text( options->getStr("log_prefix") );

    string ls=options->getStr("log_limit_suffix");
    iptables_limit_suffix->remove_menu();
    iptables_limit_suffix->set_menu( createOptionsMenu(limit_suffixes,ls,
 SigC::slot(this,&FirewallDialog::on_iptables_limit_suffix_changed)) );


    iptables_limit_value->set_value( options->getInt("log_limit_value") );

    iptables_pkt_fwd->set_active( options->getBool("pkt_fwd") );
    iptables_proxyarp->set_active( options->getBool("proxy_arp") );
    iptables_debug->set_active( options->getBool("debug") );
    iptables_load_modules->set_active( options->getBool("load_modules") );

    string aor=options->getStr("action_on_reject");
    iptables_action_on_reject->remove_menu();
    iptables_action_on_reject->set_menu(createOptionsMenu(actions_on_reject,aor,
 SigC::slot(this,&FirewallDialog::on_iptables_action_on_reject_changed)) );


    iptables_firewall_is_in_any->set_active( options->getBool("firewall_is_part_of_any_and_networks") );
    iptables_no_optimisation->set_active( options->getBool("no_optimisation") );
    iptables_manage_arp->set_active( options->getBool("manage_arp") );

    if ( string(object->getStr("platform"))=="ipchains") {
	firewall_dialog_notebook->get_nth_page(IPCHAINS_NOTEBOOK_PAGE)->show();
    }

    if ( string(object->getStr("platform"))=="ipfilter") {
	firewall_dialog_notebook->get_nth_page(IPFILTER_NOTEBOOK_PAGE)->show();
    }

    if ( string(object->getStr("platform"))=="iptables") {
	firewall_dialog_notebook->get_nth_page(IPTABLES_NOTEBOOK_PAGE)->show();
    }

    if ( string(object->getStr("platform"))=="cisco_acl") {
	firewall_dialog_notebook->get_nth_page(CISCO_NOTEBOOK_PAGE)->show();
    }



/* Compiler tab */

    compiler->set_text( options->getStr("compiler") );
    cmdline->set_text(  options->getStr("cmdline") );
	
    inst_script->set_text( options->getStr("inst_script") );
    inst_cmdline->set_text( options->getStr("inst_cmdline") );
}


/**
 *    This method is called to copy dialog data into the object (when
 *    user clicks "Apply" ) (vk)
 */
bool FirewallDialog::dlg2wrk()
{
/*
 *  check if name is valid
 */
    if ( ! checkObjectName(fw_name->get_text()) ) return(false);
    if ( ! dyn_addr->get_active() &&
	 ! checkIPaddress(fw_addr->getAsString()) ) return(false);


    Firewall *fo=Firewall::cast(object);

    string old_name=object->getName();

    object->setName( fw_name->get_text() );

    object->setComment( comment->get_chars(0,comment->get_length()) );

    object->setStr( "address"   , fw_addr->getAsString() );
    options->setBool("dyn_addr" , dyn_addr->get_active() );
    
    object->setStr( "snmp_read_community" , 
		      snmp_r_community->get_text() );

    object->setStr( "snmp_write_community" , 
		      snmp_w_community->get_text() );

    options->setStr( "snmp_description" , 
	      snmp_description->get_chars(0,snmp_description->get_length()) );
    options->setStr( "snmp_location" ,    snmp_location->get_text() );
    options->setStr( "snmp_contact"     , snmp_contact->get_text() );


    object->setStr( "version"  , fw_version->get_text() );

    string *new_platform=(string*)(fw_platform->get_menu()->get_active()->get_user_data());

    if (fw_platform->is_visible() ) {  // platform has been defined
	object->setStr( "platform"   , *new_platform );
	options->setStr("platform"   , *new_platform);

	fo->setDefaults();
    }

/*
    options->setBool("block_fragments", rule_fragments->get_active() );
*/

    if (string(object->getStr("platform"))=="ipchains") {

	options->setInt("tcp_timeout"   , ipchains_tcp_timeout->get_value_as_int() );
	options->setInt("tcpfin_timeout", ipchains_tcpfin_timeout->get_value_as_int());
	options->setInt("udp_timeout"   , ipchains_udp_timeout->get_value_as_int() );

	options->setBool("pkt_fwd",      ipchains_pkt_fwd->get_active() );
	options->setBool("debug"   ,     ipchains_debug->get_active() );
	options->setBool("load_modules", ipchains_load_modules->get_active());
    }

    if (string(object->getStr("platform"))=="ipfilter") {
	string *ll=(string*)(ipfilter_log_level->get_menu()->get_active()->get_user_data());
	options->setStr("log_level"    , *ll );

	options->setBool("log_or_block", ipfilter_log_or_block->get_active() );
	options->setBool("log_packet_body", ipfilter_log_body->get_active() );
    }

    if (string(object->getStr("platform"))=="iptables") {
	string *ll=(string*)(iptables_log_level->get_menu()->get_active()->get_user_data());
	options->setStr("log_level"    , *ll );

	options->setStr("log_prefix"   , iptables_log_prefix->get_text() );
	options->setInt("log_limit_value",
			 iptables_limit_value->get_value_as_int() );

	string *ls=(string*)(iptables_limit_suffix->get_menu()->get_active()->get_user_data());
	options->setStr("log_limit_suffix"  , *ls );


	options->setBool("log_tcp_seq", iptables_log_tcp_seq->get_active() );
	options->setBool("log_tcp_opt", iptables_log_tcp_opt->get_active() );
	options->setBool("log_ip_opt",  iptables_log_ip_opt->get_active() );

	options->setBool("pkt_fwd",      iptables_pkt_fwd->get_active() );
	options->setBool("proxy_arp",    iptables_proxyarp->get_active() );
	options->setBool("debug"   ,     iptables_debug->get_active() );
	options->setBool("load_modules", iptables_load_modules->get_active());

	string *aor=(string*)(iptables_action_on_reject->get_menu()->get_active()->get_user_data());
	options->setStr("action_on_reject"  , *aor );

	options->setBool("manage_arp",    iptables_manage_arp->get_active() );


	options->setBool("firewall_is_part_of_any_and_networks", 
			 iptables_firewall_is_in_any->get_active());
	options->setBool("no_optimisation", 
			 iptables_no_optimisation->get_active());

    }

    options->setStr("cmdline",  cmdline->get_text() );
    options->setStr("compiler", compiler->get_text() );
    options->setStr("inst_script", inst_script->get_text() );
    options->setStr("inst_cmdline", inst_cmdline->get_text() );

    return(true);
}

void FirewallDialog::on_changed()
{
    data_changed_flag(true);
}

void FirewallDialog::on_new_if_clicked()
{ 
    bool res;

    Interface *intf=new Interface();

    InterfaceDialog *id = new InterfaceDialog(intf);
    res=id->run();
    delete id;
    if (res) {
	if ( (Firewall::cast(object))->Appropriate(intf) ) 
	    object->add(intf);

	intf->add( new InterfacePolicy() );

	showInterfaces();
	data_changed_flag(true);
    } else
	delete intf;
}

void FirewallDialog::on_del_if_clicked()
{   
    Interface *intf=getSelectedInterface();
    if (intf) {
	object->remove(intf);
	showInterfaces();
	data_changed_flag(true);
    }
}


void FirewallDialog::on_edit_clicked()
{   
    bool res;

    Interface *intf=getSelectedInterface();
    if(intf) 
    {
	InterfaceDialog *id = new InterfaceDialog( intf );
	res=id->run();
	delete id;
	if(res) 
        {
	    showInterfaces();
	    data_changed_flag(true);
	}
    }
}

void FirewallDialog::on_snmp_get_descr_released()
{
#ifdef HAVE_LIBSNMP

    string addr;
    string rcomm=snmp_r_community->get_text();

    if ( dyn_addr->get_active() ) {
	addr=fw_name->get_text();
    } else {
	if ( ! checkIPaddress(fw_addr->getAsString()) ) return;
	addr=fw_addr->getAsString();
    }


    if(rcomm.empty()) 
    {
	MessageDialog::Error("No SNMP community string defined");
	return;
    }

    int t=Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPTimeout", -1);
    SNMP_sysdesc_query *q=SNMPSysdescQueryPool::getInstance()->lease();
    q->init(addr , rcomm,
	    Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPRetries",  SNMP_DEFAULT_RETRIES),
	    t==-1?SNMP_DEFAULT_TIMEOUT:(1000000L*t)
    );


    GenericBackgroundOpDialog bdisp(q);
    
    try
    {
        bdisp.execute();
        snmp_description->set_point(0);
        snmp_description->forward_delete( snmp_description->get_length() );
        snmp_description->insert( q->getDescr());
        
        snmp_location->set_text(q->getLocation());
        snmp_contact->set_text(q->getContact());

        data_changed_flag(true);
    } catch(const FWException &ex)
    {
        //do nothing
    }
#endif
}

void FirewallDialog::on_snmp_get_clicked()
{
}

void FirewallDialog::on_snmp_get_released()
{   
#ifdef HAVE_LIBSNMP
    string addr;
    string rcomm=snmp_r_community->get_text();

    if ( dyn_addr->get_active() ) {
	addr=fw_name->get_text();
    } else {
	if ( ! checkIPaddress(fw_addr->getAsString()) ) return;
	addr=fw_addr->getAsString();
    }

    if ( rcomm.empty() ) {
	MessageDialog::Error("No SNMP community string defined");
	return;
    }

    int t=Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPTimeout", -1);
    SNMP_interface_query *q=SNMPInterfaceQueryPool::getInstance()->lease();
    q->init(addr,  rcomm,
	    Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPRetries",  SNMP_DEFAULT_RETRIES),
	    t==-1?SNMP_DEFAULT_TIMEOUT:(1000000L*t)
    );

    GenericBackgroundOpDialog bdisp(q);
    
    try
    {
        bdisp.execute();
        
        vector<Interface> intf = q->getInterfaces();
        for(vector<Interface>::iterator i=intf.begin();i!=intf.end(); ++i)
        {
            Interface *ifs=new Interface(*i);
            if((Firewall::cast(object))->Appropriate(ifs))
            {
                ifs->add(new InterfacePolicy());
                object->add( ifs );
            }
        }
        showInterfaces();
        data_changed_flag(true);
    } catch(const FWException &ex)
    {
        //do nothing
    }
#endif
}

void FirewallDialog::on_find_compiler_clicked()
{   
    FileSel *fs=new FileSel("Find compiler",
                            Preferences::global_prefs->getWdir(),
                            "");
    string cmp=fs->run();
    delete fs;
    if(cmp!="")
	compiler->set_text( cmp );
}

void FirewallDialog::on_find_install_script_clicked()
{   
    FileSel *fs=new FileSel("Find install script",
                            Preferences::global_prefs->getWdir(),
                            "");
    string ins=fs->run();
    delete fs;
    if(ins!="")
	inst_script->set_text( ins );
}

void FirewallDialog::on_if_list_select_row(gint row, 
					   gint column, 
					   GdkEvent *event)
{   
    Interface *intf;

    const string*  id= (const string*)( if_list->get_row_data( row ) );
    if(id) 
    {
	intf = Interface::cast( object->getById( *id ) );
	if(intf) 
            showInterfacePolicy( intf );
    }
}

void FirewallDialog::on_add_rule_clicked()
{   
    interface_policy_list->appendRuleAtBottom();
    return;

    Interface *intf=getSelectedInterface();
    if (intf) {
	InterfacePolicy *ip=InterfacePolicy::cast(
	    intf->getFirstByType( InterfacePolicy::TYPENAME ) );
	if (ip) {
	    ip->appendRuleAtBottom();
	    showInterfacePolicy( intf );
	}
    }
}

void FirewallDialog::on_del_rule_clicked()
{   
}

Interface* FirewallDialog::getInterfaceByRow(int row)
{
    Interface *intf=NULL;
    const string* id = (const string*)(if_list->get_row_data(row));
    if (id)
	intf = Interface::cast( object->getById( *id ) );

    return intf;
}

Interface* FirewallDialog::getSelectedInterface()
{
    Gtk::CList_Helpers::SelectionList sl = if_list->selection();
    if (sl.empty()) return NULL;

    Gtk::CList_Helpers::Row row = sl.front();
    int   rn = row.get_row_num();

    return getInterfaceByRow(rn);
}

void FirewallDialog::on_dyn_addr_toggled()
{   
    fw_addr->set_sensitive( ! dyn_addr->get_active() );
}

void FirewallDialog::on_dns_lookup()
{
    vector<IPAddress> addr;

    try {
	addr=DNS::getHostByName( fw_name->get_text() );
    } catch (FWException &e)
    {
	MessageDialog::Error( e.toString() );
	return;
    }

    fw_addr->setAddress( addr.front() );
    data_changed_flag(true);
}



