/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: GroupDialog.cc,v 1.27 2001/07/12 14:45:28 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "Tools.hh"
#include "FWObject.hh"
#include "Group.hh"
#include "FWObjectDatabase.hh"
#include "GroupDialog.hh"
#include "MessageDialog.hh"
#include "helpers.hh"

#include "main_window.hh"

#include "gen_popup_menu.hh"


#include <gtk--.h>
#include <gdk_imlib.h>

GroupDialog::GroupDialog(FWObject *obj)
{
    GroupIconList=NULL;
    object=obj;

    g_assert( obj!=NULL );

    int  xstep, ystep;

    if (Preferences::global_prefs->getOptBool(
	"/FWBuilderPreferences/UI/LargeIconsInGroups")) {
	xstep=130;
	ystep=100;
    } else {
	xstep=100;
	ystep=75;
    }

    GroupIconList=manage(new  ListOfIcons(80,true));

    g_assert( GroupIconList!=NULL );

/*
 *  disable mouse-activated pop-up menu for system and non-writable groups
 *
    if ( obj->isSystem() )  GroupIconList->disableMenu();
*/

    sw=manage(new Gtk::ScrolledWindow());
    sw->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    group_internal_hbox->remove( *label_placeholder );
    group_internal_hbox->pack_start(*sw);
    sw->show();

    sw->add( *GroupIconList);
    GroupIconList->show();

    set_name("group_dialog");

  /*******************************************************************/

    /* Drag destination */
    GroupIconList->drag_dest_set ( GTK_DEST_DEFAULT_ALL,
				   target_table, n_targets,
				   static_cast < GdkDragAction > ( GDK_ACTION_COPY ) );

    GroupIconList->drag_data_received.connect( 
	slot ( this, & ( GroupDialog::obj_drag_data_received ) ) );

  /*******************************************************************/

  /*  When user presses Enter while one of the group items is selected,
   *    IconList sends signal open_item. 
   */

    GroupIconList->open_item.connect(SigC::slot(this,&GroupDialog::on_open_item));

    GroupIconList->popup_menu.connect(SigC::slot(this,&GroupDialog::on_popup_menu));


    GdkPixmap *pmap;
    GdkBitmap *mask;

    if ( gdk_imlib_load_file_to_pixmap(
	(char*)(object->getXPMIconFileName("_dlg").c_str()),&pmap, &mask)){
	group_icon->set(pmap,mask);
	gdk_imlib_free_pixmap(pmap);
    }
}

GroupDialog::~GroupDialog()
{
//    sw->remove();
//    if (GroupIconList!=NULL) delete GroupIconList;
}

void GroupDialog::on_open_item(void *arg)
{
    string   *id = (string*)arg;
    if (id!=NULL)  main_w->schedule_open_object(*id);
}

void GroupDialog::on_popup_menu(void *arg)
{
    Gtk::Widget *itm;

    char **menu_items0;
    gen_popup_menu *gpm;

    menu_items0=new char*[6];
    menu_items0[0]="Open";
    menu_items0[1]="";
    menu_items0[2]="Copy";
    menu_items0[3]="Cut";
    menu_items0[4]="Paste";
    menu_items0[5]=NULL;
    gpm=new gen_popup_menu( (const char**)menu_items0 );

      

//    FWObject *o=(FWObject*)arg;
    string *id=(string*)arg;

    FWObject *scr  = NULL;
    FWObject *org  = NULL;

    if (id==NULL) {
	itm=find_widget("Open",gpm);   if (itm) itm->set_sensitive(false);
	itm=find_widget("Copy",gpm);   if (itm) itm->set_sensitive(false);
	itm=find_widget("Cut",gpm);    if (itm) itm->set_sensitive(false);
	itm=find_widget("Paste",gpm);  if (itm) itm->set_sensitive(true );
    } else {
	scr  = FWObjectDatabase::ScratchPad->getById(*id,true);
	org  = FWObjectDatabase::db->getById(*id,true);        

	itm=find_widget("Copy",gpm); if(itm)itm->set_sensitive(!org->isSystem());
	itm=find_widget("Cut",gpm);  if(itm)itm->set_sensitive(!org->isSystem());
	itm=find_widget("Paste",gpm);if(itm)itm->set_sensitive(!org->isSystem());
    }

    gpm->popup(0,0);
    gint menu_choice=gpm->run();
    delete gpm;

    delete menu_items0;
    
    switch (menu_choice) {
    case 0:    // open
	main_w->schedule_open_object(*id);
	break;
    case 2:    // copy
	if (org) FWObjectClipboard::obj_clipboard->putObject(org);
	break;
    case 3:    // cut
	if (org) {
	    FWObjectClipboard::obj_clipboard->putObject(org);
	    GroupIconList->removeObject(-1);
	    data_changed_flag(true);
	}
	break;
    case 4:    // paste
	FWObject *obj=FWObjectClipboard::obj_clipboard->getObject();
	addObject(obj);
	break;
    }
  
}


void GroupDialog::on_viewport1_check_resize() {  }
void GroupDialog::on_viewport1_size_allocate(GtkAllocation *all)  { }

void GroupDialog::wrk2dlg()
{
    bool       show_hidden;
    FWObject  *obj;
    vector<FWObject*>::iterator m;

    g_assert( object!=NULL );

    show_hidden= (Preferences::global_prefs->getOptBool(
	"/FWBuilderPreferences/UI/ShowHidden"));

    Group     *gr=Group::cast(object);

    if (gr->isSystem()) {
	group_name_lbl->set_text( gr->getName() );
	group_name->hide();
	group_name_lbl->show();
    } else {
	group_name->show();
	group_name_lbl->hide();
	group_name->set_text( gr->getName() );
	group_name->set_editable(true);
    }

    GroupIconList->clear();

    for (m=gr->begin(); m!=gr->end(); ++m) {
	if ( (obj=(*m))!=NULL ) {
	    if ( FWReference::cast(obj)!=NULL) {
		obj=( FWReference::cast(obj) )->getPointer();
	    }
	    if (show_hidden ||
		! Resources::global_res->getObjResourceBool(obj,
							    "hidden") ) {
		GroupIconList->addObject(obj);
	    }
	}
    }
}


bool GroupDialog::dlg2wrk()
{
    g_assert( object!=NULL );
    Group     *gr=Group::cast(object);
    FWObject *obj;

/*
 *  check if name is valid
 */
    if ( ! checkObjectName(group_name->get_text()) ) return(false);

    object->setName( group_name->get_text() );

//    gr->clearChildren();

    vector<FWObject*>::iterator m;
    int i;

    string id1, id2;

 loop0:
    for (m=gr->begin(); m!=gr->end(); ++m) 
	if ( (obj=(*m))!=NULL ) {
	    id1=obj->getId();
	    for (i=0; i<GroupIconList->getCount(); ++i) {
		id2=GroupIconList->getObjectId(i);
		if (id1==id2) break;
	    }
	    if (id1==id2) continue;

	    gr->remove(obj);
	    goto loop0; // removal invalidates iterator
	}


    for (i=0; i<GroupIconList->getCount(); ++i) {
	id2=GroupIconList->getObjectId(i);
	FWObject *o=FWObjectDatabase::ScratchPad->getById(id2,true);

	if (o) {
	    for (m=gr->begin(); m!=gr->end(); ++m) {
		obj=(*m);
		id1=obj->getId();
		if (id1==id2) break;
	    }
	    if (id1==id2) continue;

	    gr->addRef( o );
	}
    }

    return(true);
}

void GroupDialog::Update()
{
  wrk2dlg();
}

void GroupDialog::addObject(FWObject *obj)
{
    Group     *gr=Group::cast(object);

    if (obj==NULL) return;

/*
 *  check if object is appropriate for this group
 */
    string id=obj->getId();

    if (gr->validateChild( obj ) && ! GroupIconList->checkObjectId(id) ) {

	GroupIconList->addObject( obj );
	data_changed_flag(true);
    }
}





void GroupDialog::obj_drag_data_received ( GdkDragContext     *context,
					   gint                x,
					   gint                y,
					   GtkSelectionData   *data,
					   guint               info,
					   guint32             time)
{
    char           obj_id[64];

    g_assert( object!=NULL );
    Group     *gr=Group::cast(object);

    Gdk_DragContext gdc ( context );

    if ( ( data -> length >= 0 ) && ( data -> format == 8 ) ) {

	//    cerr << "*** Drag&Drop: Received \""
	//         << (gpointer)(data->data)
	//         << "\" GroupDialog "
	//         << " x=" << x << " y=" << y 
	//	 << " info=" << info
	//	 << " length=" << data->length
	//         << endl;

	Gtk::Widget::drag_finish ( gdc, true, false, time );
    
	FWObject *obj;

	memcpy(obj_id,(char*)data->data,data->length);
	obj=FWObjectDatabase::ScratchPad->getById( obj_id , true );

//	memcpy(&obj,data->data,data->length);

/*
 * if this is system group or it is not writable, do not accept object
 */
	if ( gr->isSystem()  )  return;

	addObject(obj);

/*
  GroupIconList->addObject( obj );
  data_changed_flag(true);
*/
	return;
    }
    Gtk::Widget::drag_finish ( gdc , false, false, time );
}







