/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: HostDialog.cc,v 1.18 2001/07/23 05:25:25 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"

#include "HostDialog.hh"
#include "Host.hh"
#include "MessageDialog.hh"
#include "helpers.hh"
#include "dns.hh"
#include "IPAddressWidget.hh"

#include <gdk_imlib.h>


HostDialog::HostDialog(FWObject *obj) 
{
    object=obj; 

    obj_addr=manage(new IPAddressWidget());
    table19->attach(*obj_addr, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
    obj_addr->changed.connect(SigC::slot(this,&HostDialog::on_changed));


    GdkPixmap *pmap;
    GdkBitmap *mask;

    if (gdk_imlib_load_file_to_pixmap(
	(char*)(object->getXPMIconFileName("_dlg").c_str()),&pmap, &mask)){
	obj_icon->set(pmap,mask);
	gdk_imlib_free_pixmap(pmap);
    }
}


void HostDialog::wrk2dlg()
{
    obj_name->set_text(object->getName());
    obj_addr->setAddress(object->getStr("address"));

    comment->set_point(0);
    comment->forward_delete( comment->get_length() );
    comment->insert(object->getComment());

    obj_name->grab_focus();
}

bool HostDialog::dlg2wrk()
{

/*
 *  check if name is valid
 */
    if ( ! checkObjectName(obj_name->get_text()) ) return(false);
    if ( ! checkIPaddress( obj_addr->getAsString() ) )  return(false);

    object->setName( obj_name->get_text() );
    object->setComment( comment->get_chars(0,comment->get_length()) );

    object->setStr( "address" , obj_addr->getAsString() );

    return(true);
}

/**
 *  this method runs DNS lookup on the host name and enters address in obj_addr
 */
void HostDialog::on_dns_lookup()
{
    vector<IPAddress> addr;

    try {
	addr=DNS::getHostByName( obj_name->get_text() );
    } catch (FWException &e)
    {
	MessageDialog::Error( e.toString() );
	return;
    }

    obj_addr->setAddress( addr.front() );
    data_changed_flag(true);
}

void HostDialog::on_changed()
{
    data_changed_flag(true);
}
