/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: Interface.cc,v 1.15 2001/07/13 16:24:39 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "Interface.hh"
#include "InterfacePolicy.hh"
#include "XMLTools.hh"
#include "Preferences.hh"

const char *Interface::TYPENAME={"Interface"};

Interface::Interface(const Interface &i):FWObject((FWObject&)i)
{
    physical_address = i.physical_address;
}

Interface::Interface(Interface *i):FWObject(*i)
{
    physical_address = i->physical_address;
}

Interface::Interface():FWObject()
{
    setStr("address", "");
    setStr("netmask", "");
    setBool("ext",false);
    setBool("dyn",false);
}

Interface::Interface(const string& n,const string& a,const string& nm,
		     bool ext,bool dyn, string _physical_address)
{
  setName(n);
  setStr("address",a);
  setStr("netmask",nm);
  setBool("ext",ext);
  setBool("dyn",dyn);
  physical_address = _physical_address;
}

Interface::Interface(const string& n,const string& a,const string& nm,
		     const string& ext,const string& dyn, string _physical_address)
{
  setName(n);
  setStr("address",a);
  setStr("netmask",nm);
  setBool("ext", (ext=="ext") );
  setBool("dyn", (dyn=="dyn") );
}

const IPAddress Interface::getIPAddess() const throw(FWException)
{
    return IPAddress(getStr("address"));
}

void Interface::fromXML(xmlNodePtr root)
{
    FWObject::fromXML(root);

    const char *n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("address")));
    assert(n!=NULL);
    setStr("address", n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("netmask")));
    assert(n!=NULL);
    setStr("netmask", n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("ext")));
    if (n!=NULL) setStr("ext",n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("dyn")));
    if (n!=NULL) setStr("dyn",n);
}


bool Interface::isExt() {  return(getBool("ext")); }
bool Interface::isDyn() {  return(getBool("dyn")); }

bool  Interface::validateChild(FWObject *o)
{
    string otype=o->getTypeName();
    return (FWObject::validateChild(o) && 
            (otype==InterfacePolicy::TYPENAME ));
}

IPNetwork Interface::getIPNetwork() const throw(FWException)
{
    return IPNetwork(IPAddress(getStr("address")), 
                     Netmask(getStr("netmask"))
    );
}




















