/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: InterfaceDialog.cc,v 1.14 2001/07/17 06:29:18 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <gtk--/main.h>

#include "IPAddress.hh"
#include "InterfaceDialog.hh"
#include "IPAddressWidget.hh"

#include "helpers.hh"

InterfaceDialog::InterfaceDialog( Interface *i , bool hc)
{
    choice=false;
    hide_checkbuttons=hc;
    interface=i;

    addr=manage(new IPAddressWidget());
    table1->attach(*addr, 1, 2, 3, 4, GTK_EXPAND|GTK_FILL, 0, 0, 0);

    netmask=manage(new IPAddressWidget());
    table1->attach(*netmask, 1, 2, 4, 5, GTK_EXPAND|GTK_FILL, 0, 0, 0);

    addr->setAssociatedNetmaskWidget( netmask );

    name->set_text( i->getName() );
    if (! i->isDyn() ) {
	addr->setAddress( i->getStr("address") );
	netmask->setAddress( i->getStr("netmask") );
//	addr->set_editable(true);
//	netmask->set_editable(true);
    } else {
//	addr->set_text( "DYNAMIC" );
//	netmask->set_text( "" );
//	addr->set_editable(false);
//	netmask->set_editable(false);
	addr->clear();
	addr->set_sensitive(false);
	netmask->clear();
	netmask->set_sensitive(false);
    }
    extint->set_active( i->isExt() );
    dyn_addr->set_active( i->isDyn() );

    name->grab_focus();

    button17->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
    button18->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
    button17->grab_default();
}

bool InterfaceDialog::run()
{
    if (hide_checkbuttons) {
//	extint->set_sensitive(false);
//	dyn_addr->set_sensitive(false);

	extint->hide();
	dyn_addr->hide();
    }


    while (true) {
	Gtk::Main::grab_add(*this);
	Gtk::Main::run();
	Gtk::Main::grab_remove(*this);

	if (choice) {
	    if (  ! checkObjectName(name->get_text() ) ) continue;
	    if (  ! dyn_addr->get_active() ) {
		if ( ! checkIPaddress(addr->getAsString()) ) continue;
		if ( ! checkNetmask(netmask->getAsString()) ) continue;
	    }
	}
	break;
    }

    hide();

    if (choice) {
	interface->setName(name->get_text() );
	interface->setStr("address",addr->getAsString());
	interface->setStr("netmask",netmask->getAsString());
	interface->setBool("ext",extint->get_active());
	interface->setBool("dyn",dyn_addr->get_active());
    }
    return(choice);
}



void InterfaceDialog::on_ok_clicked()
{   
    choice=true;
    Gtk::Main::quit();
}

void InterfaceDialog::on_cancel_clicked()
{   
    Gtk::Main::quit();
}

void InterfaceDialog::on_extint_toggled()
{   
}

void InterfaceDialog::on_dyn_addr_toggled()
{

    if (dyn_addr->get_active()) {
//	addr->set_text( "DYNAMIC" );
//	netmask->set_text("");
//	addr->set_editable(false);
//	netmask->set_editable(false);

	addr->clear();
	addr->set_sensitive(false);
	netmask->clear();
	netmask->set_sensitive(false);
    } else {
//	addr->set_editable(true);
//	netmask->set_editable(true);

	addr->set_sensitive(true);
	netmask->set_sensitive(true);
    }

}


