/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: ListOfIcons.cc,v 1.17 2001/07/12 14:45:28 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <gdk_imlib.h>

#include "glademm_support.hh"

#include "FWObjectDatabase.hh"
#include "FWReference.hh"
#include "BuiltinDialog.hh"
#include "ListOfIcons.hh"
#include "helpers.hh"

/*
 *   this class uses our gnome UI wrapper IconList and adds events handling
 *   to it
 *   (vadim)
 */
ListOfIcons::ListOfIcons(gint icon_width ,bool allow_popup_menu )
{
    current_selected=-1;
    count=0;
    popup=allow_popup_menu;

    set_flags( GTK_CAN_FOCUS );


    show();

    add_events(  GDK_EXPOSURE_MASK |
		 GDK_BUTTON_PRESS_MASK |
		 GDK_BUTTON_RELEASE_MASK |
		 GDK_KEY_PRESS_MASK |
		 GDK_KEY_RELEASE_MASK);

    button_press_event.connect(SigC::slot(this, 
				  &ListOfIcons::on_button_press_event));
    
    button_release_event.connect_after(SigC::slot(this, 
				  &ListOfIcons::on_button_release_event));
    
    key_press_event.connect(SigC::slot((ListOfIcons*)this, 
				       &ListOfIcons::on_key_press_event));
    
}

ListOfIcons::~ListOfIcons()
{
//    IconList::clear();
}


void ListOfIcons::set_data_changed_flag(bool flag)
{
    BuiltinDialog *b= (BuiltinDialog *)find_widget("BuiltinDialog",this);
    b->data_changed_flag(flag);
}


gint ListOfIcons::on_key_press_event(GdkEventKey *ev)
{
    /*
     * Keys we are looking for:
     * GDK_Tab
     * GDK_Up
     * GDK_Down
     *   etc.
     */

    if (current_selected==-1) return(false);

    switch (ev->keyval ) {
	//  case GDK_Tab:
	//    cerr << "               Tab\n";
	//    break;
    case GDK_Up:
	move_focus(GTK_DIR_UP);
	break;
    case GDK_Down:
	move_focus(GTK_DIR_DOWN);
	break;
    case GDK_Left:
	move_focus(GTK_DIR_LEFT);
	break;
    case GDK_Right:
	move_focus(GTK_DIR_RIGHT);
	break;
    case GDK_Return:
	open_item( get_icon_data( current_selected )  );    // this is signal
	break;

    default:
	return(false);
    }
    gtk_signal_emit_stop_by_name( GTK_OBJECT( gtkobj() ),
				"key_press_event" );
    return(true);
}

gint ListOfIcons::on_button_press_event(GdkEventButton *ev)
{
    int       x,y,icn;

    x=gint(ev->x);     y=gint(ev->y);
    icn=get_icon_at(x,y);

    
    if (!has_focus()) {
	grab_focus();
	draw_focus();
    }

    if (icn!=-1)  current_selected=icn;

    if ( ev->type == GDK_2BUTTON_PRESS  && ev->button==1 ) {

	open_item( get_icon_data( current_selected ) );    // this is signal

//	    cerr << "Doubleclick: o=" << o 
//		 << " name=" << o->getName()
//		 << " type=" << o->getTypeName()
//		 << " root=" << o->getRoot()
//		 << endl;
    }
    return(true);
}

gint ListOfIcons::on_button_release_event(GdkEventButton *ev)
{
    gint  x,y,icn;

    x=gint(ev->x);     y=gint(ev->y);
    icn=get_icon_at(x,y);

    string   *id=NULL;

    if (icn!=-1) 
	id=(string*)(get_icon_data( icn ));

    if ( ev->type == GDK_BUTTON_RELEASE    && ev->button==3 ) {
	if (! popup ) return(true);   // popup menu disabled
    	popup_menu(id);
    }
    return(true);
}


void ListOfIcons::move_focus(GtkDirectionType dir)
{

    int  n=get_items_per_line();

    if (current_selected==-1) {
	current_selected=0;
	select_icon(0);
    }

    unselect_icon(current_selected);

    switch (dir) {
    case GTK_DIR_UP:
	current_selected-=n;
	break;
    case GTK_DIR_DOWN:
	current_selected+=n;
	break;
    case GTK_DIR_LEFT:
	--current_selected;
	break;
    case GTK_DIR_RIGHT:
	++current_selected;
	break;
    default:
	return;
    }
    if (current_selected<0)      current_selected=0;
    if (current_selected>=count) current_selected=count-1;

    select_icon(current_selected);
}

void ListOfIcons::clear()
{
    IconList::clear();
    count=0;
}


gint ListOfIcons::getCount()
{
    return( count );
}

void ListOfIcons::addObject(FWObject *obj)
{
    int            icn;
    GdkImlibImage *img;
    string         suffix="";
    string         iconfile;
    string         id;

    if (obj==NULL) return;

    id=obj->getId();

    if ( ! checkObjectId(id) ) {

	if (Preferences::global_prefs->getOptBool(
	    "/FWBuilderPreferences/UI/LargeIconsInGroups")) 
	    suffix="_dlg";

	iconfile=obj->getXPMIconFileName(suffix);
        if((img=gdk_imlib_load_image( (char*)(iconfile.c_str()) ))!=NULL)
        {
	    icn=append_imlib( img , obj->getName() );

            // We return pointer to actuall string residing
            // within other object holding it's ID.
            // If this object is deleted, this string will
            // become invalid.
            // We assume here that this would not happen, while
            // this dialogue is running.
            // (lord)
	    set_icon_data(icn, (void*)&obj->getId());
	}
	count++;
    }
}

void ListOfIcons::removeObject(int no)
{
    if (no==-1) {
	remove(current_selected);
	count--;
	current_selected--;
	if (current_selected<0) current_selected=0;
	select_icon(current_selected);
    } else {

	remove(no);
	count--;
    }
}

bool     ListOfIcons::checkObjectId(const string& id)
{
    string id2;
    for (int i=0; i<getCount(); ++i) {
	id2=getObjectId(i);
	if (id==id2) return true;
    }
    return false;
}

string   ListOfIcons::getObjectId(int no)
{
    string     *id=(string*)(get_icon_data(no));
    if (id==NULL) return "";
    return (*id);
}


