/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: MessageDialog.cc,v 1.16 2001/06/23 18:32:48 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <gtk--/main.h>

#include "MessageDialog.hh"
#include "helpers.hh"

#include <gdk_imlib.h>

#include <libgnomeui/gnome-stock.h>
#include "gnome-wrappers/wrappers.hh"



MessageDialog::MessageDialog(const string& msg,DlgType t)
{
    GdkPixmap *pmap;
    GdkBitmap *mask;
    string     icn;


    running=false;

    result=CANCEL;

    dialog_message->set_text(msg);
    dialog_message->set_justify(GTK_JUSTIFY_LEFT);

    GtkButton *gtk_button1;

    switch (t) {
    case QuestionDlg:
	icn = Resources::global_res->getIconPath("Question");
	button1->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
	button1->grab_default();
	button1->grab_focus();
	button1->show();
	break;

    case YesNoDlg:
	icn = Resources::global_res->getIconPath("Question");
	get_action_area()->remove( *button3 );
	button1->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
	button1->grab_default();
	button1->grab_focus();
	button1->show();
	break;

    case ErrorDlg:
	icn = Resources::global_res->getIconPath("Error");

	get_action_area()->remove( *button1 );
	get_action_area()->remove( *button2 );
	get_action_area()->remove( *button3 );

	gtk_button1 = GTK_BUTTON(gnome_stock_button(GNOME_STOCK_BUTTON_OK));
	button1 = new Button(gtk_button1);
	button1->set_usize(80, 40);
	button1->set_name("button1");
	button1->clicked.connect(SigC::slot((MessageDialog*)this, &MessageDialog::on_button1_clicked));
	get_action_area()->pack_start(*button1, false, false, 0);

	button1->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
	button1->grab_default();
	button1->grab_focus();
	button1->show();
	break;

    case WarningDlg:
	icn = Resources::global_res->getIconPath("Warning");


	get_action_area()->remove( *button1 );
	get_action_area()->remove( *button2 );
	get_action_area()->remove( *button3 );

	gtk_button1 = GTK_BUTTON(gnome_stock_button(GNOME_STOCK_BUTTON_OK));
	button1 = new Button(gtk_button1);
	button1->set_usize(80, 40);
	button1->set_name("button1");
	button1->clicked.connect(SigC::slot((MessageDialog*)this, &MessageDialog::on_button1_clicked));
	get_action_area()->pack_start(*button1, false, false, 0);

	button1->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
	button1->grab_default();
	button1->grab_focus();
	button1->show();
	break;
    }
    
    if (gdk_imlib_load_file_to_pixmap((char*)(icn.c_str()),&pmap, &mask)){
	pixmap->set(pmap,mask);
	gdk_imlib_free_pixmap(pmap);
    }

    show_all();
    delete_event.connect( SigC::slot(this, &MessageDialog::on_delete) );
}

MessageDialog::DlgReturnValue MessageDialog::run()
{
    Gtk::Main::grab_add(*this);
    running=true;
    Gtk::Main::run();
    running=false;
    Gtk::Main::grab_remove(*this);
    hide();
    return result;
}

gint MessageDialog::on_delete(GdkEventAny* ev)
{
    if (running) {
	Gtk::Main::quit();
	running=false;
    }
    return(true);
}


void MessageDialog::on_button1_clicked()
{   
    result=OK_YES;
    Gtk::Main::quit();
//    destroy();
    running=false;
}

void MessageDialog::on_button2_clicked()
{   
    result=NO;
    Gtk::Main::quit();
//    destroy(); 
    running=false;
}

void MessageDialog::on_button3_clicked()
{   
    result=CANCEL;
    Gtk::Main::quit();
//    destroy();
    running=false;
}

void MessageDialog::Error(const string& msg)
{
    MessageDialog *md = new MessageDialog(msg,ErrorDlg);
    md->run();
    delete md;
}

void MessageDialog::Error(const string& msg, const string& errcode)
{
    string m;
    m=msg + "\n" + errcode;
    MessageDialog *md = new MessageDialog(m,ErrorDlg);
    md->run();
    delete md;
}

void MessageDialog::Warning(const string& msg)
{
    MessageDialog *md = new MessageDialog(msg,WarningDlg);
    md->run();
    delete md;
}

MessageDialog::DlgReturnValue  MessageDialog::Question(const string& msg)
{
    DlgReturnValue r;
    MessageDialog *md = new MessageDialog(msg,QuestionDlg);
    r=md->run();
    delete md;
    return r;
}


MessageDialog::DlgReturnValue  MessageDialog::YesNo(const string& msg)
{
    DlgReturnValue r;
    MessageDialog *md = new MessageDialog(msg,YesNoDlg);
    r=md->run();
    delete md;
    return r;
}












