/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATDialog.cc,v 1.7 2001/05/18 09:08:14 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "FWObjectDatabase.hh"
#include "NATDialog.hh"
#include "helpers.hh"

#include "Firewall.hh"
#include "NAT.hh"
#include "PolicyList.hh"
#include "main_window.hh"

NATDialog::NATDialog(FWObject *obj)
{
    RuleSet *pol;

    object=obj;
    dialog_buttons=true;
    
    policylist_ptr=NULL;
    wpptr=NULL;

//    if (obj->getTypeName()=="NAT")  p =(Policy*)obj;
    if ( NAT::isA(obj) ) pol=RuleSet::cast(obj);
    else {

	remove();
	Gtk::Label *l=
	    manage(new Gtk::Label("Unsupported firewall platform "));
	add(*l);
	l->show();
	return;
    }

    policylist_ptr = manage(new PolicyList(pol));

    /*******************************************************************/

    /*  When user presses Enter while one of the policy items is selected,
     *  or chooses "Edit" pop-up menu item ,
     *    PolicyList sends signal open_item. 
     */

    policylist_ptr->open_item.connect(SigC::slot(this,&NATDialog::on_open_item));

    remove();

    Gtk::ScrolledWindow *sw=new Gtk::ScrolledWindow();
    sw->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    add(*sw);
    sw->show();

    sw->add_with_viewport(*policylist_ptr);
    policylist_ptr->show();

    set_name("nat_dialog");
}

NATDialog::~NATDialog()
{
//  if (policylist_ptr!=NULL)  delete policylist_ptr;
//  policylist_ptr=NULL;
}

void NATDialog::on_open_item(void *arg)
{
    FWObject *obj=(FWObject*)arg;
    if (obj!=NULL) main_w->schedule_open_object(obj->getId());
}


void NATDialog::on_policy_line_add()
{
  data_changed_flag(true);
}

void NATDialog::on_policy_line_remove()
{
  data_changed_flag(true);
}

void NATDialog::wrk2dlg()
{
    if (policylist_ptr!=NULL)
	policylist_ptr->Build();
}

bool NATDialog::dlg2wrk()
{
    return(true);
}



Gtk::Menu* NATDialog::BuildCustomMenu()
{
  Gtk::Menu *policy_menu = new Gtk::Menu();
/*
  Gtk::MenuItem *insert_top=new Gtk::MenuItem("Insert rule at the top");
  Gtk::MenuItem *append_bottom=new Gtk::MenuItem("Append rule at the bottom");
  Gtk::MenuItem *add_rule=new Gtk::MenuItem("Add rule after current");
  Gtk::MenuItem *remove_rule=new Gtk::MenuItem("Remove current rule");
  Gtk::MenuItem *move_up=new Gtk::MenuItem("Move current rule up");
  Gtk::MenuItem *move_down=new Gtk::MenuItem("Move current rule down");

  insert_top->set_name("insert_top");
  insert_top->activate.connect(SigC::slot((NATDialog*)this, 
  			  &NATDialog::on_insert_top));
  append_bottom->set_name("append_bottom");
  append_bottom->activate.connect(SigC::slot((NATDialog*)this, 
  			  &NATDialog::on_append_bottom));
  add_rule->set_name("add_rule");
  add_rule->activate.connect(SigC::slot((NATDialog*)this, 
  			  &NATDialog::on_add_rule));

  remove_rule->set_name("remove_rule");
  remove_rule->activate.connect(SigC::slot((NATDialog*)this, 
  			  &NATDialog::on_remove_rule));

  move_up->set_name("move_up");
  move_up->activate.connect(SigC::slot((NATDialog*)this, 
  			  &NATDialog::on_move_up));

  move_down->set_name("move_down");
  move_down->activate.connect(SigC::slot((NATDialog*)this, 
			  &NATDialog::on_move_down));

  policy_menu->set_name("policy_menu");
  policy_menu->append(*insert_top);
  policy_menu->append(*append_bottom);
  policy_menu->append(*add_rule);
  policy_menu->append(*remove_rule);
  policy_menu->append(*move_up);
  policy_menu->append(*move_down);
*/
  return(policy_menu);
}



/*
 * Submenu Policy
 *
void NATDialog::on_insert_top()    { insertRuleBefore(0); }
void NATDialog::on_append_bottom() { appendRuleAfter(1000);  }
void NATDialog::on_add_rule()      { appendRuleAfter(-1);    }
void NATDialog::on_remove_rule()   { delRule(-1);         }
void NATDialog::on_move_up()       { }
void NATDialog::on_move_down()     { }

void NATDialog::insertRuleBefore(gint row_n)
{
    if (policylist_ptr!=NULL)
	policylist_ptr->insertRuleBefore(row_n);
}

void NATDialog::appendRuleAfter(gint row_n)
{
    if (policylist_ptr!=NULL) 
	policylist_ptr->appendRuleAfter(row_n);
}

void NATDialog::delRule(gint row_n)
{
    if (policylist_ptr!=NULL) 
	policylist_ptr->delRule(row_n);
}


*/


