/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: ObjectTree.cc,v 1.40 2001/07/07 07:19:16 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "glademm_support.hh"

#include "FWObject.hh"
#include "FWObjectDatabase.hh"
#include "ObjectTree.hh"
#include "Group.hh"

#include "GroupDialog.hh"

#include "Firewall.hh"
#include "FWOptions.hh"
#include "Policy.hh"
#include "NAT.hh"


#include "main_window.hh"
#include "gen_popup_menu.hh"
#include "helpers.hh"

#include <gdk/gdk.h>
#include <gdk_imlib.h>


#include <list>
#include <vector>
#include <map>

#include <stdlib.h>
#include <stdio.h>

static char drag_obj_id[64];

ObjectTree::ObjectTree() : CTree(1) 
{
    show_all=false;
    rebuild_scheduled=false;

//  set_selection_mode( GTK_SELECTION_BROWSE );
    set_selection_mode( GTK_SELECTION_SINGLE );

    selected_object="";
    last_clicked_object="";
    drag_object="";
    show_object="";

    //  rebuildTree();
    //  schedule_rebuild();

    set_name("ObjectTree");
    glademm_set_Widget("ObjectTree", this);

    //  set_expander_style( GTK_CTREE_EXPANDER_SQUARE );

    set_expander_style( GTK_CTREE_EXPANDER_CIRCULAR );
    set_line_style( GTK_CTREE_LINES_SOLID  );

    button_press_event.connect(
	slot(this, &ObjectTree::on_button_press_event));
    
    button_release_event.connect_after(
	slot(this, &ObjectTree::on_button_release_event));


    /*****************************************************************
     *  Initialize Drag&DRop mechanism 
     */
    drag_source_set ( 
	static_cast<GdkModifierType>(GDK_BUTTON1_MASK) ,
	target_table, n_targets, 
	static_cast<GdkDragAction>(GDK_ACTION_COPY) );

    drag_begin.connect(
	slot(this,&ObjectTree::source_drag_begin));
    drag_data_get.connect(
	slot(this,&ObjectTree::source_drag_data_get));
    drag_data_delete.connect(
	slot(this,&ObjectTree::source_drag_data_delete));


    select_row.connect( slot(this,&ObjectTree::selection_made));

    key_press_event.connect(SigC::slot((ObjectTree*)this, 
				       &ObjectTree::on_key_press_event));

}

gint ObjectTree::on_key_press_event(GdkEventKey *ev)
{
    gchar  str[32];
    FWObject  *o=NULL;
    guint      n;
    gchar      c;
    string     id;

    o=FWObjectDatabase::ScratchPad->getById( get_current_selection(), true ) ;
    assert(o!=NULL);

    if ( ev->keyval == GDK_F7 ) {   
// F7, dump selected object
	o->dump(true,false);
	return(0);
    }

    if ( ev->keyval == GDK_F8 ) {   
// F8, dump scratch pad
	std::ofstream out("fwbuilder_scratchpad.dump");
	FWObjectDatabase::ScratchPad->dump(out,true,false);
	return(0);
    }

    if ( ! main_w->safe_to_close_right_pane() ) {
	/*
	 *  All key presses change selection in the tree. If it is not safe
	 *  to close right pane and show another object there, then igore it.
	 */
	gtk_signal_emit_stop_by_name( GTK_OBJECT( gtkobj() ),
				      "key_press_event" );
 
	return(1);
    }


    Gtk::CTree_Helpers::RowList          rl=rows();
    if (rl.empty()) return(0);
  
    strncpy(str,ev->string,ev->length);
    c=str[0];

    if ( ev->keyval == GDK_F1 ) {   
// F1, just print objects address,ID, name etc.
	o->dump(true,true);
	return(0);
    }

    if ( ev->keyval == GDK_F4 ) {   
// F4, sort children of selected object
	on_sort();
	return(0);
    }


    if ( ev->keyval == GDK_F5 ) {   
// F5, refresh the tree
	schedule_rebuild();
	return(0);
    }

    if ( ev->keyval == GDK_F6 ) {   
// F6, toggle show_all flag and refresh the tree
	show_all= ! show_all;
	schedule_rebuild();
	return(0);
    }


    if ( ev->keyval == GDK_Return ) {   // ENTER

	RowList::iterator k;
	for (n=0,k=rl.begin(); k!=rl.end(); ++k,++n) {
	    id=(char*)(k->get_data());
	    if (id==get_current_selection()) {

		//    n = current_selected_row;
		// subtree ?
		Gtk::CTree_Helpers::Row  row=rows()[n];
		Gtk::CTree_Helpers::RowList srl = row.subtree();
		if (! srl.empty()) {
		    row.expand();
		    return(0);
		}
		main_w->schedule_open_object(id);
		return(0);
	    }
	}
	return(0);
    }
  
    RowList::iterator k;
    for (n=0,k=rl.begin(); k!=rl.end(); ++k,++n) {
	id=(char*)(k->get_data());
	if (id==get_current_selection()) break;
    }
    ++k; ++n;
    for (; k!=rl.end(); ++k,++n) {
	id=(char*)(k->get_data());
	o=FWObjectDatabase::ScratchPad->getById( id , true );
	assert(o!=NULL);
	if (tolower( o->getName()[0] )==c ) {
	    selectRow(n);
	    selected_object=id;
	    return(0);
	}
    }

    return 0;
}

void ObjectTree::selectRow(gint row_n)
{
    /*
     * calling "select()" for CTree_Helpers::Row
     * does not move focus (but moves selection). There is no "focus()"
     * call for CTree_Helpers::Row, so we have to dig out its 
     * equivalentfrom CList_Helpers and call focus() for it
     *
     * It does not really matter which one we use here to call select()
     */
    Gtk::CList_Helpers::Row r = Gtk::CList::row(row_n);
    r->focus();
    r->select();
}

void ObjectTree::selectObject(string id)
{
    gint       n;
    Gtk::CTree_Helpers::RowList          rl=rows();
    if (rl.empty()) return;
  
    RowList::iterator k;
    for (n=0,k=rl.begin(); k!=rl.end(); ++k,++n) {
	if ( id==(char*)(k->get_data()) ) {
	    selectRow(n);
	    selected_object=id;
	}
    }
}

static bool TraverseTreeAndExpand(Gtk::CTree_Helpers::RowList rl,
				  string &id)
{
    using namespace Gtk::CTree_Helpers;

    RowList::iterator k;

    if (rl.empty()) return(false);
  
    for (k=rl.begin(); k!=rl.end(); ++k) {
	if ( id==(char*)(k->get_data()) ) {
	    k->select();
	    return(true);
	}
	RowList srl = k->subtree();
	if (TraverseTreeAndExpand(srl,id))
	    k->expand();
    }
    return(false);
}

void ObjectTree::rebuildTreeAndShow(string &id)
{
    rebuildTree();
    TraverseTreeAndExpand(rows(), id);
    selected_object=id;
    main_w->schedule_open_object(id);
}

void ObjectTree::ShowObject(string &id)
{
    if (id=="")	clearSelection();
    else
	if (id!=get_current_selection()) {
	    TraverseTreeAndExpand(rows(), id);
	    selected_object=id;
	}
}

void ObjectTree::clearSelection()
{
    selected_object="";
}

void ObjectTree::schedule_rebuild()
{
    if ( !rebuild_scheduled ) {
	Gtk::Main::idle.connect(slot(this,&ObjectTree::rebuild_when_idle));
	rebuild_scheduled=true;
    }
}

gint ObjectTree::rebuild_when_idle()
{
  rebuildTree();
  return false;
}

void ObjectTree::schedule_rebuild_and_show(string id)
{
    if (id=="") return;
    show_object=id;
    Gtk::Main::idle.connect(slot(this,&ObjectTree::rebuild_and_show_when_idle));
}

gint ObjectTree::rebuild_and_show_when_idle()
{
    if (show_object=="") return false;
    rebuildTreeAndShow(show_object);
    show_object="";
    return false;
}

void ObjectTree::displaySubtree(  Gtk::CTree_Helpers::RowList::iterator &i , 
				  FWObject *obj )
{
    vector<const gchar*>      item;
    RowList::iterator         j;
    vector<FWObject*>::iterator m;
    string                    name;
    FWObject                 *o;

    if (obj==NULL) return;

    if (FWObjectReference::isA(obj) && ! show_all) return;

    if (Resources::global_res->getObjResourceBool(obj,"hidden") &&
	! show_all ) return;

    name=obj->getName();
    if (name=="") {  // no name, use type description string instead
	name=Resources::global_res->getObjResourceStr(obj,"description");
    }
    item.push_back(name.c_str());
    i->subtree().push_back(Gtk::CTree_Helpers::Element(item));
    j= --i->subtree().end();

    /*
     *  store object ID in the tree widget element
     */
    char *id_storage=cxx_strdup( obj->getId().c_str() );
    j->set_data( id_storage );

    for (m=obj->begin(); m!=obj->end(); ++m) {
	o=(*m);
	if (o==NULL) continue;
	displaySubtree( j , o );
    }
}

static void TraverseTreeAndFreeData(Gtk::CTree_Helpers::RowList rl)
{
    using namespace Gtk::CTree_Helpers;

    RowList::iterator k;
    char*      id_storage;

    for (k=rl.begin(); k!=rl.end(); ++k) {
	id_storage= (char*)(k->get_data());
	if (id_storage!=NULL) {
	    delete id_storage;
	    k->set_data(NULL);
	}
	RowList srl = k->subtree();
	TraverseTreeAndFreeData(srl);
    }
}

void ObjectTree::rebuildTree()
{
    string cur=get_current_selection();

    using namespace Gtk::CTree_Helpers;

    TraverseTreeAndFreeData(rows());
    clear();

    vector<FWObject*>::iterator m;
    RowList::iterator j;

    j=--rows().end();
    for (m=FWObjectDatabase::ScratchPad->begin(); 
	 m!=FWObjectDatabase::ScratchPad->end(); 
	 ++m) {
	displaySubtree( j , (*m) );
    }

    rebuild_scheduled=false;

    if (cur!="")
	main_w->schedule_open_object(cur);
}

void ObjectTree::source_drag_begin(GdkDragContext     *context)
{
    drag_object=last_clicked_object;

    GdkPixmap *pmap;
    GdkBitmap *mask;
    Gdk_Colormap cmap ( get_colormap () );
    string    icn=Resources::global_res->getIconPath("Drag");

    if (gdk_imlib_load_file_to_pixmap((char*)(icn.c_str()), &pmap, &mask)){

	Gdk_Pixmap drag_icon=Gdk_Pixmap(pmap);
	Gdk_Bitmap drag_mask=Gdk_Bitmap(mask);

	drag_source_set_icon(cmap, drag_icon, drag_mask);

	gdk_imlib_free_pixmap(pmap);
    }
}


void ObjectTree::source_drag_data_get  ( GdkDragContext     *context,
					  GtkSelectionData   *selection_data,
					  guint               info,
					  guint32             time )
{
  const guchar    *idptr;
  
  strcpy(drag_obj_id , drag_object.c_str() );
  idptr= (const guchar*)drag_obj_id;

  gtk_selection_data_set (selection_data,
			  gdk_atom_intern("ObjectTree",TRUE),
			  8,
			  (const guchar*)drag_obj_id,
			  strlen(drag_obj_id)+1 );
}

void ObjectTree::source_drag_data_delete ( GdkDragContext     *context )
{
}



void ObjectTree::selection_made(gint row,gint col,GdkEvent  *ev)
{
    Gtk::CTree_Helpers::RowList          rl=rows();
    string     id=(char*)(rl[row].get_data());

    if (rl.empty()) return;  

    if (id!=get_current_selection()) {
	if (  main_w->safe_to_close_right_pane() ) {
	    main_w->schedule_open_object(id);
	    selected_object=id;
	    Gtk::CList::moveto(row,col);
	}
	else  /* restore previous selection */
	    selectObject( get_current_selection() );
    }
}


gint ObjectTree::on_button_press_event(GdkEventButton *ev)
{
    gint              r,c;
    char             *oid;

    get_selection_info( int(ev->x) , int(ev->y) , &r , &c );

    if (r<0 || r>get_rows()) return(0);

    oid=(char*)(row(r).get_data());
    if (oid==NULL) return(0);

    if (ev->button != 1) selectRow(r);

    last_clicked_object=oid;

    return(0);
}

gint ObjectTree::on_button_release_event(GdkEventButton *ev)
{
    gint              r,c;
    char             *oid;

    get_selection_info( int(ev->x) , int(ev->y) , &r , &c );

    if (r<0 || r>get_rows()) return(0);

    oid=(char*)(row(r).get_data());
    if (oid==NULL) return(0);

    if (ev->type == GDK_BUTTON_RELEASE && ev->button==3  ) {
	TreePopupMenu();
    }

    return(0);
}


string ObjectTree::get_current_selection()
{
  return( selected_object );
}

FWObject*  ObjectTree::newObject(const string& where_id , FWObject *o)
{
    FWObject *s=FWObjectDatabase::create( o->getTypeName() );

    FWObjectDatabase::db->addAt(where_id,o);
    *s = *o;
    FWObjectDatabase::ScratchPad->addAt(where_id,s);

    schedule_rebuild_and_show(o->getId());

    return o;
}


FWObject*  ObjectTree::newObject(const string& where_id , const string& type)
{
    FWObject *o=FWObjectDatabase::create(type);
    newObject(where_id,o);
    return o;
}


void  ObjectTree::on_new_fw()
{
    if ( main_w->safe_to_close_right_pane() ) {
	FWObject *fw=FWObjectDatabase::create("Firewall");
	fw->add( new FirewallOptions() );
	fw->add( new Policy() );
	fw->add( new NAT() );
	newObject( FWObjectDatabase::db->std.FirewallsId , fw );
    }
}

void  ObjectTree::on_new_host()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.HostsId , "Host" );
}

void  ObjectTree::on_new_net()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.NetworksId , "Network" );
}

void  ObjectTree::on_new_obj_group()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.ObjectGroupsId,"ObjectGroup");
}

void  ObjectTree::on_new_ip()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject(FWObjectDatabase::db->std.IPServicesId, "IPService");
}

void  ObjectTree::on_new_icmp()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.ICMPServicesId, "ICMPService");
}

void  ObjectTree::on_new_tcp()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.TCPServicesId, "TCPService");
}

void  ObjectTree::on_new_udp()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.UDPServicesId, "UDPService");
}

void  ObjectTree::on_new_srv_group()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.ServiceGroupsId, "ServiceGroup");
}

void  ObjectTree::on_new_time()
{
    if ( main_w->safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.TimeId, "Interval");
}


void ObjectTree::on_cutobj()
{   
    if ( main_w->safe_to_close_right_pane() ) {

	main_w->ClearRightPane(true);

	string id=get_current_selection();
	FWObject *obj=FWObjectDatabase::db->getById( id, true);
	if ( ! obj->isSystem() ) {
	    FWObject *p=obj->getParent();

	    FWObjectClipboard::obj_clipboard->putObject(obj);
	    FWObjectDatabase::db->removeAllInstances(obj);

	    obj=FWObjectDatabase::ScratchPad->getById( id , true);
	    FWObjectDatabase::ScratchPad->removeAllInstances(obj);

	    clearSelection();

	    schedule_rebuild_and_show( p->getId() );
	}
    }
}

void ObjectTree::on_duplicate()
{   
    string id=get_current_selection();

    if ( main_w->safe_to_close_right_pane() ) {

	FWObject *obj=FWObjectDatabase::db->getById( id , true);
	if (obj) {
	    FWObject *parent=obj->getParent();
	    if (parent) 
		pasteTo(parent,obj);
	}
    }
}

void ObjectTree::on_copyobj()
{   
    string id=get_current_selection();
    if ( main_w->safe_to_close_right_pane() ) {
	FWObject *obj=FWObjectDatabase::db->getById( id , true);
	if (obj && ! obj->isSystem() ) 
	    FWObjectClipboard::obj_clipboard->putObject( id );
    }
}

void  ObjectTree::on_pasteobj()
{
    if ( main_w->safe_to_close_right_pane() ) {

	FWObject *obj=FWObjectClipboard::obj_clipboard->getObject();
	if (obj==NULL) return;

	FWObject *cursel=
	    FWObjectDatabase::db->getById( get_current_selection() , true);
	if (cursel==NULL) return;

	pasteTo(cursel,obj);
    }
}

void ObjectTree::pasteTo(FWObject *target, FWObject *obj)
{
    if (! target->isSystem() &&
	( target->getTypeName()==ObjectGroup::TYPENAME ||
	  target->getTypeName()==ServiceGroup::TYPENAME ) ) {

	BuiltinDialog *b=main_w->getRightPaneDialog();
	if (b) {
	    GroupDialog   *gd=dynamic_cast<GroupDialog*>(b->getDialog());
	    if (gd) {
		gd->addObject(obj);
	    }
	}
	return;
    }

    FWObject *no=FWObjectDatabase::create(obj->getTypeName());
    g_assert(no!=NULL);

    if (target->validateChild(no) ) {
	no->duplicate(*obj);
	newObject( target->getId() , no );
    } else 
	delete no;
}

void ObjectTree::on_sort()
{
    FWObject *o;
    string id=get_current_selection();

    o=FWObjectDatabase::db->getById( id , true ) ;
    assert(o!=NULL);
    o->sortChildren();

    o=FWObjectDatabase::ScratchPad->getById( id , true ) ;
    assert(o!=NULL);
    o->sortChildren();

    schedule_rebuild_and_show(id);
}

void ObjectTree::TreePopupMenu()
{
    FWObject *obj=
	FWObjectDatabase::ScratchPad->getById( get_current_selection() , 
					       true);
    assert(obj!=NULL);

    if (obj->getTypeName()==Policy::TYPENAME ||
	obj->getTypeName()==NAT::TYPENAME ) return;

    if ( obj->isSystem() ) {

	const char  *menu_items[] ={ "Sort subtree", "Paste", NULL};
	int          menu_choice;
	gen_popup_menu *gpm;

	gpm=new gen_popup_menu( menu_items );

	gpm->popup(0,0);
	menu_choice=gpm->run();
	delete gpm;

	switch (menu_choice) {
	case 0: on_sort();	  break;
	case 1:	on_pasteobj();	  break;
	}

    } else {

	const char  *menu_items[] ={ "Duplicate", "Copy","Cut","Paste",NULL};
	int          menu_choice;
	gen_popup_menu *gpm;

	gpm=new gen_popup_menu( menu_items );

	Gtk::Widget *itm;
	itm=find_widget("Paste",gpm);   
	if (itm) itm->set_sensitive(
	    (obj->getTypeName()==ObjectGroup::TYPENAME ||
	     obj->getTypeName()==ServiceGroup::TYPENAME )  );

	gpm->popup(0,0);
	menu_choice=gpm->run();
	delete gpm;

	switch (menu_choice) {
	case 0:  on_duplicate();  break;
	case 1:  on_copyobj();    break;
	case 2:  on_cutobj();	  break;
	case 3:  on_pasteobj();	  break;
	}
    }
}






