/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyList.cc,v 1.13 2001/07/22 06:13:19 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "glademm_support.hh"
#include "helpers.hh"
#include "BuiltinDialog.hh"
#include "gen_popup_menu.hh"

#include "PolicyListItem.hh"
#include "PolicyListElement.hh"
#include "PolicyList.hh"


#include <gdk_imlib.h>


PolicyList::PolicyList(RuleSet *p) : Gtk::Table(1,2)
{
    ruleset=p;
    current_selected_row=0;
    current_selected=0;
    rebuild=false;
    width=height=0;

    set_flags( GTK_CAN_FOCUS );

    realize.connect(SigC::slot(this, &PolicyList::on_realize_event));
    size_allocate.connect(SigC::slot(this, &PolicyList::on_size_allocate));

    button_release_event.connect_after(
	SigC::slot(this,&PolicyList::on_button_release_event));

    show();
}

PolicyList::~PolicyList()
{
//  Clear();


  /*
   *  What follows is a workaround for the crash which seems to be extremely 
   *  difficult to find. Program crashes in the following spot:


#0  0x403066e0 in gtk_bin_remove (container=0x81c0dc8, child=0x1) at gtkbin.c:244
#1  0x4013e4c0 in Gtk::Container::remove_impl (this=0x81c3d10, p0=@0x81b9784) at container.cc:225
#2  0x4013e47e in Gtk::Container_Class::remove_callback (p0=0x81c0dc8, p1=0x81bb660) at container.cc:214
#3  0x40367279 in gtk_marshal_NONE__POINTER (object=0x81c0dc8, 
    func=0x4013e3e8 <Gtk::Container_Class::remove_callback(_GtkContainer *, _GtkWidget *)>, func_data=0x0, args=0xbffff598) at gtkmarshal.c:193
#4  0x4039204b in gtk_signal_real_emit (object=0x81c0dc8, signal_id=65, params=0xbffff598) at gtksignal.c:1440
#5  0x40390420 in gtk_signal_emit (object=0x81c0dc8, signal_id=65) at gtksignal.c:552
#6  0x40329268 in gtk_container_remove (container=0x81c0dc8, widget=0x81bb660) at gtkcontainer.c:732
#7  0x403c6cfe in gtk_widget_shutdown (object=0x81bb660) at gtkwidget.c:4380
#8  0x403770cb in gtk_object_destroy (object=0x81bb660) at gtkobject.c:246
#9  0x40151aac in Gtk::Object::destroy_ (this=0x81b9750) at object.cc:31
#10 0x4015e40d in Gtk::Table::~Table (this=0x81b9750, __in_chrg=0) at table.cc:136
#11 0x8094535 in PolicyList::~PolicyList (this=0x81b9750, __in_chrg=3) at PolicyList.cc:255

    Something seems to go wrong between frames 2 and 1 or 0. Disconnecting from
    parent here helps.

   */
//  unparent();

}

/*
 *   We keep track of our physical size
 */
void PolicyList::on_size_allocate(GtkAllocation *all)
{
    x=all->x;
    y=all->y;
    width=all->width;
    height=all->height;
}

gint PolicyList::on_button_release_event(GdkEventButton *ev)
{
    if ( ev->type == GDK_BUTTON_RELEASE && ev->button==3 ) {

	gen_popup_menu         *gpm;

	const char *menu_items0[] = 
	{ "Insert rule at the top",
	  "Append rule at the bottom",
	  NULL } ;

	if ( ruleset->size()==0 ) {
	    menu_items0[0]="Insert rule";
	    menu_items0[1]=NULL;
	}

	gpm=new gen_popup_menu( menu_items0 );
	gpm->popup(0,0);
	gint menu_choice=gpm->run();
	delete gpm;

	switch (menu_choice) {
	case 0:
	    insertRuleAtTop();
	    break;
	case 1:
	    appendRuleAtBottom();
	    break;
	}
    }
    return( true );
}


void PolicyList::move_focus(GtkDirectionType dir)
{
  gint r,c;

  if (current_selected!=0) {

    //    current_selected->deselect();

    r=current_selected->row;
    c=current_selected->col;

    switch (dir ) {
    case GTK_DIR_UP:     r-=2;    break;
    case GTK_DIR_DOWN:   r+=2;    break;
    case GTK_DIR_LEFT:   c-=2;    break;
    case GTK_DIR_RIGHT:  c+=2;    break;
    default:         return;
    }
    select_element(r,c);
  } else
    select_element(0,0);
}

void PolicyList::select_element(gint r,gint c)
{
  PolicyListElement *pe;
  if ( (pe=get_element(r,c))!=0 )     select(pe);
}

void PolicyList::select(PolicyListElement *pe)
{
  if ( pe!=0 )  {
    if ( current_selected!=0) current_selected->deselect();
    if (pe->row!=current_selected_row)    activate_row(pe->row);
    pe->_select();
    current_selected=pe;
  }
}

void PolicyList::request_focus()
{
  if (!has_focus()) {
    grab_focus();
    draw_focus();
  }
}


void PolicyList::activate_row(gint row)
{
  PolicyListElement *pe;

  deactivate_row( current_selected_row );
  current_selected_row=row;

  vector<Gtk::Widget*>::iterator k;
  if (!elements.empty()) {
    for (k=elements.begin(); k!=elements.end(); ++k) {
// class PolicyListElement : public  Gtk::VBox {
      if (Gtk::VBox::isA( (*k) )) {   
	pe=(PolicyListElement*)(*k);
	if (pe->get_row()==row) pe->activate();
      }
    }
  }
}

void PolicyList::activate_rule(gint r)
{
  activate_row( get_row_by_rule_num(r) );
}

void PolicyList::deactivate_row(gint row)
{
  PolicyListElement *pe;

  vector<Gtk::Widget*>::iterator k;
  if (!elements.empty()) {
    for (k=elements.begin(); k!=elements.end(); ++k) {
// class PolicyListElement : public  Gtk::VBox {
      if (Gtk::VBox::isA( (*k) )) {   
	pe=(PolicyListElement*)(*k);
	if (pe->get_row()==row) pe->deactivate();
      }
    }
  }
}

void PolicyList::deactivate_rule(gint r)
{
  deactivate_row( get_row_by_rule_num(r) );
}


void PolicyList::schedule_rebuild(int r)
{
    activate_r=r;
    Gtk::Main::idle.connect(slot(this,&PolicyList::rebuild_when_idle));
}

gint PolicyList::rebuild_when_idle()
{
    Build();
    activate_rule(activate_r);
    return false;
}


void PolicyList::on_realize_event()
{
}


void PolicyList::set_data_changed_flag(bool flag)
{
  BuiltinDialog *b= (BuiltinDialog *)find_widget("BuiltinDialog",this);
  b->data_changed_flag(flag);
}

void PolicyList::__print_children()
{
  vector<Gtk::Widget*>::iterator k;
  if (!elements.empty()) {
    cerr << "  Table size: " << elements.size() << endl;

    cerr << "  Table: " << this << " gtkobj=" << gtkobj() <<  endl;
    for (k=elements.begin(); k!=elements.end(); ++k)
      print_widget_tree( 0 , (*k) );
    cerr << endl;
  }
  /*
  Gtk::Table_Helpers::TableList tbl=children();
  Gtk::Table_Helpers::TableList::iterator m;
  for (m=tbl.begin(); m!=tbl.end(); ++m) {
    cerr << "Child: " << (*m) << endl;
  }
  cerr << endl;
  */
}

void PolicyList::Clear()
{
  vector<Gtk::Widget*>::iterator k;

  if (!elements.empty()) {
    for (k=elements.begin(); k!=elements.end(); ++k) {
      Gtk::Widget *w=(*k);
      g_assert( w!=NULL );
//      w->destroy();
      w->hide();
      remove(*w);
    }
  }
  elements.clear();
  current_selected=0;
}

Gtk::Widget* PolicyList::_register_child(Gtk::Widget *c)
{
  elements.push_back(c);
  return c;
}


/************************************************************
 *                                                          *
 *     Rebuild the list                                     *
 *                                                          *
 ************************************************************/

void PolicyList::Build()
{
    gint                row, col, rule_n;
    gint                tbl_size;
    Gtk::Button        *btn;
    string              s1;
    PolicyListElement  *pe;


    hide();

    //
    // destroy current list elements
    //

    Clear();

    //
    //   Cosntruct new one
    //
    if (ruleset==NULL) {
	return;
    } else {
	tbl_size=ruleset->size()*2+1 +1;
    }


    resize( tbl_size , (ruleset->get_n_columns()+1)*2 );

    row=0;
    col=0;

    FWObject *o;

    btn= manage(new Gtk::Button( "Num" ));
    btn->set_name("Num");
    btn->set_state(GTK_STATE_NORMAL);
    attach( *btn , col , col+1 , row , row+1 , GTK_FILL, 0, 0, 1);
    _register_child(btn);

    btn->show();

    col++;
    _add_vseparator(row,col);

    col++;


    list<const  char*> titles;
    list<const char*>::iterator ci;

    ruleset->getTitles(titles);

    for (ci=titles.begin(); ci!=titles.end(); ci++) {
	if ( (*ci) ) {

	    btn= manage(new Gtk::Button( (*ci) ));

	    btn->set_state(GTK_STATE_NORMAL);
	    attach( *btn , col , col+1 , row , row+1 , 
		    GTK_FILL|GTK_EXPAND, 0, 0, 1);
	    _register_child(btn);

	    btn->show();

	    col++;
	    _add_vseparator(row,col);
	    
	    col++;
	}
    }

    row++;
    _add_hseparator(row,col);

    row++;

    rule_n=0;
    /*
     *   this outer loop choses policy rule
     */
    vector<FWObject*>::iterator m;
    for (m=ruleset->begin(); m!=ruleset->end(); ++m,++rule_n ) {
//	if ( (o=(*m))!=NULL && o->getTypeName()=="RULE") {

	o=(*m);
	if (Rule::cast(o)!=NULL) {
	    Rule *rule=Rule::cast(o);

	    col=0;

	    pe=manage(new PolicyListRuleNum(row,col,rule_n,rule->isDisabled()));
	    pe->set_user_data(this);

	    attach( *pe , col , col+1 , row , row+1 , 
		    GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
	    _register_child(pe);

	    col++;
	    _add_vseparator(row,col);
	    col++;


/*
 *  this inner loop choses rule element
 */


	    list<const char*> elements;

	    ruleset->getElements(elements);

	    for (ci=elements.begin(); ci!=elements.end(); ci++) {

		pe=manage(ruleset->createPolicyListElement(row,col,(*ci),rule));

		pe->set_user_data(this);

		attach( *pe , col , col+1 , row , row+1 , 
			GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
		_register_child(pe);
		col++;
		_add_vseparator(row,col);
		col++;
	    }


	    row++;
	    _add_hseparator(row,col);
	    
	    row++;
	}
    }

/*
 *  this adds row of event boxes at the very bottom - EXPERIMENTAL
 *
 */

    Gtk::EventBox *eb = manage(new Gtk::EventBox());
    attach( *eb , 0 , (ruleset->get_n_columns()+1)*2 , row , row+1 , 
	    GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 1);
    _register_child(eb);
    eb->show();


    show_all();
	    
    activate_rule(current_selected_row);
    select_element(current_selected_row,0);
    
    request_focus();
}

void PolicyList::_add_hseparator(gint row,gint col)
{
  Gtk::HSeparator *sep= manage(new Gtk::HSeparator());
  attach( *sep , 0 , col , row , row+1 , GTK_FILL, GTK_FILL, 0, 0);
  _register_child(sep);
}

void PolicyList::_add_vseparator(gint row,gint col)
{
  Gtk::VSeparator *sep= manage(new Gtk::VSeparator());
  attach( *sep , col , col+1 , row , row+1 , 0, GTK_FILL, 0, 0);
  _register_child(sep);
}


gint PolicyList::get_row_by_rule_num(gint rule_n)
{
  return( rule_n*2+2 );
}

gint PolicyList::get_rule_by_row(gint row_n)
{
  return( (row_n-2)/2 );
}



void PolicyList::insertRuleAtTop()
{
    if ( ruleset->insertRuleAtTop()!=NULL ) {
	schedule_rebuild(0);
	set_data_changed_flag(true);
    }
}

void PolicyList::insertRuleBefore(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->insertRuleBefore(nr)!=NULL ) {
	schedule_rebuild(nr);
	set_data_changed_flag(true);
    }
}

void PolicyList::appendRuleAfter(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->appendRuleAfter( nr )!=NULL ) {
	schedule_rebuild(nr);
	set_data_changed_flag(true);
    }
}

void PolicyList::appendRuleAtBottom()
{
    if ( ruleset->appendRuleAtBottom()!=NULL ) {
	schedule_rebuild(ruleset->getRuleSetSize() - 1);
	set_data_changed_flag(true);
    }
}

void PolicyList::delRule(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->deleteRule(nr) ) {
	schedule_rebuild(nr-1);
	set_data_changed_flag(true);
    }
}

void PolicyList::moveRuleUp(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->moveRuleUp(nr) ) {
	schedule_rebuild(nr);
	set_data_changed_flag(true);
    }
}

void PolicyList::moveRuleDown(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->moveRuleDown(nr) ) {
	schedule_rebuild(nr);
	set_data_changed_flag(true);
    }
}

void PolicyList::disableRule(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->disableRule(nr) ) {
	schedule_rebuild(nr);
	set_data_changed_flag(true);
    }
}

void PolicyList::enableRule(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->enableRule(nr) ) {
	schedule_rebuild(nr);
	set_data_changed_flag(true);
    }
}

bool  PolicyList::isRuleDisabled(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    return ( ruleset->isRuleDisabled(nr) );
}

PolicyListElement* PolicyList::get_element(gint r,gint c)
{
  PolicyListElement    *pe;
  vector<Gtk::Widget*>::iterator k;
  if (!elements.empty()) {
    for (k=elements.begin(); k!=elements.end(); ++k) {
// class PolicyListElement : public  Gtk::VBox {
      if (Gtk::VBox::isA( (*k) )) {   
	pe=(PolicyListElement*)(*k);
	if (pe->get_row()==r && pe->get_col()==c) 
	  return(pe);
      }
    }
  }
  return(0);
}



