/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyListElement.cc,v 1.33 2001/06/29 08:40:13 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <gdk_imlib.h>


#include "glademm_support.hh"
#include "helpers.hh"
#include "FWObjectDatabase.hh"
#include "BuiltinDialog.hh"
#include "gen_popup_menu.hh"
#include "Host.hh"
#include "Firewall.hh"
#include "Resources.hh"
#include "Preferences.hh"
#include "CommentDialog.hh"
#include "RuleOptionsDialog.hh"

#include "FWObjectReference.hh"
#include "FWServiceReference.hh"
#include "FWIntervalReference.hh"

#include "PolicyListElement.hh"
#include "PolicyListItem.hh"
#include "PolicyList.hh"

#include "FWOptions.hh"

static string obj_n; // to store name of the object to be opened in editor


/*******************************************************************/

void PolicyListElement::constructor(gint r,gint c)
{
    row=r;
    col=c;
//    any=false;
    active=false;
    selected=false;
    width=height=0;

    size_allocate.connect(SigC::slot(this, 
				     &PolicyListElement::on_size_allocate));

    key_press_event.connect(
	SigC::slot((PolicyListElement*)this, 
		   &PolicyListElement::on_key_press_event));

    
    set_flags( GTK_CAN_FOCUS );

    set_name("PolicyListElement");

    drag_dest_set ( GTK_DEST_DEFAULT_ALL,
		    target_table, n_targets , 
		    static_cast<GdkDragAction>(GDK_ACTION_COPY) );
    
    drag_data_received.connect(slot( this, 
				 &PolicyListElement::obj_drag_data_received));
}

PolicyListElement::PolicyListElement(gint row,gint col)
{

    constructor(row,col);

    rel=NULL;

    show_all();
}

PolicyListElement::PolicyListElement(gint r,gint c, RuleElement *re)
{
    PolicyListItem    *itm;
    FWObject          *o, *oref;

    rel=re;

    constructor(r,c);

    change_any_to_orig=false;
    if (RuleElementTSrc::isA(re) ||
	RuleElementTDst::isA(re) ||
	RuleElementTSrv::isA(re) )  change_any_to_orig=true;
    

//    any=false;
    vector<FWObject*>::iterator m;
    for (m = rel->begin(); m!= rel->end(); ++m) {
	o=(*m);
	oref=NULL;
	if (FWObjectReference::isA(o) || FWServiceReference::isA(o) ||
	    FWIntervalReference::isA(o) )
	    oref=(FWReference::cast(o))->getPointer();

	if (oref!=NULL) {
	    itm= manage(new PolicyListObjectItem(oref, 
						 rel->getNeg(), 
						 change_any_to_orig) );
	    pack_start( *itm , true , true , 0 );
	}

    }

    selected_item=children().begin();
    show_all();
}

void PolicyListElement::set_data_changed_flag(bool f)
{
    PolicyList *pl=(PolicyList*)get_user_data();
    pl->set_data_changed_flag( f );
}

/*
 *   We keep track of our physical size
 */
void PolicyListElement::on_size_allocate(GtkAllocation *all)
{
    x=all->x;
    y=all->y;
    width=all->width;
    height=all->height;
}



void PolicyListElement::remove_item_from_policy(PolicyListItem* pi)
{
    assert (rel!=NULL);

    if (pi==NULL) return;
    if (rel->isAny()) return;

    FWObject *obj     = pi->getObject();

    remove(*pi);   // do not need to delete because it is managed
    
    if (obj) {
	rel->removeRef(obj);

	if (rel->getChildrenCount()==0) {   // there is nothing left

	    obj=obj->getRoot()->getById( rel->getAnyElementId() , true );
	    if (obj)  {
		rel->addRef(obj);
		PolicyListItem* itm= 
		    manage(new PolicyListObjectItem( obj, 
						     rel->getNeg(), 
						     change_any_to_orig) );
		pack_start( *itm , true , true , 0 );
		itm->show();
		selected_item=children().begin();
		select();
	    }

	    if (rel->getNeg())  toggleNeg();
	}	

	selected_item=children().begin();

	set_data_changed_flag(true);
    }
}

bool PolicyListElement::add_item_to_policy(FWObject *obj)
{
    if (rel==NULL) return(false);
    if (! rel->validateChild(obj) ) return(false);
    if (rel->getById(obj->getId())!=NULL) return(false); // no duplicates 
    
    if (rel->isAny()) {
	// element contains "Any". Remove it before adding new item
	// there should be only one child, too
	//
	// this operation invalidates  selected_item

	Gtk::Box_Helpers::BoxList::iterator bl_i;
	Gtk::Box_Helpers::Child    *ch;
	Gtk::Widget                *cc;
	bl_i=children().begin();
	if ( (ch=*bl_i)!=0 ) {
	    cc=ch->get_widget();
	    remove( *cc );
	}

	rel->clearChildren();
    }


    rel->addRef(obj);

    deselect();

    PolicyListItem* itm= 
	manage(new PolicyListObjectItem( obj, 
					 rel->getNeg(),change_any_to_orig) );
    pack_start( *itm , true , true , 0 );
    itm->show();
	
    selected_item=children().end();
    selected_item--;
	
    select();
	
    set_data_changed_flag(true);

    return(true);
}


void PolicyListElement::toggleNeg()
{
    rel->toggleNeg();

    Gtk::Box_Helpers::BoxList::iterator bl_i;
    for (bl_i=children().begin(); bl_i!=children().end(); bl_i++) {
	((PolicyListItem*)(*bl_i)->get_widget()) ->setNeg( rel->getNeg() );
    }

    set_data_changed_flag(true);
}

void PolicyListElement::activate()
{
    if (!active) {
	active=true;
	Gtk::Box_Helpers::BoxList::iterator bl_i;
	for (bl_i=children().begin(); bl_i!=children().end(); bl_i++) {
	    ((PolicyListItem*)(*bl_i)->get_widget()) ->activate();
	}
    }
}

void PolicyListElement::deactivate()
{
    if (active) {
	active=false;
	selected=false;
	Gtk::Box_Helpers::BoxList::iterator bl_i;
	for (bl_i=children().begin(); bl_i!=children().end(); bl_i++) {
	    ((PolicyListItem*)(*bl_i)->get_widget()) ->deactivate();
	}
    }
}

void PolicyListElement::_select()
{
    activate();
    if ( *selected_item !=NULL)
	((PolicyListItem*)(*selected_item)->get_widget()) ->select();
    selected=true;
    request_focus();
}


void PolicyListElement::select()
{
    PolicyList* pp=(PolicyList*)get_parent();
    pp->select(this);
}

void PolicyListElement::select(PolicyListItem *pi)
{
    if ( *selected_item !=NULL)
	((PolicyListItem*)(*selected_item)->get_widget()) ->deselect();
    selected_item=children().find(*pi);
    select();
}

void PolicyListElement::select(Gtk::Box_Helpers::BoxList::iterator bl_i)
{
    if ( *selected_item !=NULL)
	((PolicyListItem*)(*selected_item)->get_widget()) ->deselect();
    selected_item=bl_i;
    select();
}

void PolicyListElement::deselect()
{
    if ( *selected_item !=NULL)
	((PolicyListItem*)(*selected_item)->get_widget()) ->deselect();
    selected=false;
}


void PolicyListElement::request_focus()
{
    if (!has_focus()) {
	grab_focus();
	draw_focus();
    }
}

void PolicyListElement::redraw()
{
    Gtk::Box_Helpers::BoxList::iterator bl_i;
    for (bl_i=children().begin(); bl_i!=children().end(); bl_i++) {
	((PolicyListItem*)(*bl_i)->get_widget()) ->redraw();
    }
}


void PolicyListElement::obj_drag_data_received ( GdkDragContext    *context,
						 gint               x,
						 gint               y,
						 GtkSelectionData  *data,
						 guint              info,
						 guint32            time)
{
    void           *ptr;
    FWObject       *obj;
    PolicyListItem *itm=NULL;
    char           obj_id[64];

    Gdk_DragContext gdc ( context );
    Gtk::Widget::drag_finish ( gdc , false, false, time );

    if ( data->length >= 0 ) {
/*
 *   ObjectTree sends FWObject* via drag&drop mechanism, while
 *   PolicyListItem sends itself (PolicyListItem*, that is).  We will
 *   differentiate them using data->format here. This allows for
 *   different actions to be performed depending on where dragged
 *   object has been taken from
 *
 *   switch (data->format) {
 *
 *     case 6:    moving the whole rule, ptr is PolicyListItem*
 *     case 7:    moving object, ptr is PolicyListItem*
 *     case 8:    moving object, ptr is char* ( its Id )
 *    
 */
	obj=NULL;

	if (data->format==7) {
	    memcpy(&ptr,data->data,data->length);
	    itm=(PolicyListItem*)ptr;
	    obj=itm->getObject();
	}

	if (data->format==8) {
	    memcpy(obj_id,(char*)data->data,data->length);
	    obj=rel->getRoot()->getById( obj_id , true );
	}

	if (obj==NULL) return;
	
	if ( add_item_to_policy(obj) ) {
/* object has successfully been added */
	    if (itm) {
		PolicyListElement* pe=(PolicyListElement*)itm->get_parent();
		if (pe)
		    pe->remove_item_from_policy(itm);
	    }
	}
    }
}

gint PolicyListElement::on_key_press_event(GdkEventKey *ev)
{
  PolicyList* pp=(PolicyList*)get_parent();
  Gtk::Box_Helpers::BoxList::iterator bl_i;

  if ( *selected_item ==NULL) goto another_element;

  bl_i=selected_item;

  switch (ev->keyval) {

  case GDK_space:
      popup_menu( (PolicyListItem*)(*selected_item)->get_widget() );
      break;

  case GDK_Up:
      bl_i--;
      if ( (*bl_i)!=0)  select(bl_i);
      else goto another_element;
      break;

  case GDK_Down:
      bl_i++;
      if (bl_i!=children().end()) select(bl_i);
      else goto another_element;
      break;

  case GDK_F5:
  {
      cerr << "------- PolicyListElement dump:\n";
      Gtk::Box_Helpers::BoxList::iterator bl_i;
      Gtk::Box_Helpers::Child    *ch;
      Gtk::Widget                *cc;
      for (bl_i=children().begin(); bl_i!=children().end(); ++bl_i) {
	  ch= *bl_i;
	  cc=ch->get_widget();
	  cerr << " child widget: " << cc << endl;
      }
      rel->dump(true,true);

      break;
  }

  default:
	
another_element:

      switch (ev->keyval ) {
      case GDK_Up:     pp->move_focus(GTK_DIR_UP);     break;
      case GDK_Down:   pp->move_focus(GTK_DIR_DOWN);   break;
      case GDK_Left:   pp->move_focus(GTK_DIR_LEFT);   break;
      case GDK_Right:  pp->move_focus(GTK_DIR_RIGHT);  break;
      default:         return(false);
      }
  }
  gtk_signal_emit_stop_by_name( GTK_OBJECT( gtkobj() ),
                                  "key_press_event" );
  return(true);
}


void PolicyListElement::popup_menu(PolicyListItem *itm)
{
    FWObject               *obj;
    gen_popup_menu         *gpm;
    PolicyList             *pl=(PolicyList*)get_user_data();

    g_assert(rel!=NULL);

    obj=itm->getObject();

    const char *menu_items0[] = 
    { "Edit","","Copy","Cut","Paste","","Negate",NULL } ;
    gpm=new gen_popup_menu( menu_items0 );

    if ( rel->isAny() ) {
	Gtk::Widget *itm;
	itm=find_widget("Edit",gpm);   if (itm) itm->set_sensitive(false);
	itm=find_widget("Negate",gpm); if (itm) itm->set_sensitive(false);
	itm=find_widget("Copy",gpm);   if (itm) itm->set_sensitive(false);
	itm=find_widget("Cut",gpm);    if (itm) itm->set_sensitive(false);
    }

    gpm->popup(0,0);
    gint menu_choice=gpm->run();
    delete gpm;


    switch (menu_choice) {
    case 0:    // edit
/*
 *  pass object via clipboard to avoid passing reference to the stack
 *
 *  do not edit "Any"
 */
	if ( obj!=NULL &&  ! rel->isAny() ) {
	    FWObjectClipboard::obj_clipboard->putObject( obj );
// open_item is a signal
	    pl->open_item( FWObjectClipboard::obj_clipboard->getObject() );
	}
	return;
    case 2:    // copy
	if ( obj!=NULL && ! rel->isAny() ) {
	    FWObjectClipboard::obj_clipboard->putObject( obj );
	}
	return;
    case 3:    // cut
	if ( obj!=NULL && ! rel->isAny() ) { 
	    FWObjectClipboard::obj_clipboard->putObject( obj );
	    remove_item_from_policy(itm);
	}
	return;
    case 4:    // paste
	if ( (obj=FWObjectClipboard::obj_clipboard->getObject())!=NULL ) {
	    add_item_to_policy(obj);
	    set_data_changed_flag(true);
	}
	return;
    case 6:    // negate
	toggleNeg();   
	return;
    }

}

/*******************************************************************/

static int last_drag_motion_x=-1;


PolicyListRuleNum::PolicyListRuleNum(gint r,gint c,gint rule_n,bool neg) :
    PolicyListElement(r,c)
{
    PolicyListRuleNumItem    *itm;
    char  str[64];

    sprintf(str,"%02d",rule_n);
    itm= manage(new PolicyListRuleNumItem( str , neg ));
    pack_start( *itm , true , true , 0 );

    selected_item=children().begin();
    set_name("PolicyListRuleNum");

    drag_data_received.connect(slot( this, &PolicyListRuleNum::obj_drag_data_received ) );

    last_drag_motion_x=-1;
}

int PolicyListRuleNum::on_drag_motion( GdkDragContext *context,
				       gint x,gint y,guint time)
{
    if (last_drag_motion_x == -1) last_drag_motion_x=x;
    
    if (last_drag_motion_x!=x) {

    }
    return false;
}

gint PolicyListRuleNum::on_motion_notify_event(GdkEventMotion* ev)
{
    return(0);
}

void PolicyListRuleNum::obj_drag_data_received ( GdkDragContext    *context,
						 gint               x,
						 gint               y,
						 GtkSelectionData  *data,
						 guint              info,
						 guint32            time)
{
    PolicyListItem *itm=NULL;

    Gdk_DragContext gdc ( context );
    Gtk::Widget::drag_finish ( gdc , false, false, time );

    if ( data->length >= 0 ) {
/*
 *   ObjectTree sends FWObject* via drag&drop mechanism, while
 *   PolicyListItem sends itself (PolicyListItem*, that is).  We will
 *   differentiate them using data->format here. This allows for
 *   different actions to be performed depending on where dragged
 *   object has been taken from
 *
 *   switch (data->format) {
 *
 *     case 6:    moving the whole rule, ptr is PolicyListItem*
 *     case 7:    moving object, ptr is PolicyListItem*
 *     case 8:    moving object, ptr is FWObject*
 *    
 */
	void* ptr;
	memcpy(&ptr,data->data,data->length);

	if (data->format==6) {
	    itm=(PolicyListItem*)ptr;
	    PolicyListElement  *pe=(PolicyListElement*)(itm->get_parent());
	    gint src_rule = PolicyList::get_rule_by_row( pe->get_row() );
	    gint dst_rule = PolicyList::get_rule_by_row( get_row() );

/* rule number src_rule moves right above rule dst_rule */
	    PolicyList         *pl=(PolicyList*)get_user_data();
	    RuleSet            *pol=pl->getRuleSet();
	    pol->moveRule(src_rule,dst_rule);

	    pl->schedule_rebuild((src_rule<dst_rule)?dst_rule-1:dst_rule);

	    set_data_changed_flag(true);
	    return;
	}
    }

//    motion_event_handler.disconnect();
}

      


void PolicyListRuleNum::popup_menu(PolicyListItem *itm)
{
    gen_popup_menu         *gpm;
    PolicyList             *pl=(PolicyList*)get_user_data();
    gint rule = PolicyList::get_rule_by_row( get_row() );

    const char *menu_items0[] = 
    { "Insert rule above",
      "Add rule below",
      "Remove rule",
      "Move rule up",
      "Move rule down",
      "",
      NULL,
      NULL } ;

    if (pl->isRuleDisabled( rule ))  menu_items0[6]="Enable rule";
    else                             menu_items0[6]="Disable rule";

    gpm=new gen_popup_menu( menu_items0 );
    gpm->popup(0,0);
    gint menu_choice=gpm->run();
    delete gpm;

    switch (menu_choice) {
    case 0:
	pl->insertRuleBefore( rule );
	set_data_changed_flag(true);
	break;
    case 1:
	pl->appendRuleAfter( rule );
	set_data_changed_flag(true);
	break;
    case 2:
	pl->delRule( rule );
	set_data_changed_flag(true);
	break;
    case 3:
	pl->moveRuleUp( rule );
	set_data_changed_flag(true);
	break;
    case 4:
	pl->moveRuleDown( rule );
	set_data_changed_flag(true);
	break;
    case 6:
	if (pl->isRuleDisabled( rule )) pl->enableRule( rule );
	else                            pl->disableRule( rule );
	set_data_changed_flag(true);
    }
}

/*******************************************************************/


PolicyListRuleAction::PolicyListRuleAction(gint r,gint c, Rule *ru)
  :    PolicyListElement(r,c)
{
    rule=ru;

    string action=rule->getStr("action");
    if (action=="") action="Deny";

    string icn=	Resources::global_res->getIconPath(action);
    g_assert(icn!="");
   
    PolicyListItem  *itm= manage(new PolicyListItem( icn, action ));

    pack_start( *itm , true , true , 0 );

    selected_item=children().begin();
}
      
void PolicyListRuleAction::popup_menu(PolicyListItem *itm)
{
    gen_popup_menu         *gpm;

    const char *menu_choices[] = 
    { "Accept",
      "Deny",
      "Reject",
      NULL } ;
    gpm=new gen_popup_menu( menu_choices );
    gpm->popup(0,0);
    gint res=gpm->run();
    delete gpm;

    if (res>=0) {
	string s=menu_choices[res];
	rule->setStr("action",s);
	itm->setLabel(s);
	string icn= Resources::global_res->getIconPath(s);
	if (icn!="")      itm->setIcon(icn);
	set_data_changed_flag(true);
    }

}

/*******************************************************************/


PolicyListRuleLog::PolicyListRuleLog(gint r,gint c, Rule *ru) :
    PolicyListElement(r,c)
{
    rule=ru;

    PolicyListItem    *itm;
    bool    log=rule->getBool("log");

    if (log) {
	string icn= Resources::global_res->getIconPath("Log");
	g_assert(icn!="");
	itm= manage(new PolicyListItem( icn, "Log" ));
    }
    else 
	itm= manage(new PolicyListItem( "", "" ));

    pack_start( *itm , true , true , 0 );

    selected_item=children().begin();
}
      
void PolicyListRuleLog::popup_menu(PolicyListItem *itm)
{
    gen_popup_menu         *gpm;
    string icn;

    const char *menu_choices[] = 
    { "Log",
      "Do not log",
      NULL } ;
    gpm=new gen_popup_menu( menu_choices );
    gpm->popup(0,0);
    gint res=gpm->run();
    delete gpm;

    if (res>=0) {
	if (res==0) {
	    rule->setBool("log",true);
	    itm->setLabel("Log");
	    icn=Resources::global_res->getIconPath("Log");
	} else {
	    rule->setBool("log",false);
	    itm->setLabel("");
	    icn=Resources::global_res->getIconPath("Blank");
	}
	if (icn!="")      itm->setIcon(icn);
	set_data_changed_flag(true);
    }

}
/*******************************************************************/


PolicyListRuleComment::PolicyListRuleComment(gint r,gint c, Rule *ru) : 

    PolicyListElement(r,c)
{
    rule=ru;

    PolicyListItem    *itm;
    string   comm=rule->getStr("comment");

    itm= manage(new PolicyListCommentItem( comm ));
    pack_start( *itm , true , true , 0 );
    selected_item=children().begin();
}
      
void PolicyListRuleComment::popup_menu(PolicyListItem *itm)
{
    gen_popup_menu         *gpm;
    //PolicyList             *pl=(PolicyList*)get_user_data();

    const char *menu_choices[] = 
    { "Edit Comment",
      NULL } ;

    gpm=new gen_popup_menu( menu_choices );
    gpm->popup(0,0);
    int res=gpm->run();
    delete gpm;

    if (res==0) {
	CommentDialog *cmdlg=new CommentDialog( rule->getStr("comment") );
	string str=cmdlg->run();
	delete cmdlg;
	rule->setStr("comment", str );
	itm->setLabel( str );
	set_data_changed_flag(true);
    }
}

/*******************************************************************/

PolicyListRuleDir::PolicyListRuleDir(gint r,gint c, Rule *ru) : 

    PolicyListElement(r,c)
{
    rule=ru;

    PolicyListItem    *itm;
    string  dir=rule->getStr("direction");

    string icn=	Resources::global_res->getIconPath(dir);
   
    itm= manage(new PolicyListItem( icn, dir ));
    pack_start( *itm , true , true , 0 );
    selected_item=children().begin();
}
      
void PolicyListRuleDir::popup_menu(PolicyListItem *itm)
{
    gen_popup_menu         *gpm;

    const char *menu_choices[] = 
    { "Inbound",
      "Outbound",
      "Both",
      NULL } ;

    gpm=new gen_popup_menu( menu_choices );
    gpm->popup(0,0);
    gint res=gpm->run();
    delete gpm;

    if (res>=0) {
	rule->setStr("direction", menu_choices[res] );
	itm->setLabel( menu_choices[res] );
	string icn=Resources::global_res->getIconPath( menu_choices[res] );
	itm->setIcon(icn);

	set_data_changed_flag(true);
    }
}

/*******************************************************************/


PolicyListRuleOpt::PolicyListRuleOpt(gint r,gint c, Rule *ru) : 
    PolicyListElement(r,c)
{
    rule=ru;
    PolicyListItem  *itm;

    string icn=	Resources::global_res->getIconPath("Options");
    g_assert(icn!="");

    itm= manage(new PolicyListItem( 
	(rule->getOptionsObject()->isDefault()) ? "" : icn , 
	"" ));


    pack_start( *itm , true , true , 0 );
    selected_item=children().begin();
}
      
void PolicyListRuleOpt::popup_menu(PolicyListItem *itm)
{
    gen_popup_menu         *gpm;
    string icn;

    const char *menu_choices[] = 
    { "Modify Options",
      NULL } ;

    gpm=new gen_popup_menu( menu_choices );
    gpm->popup(0,0);
    gint res=gpm->run();
    delete gpm;

    if (res>=0) {
	RuleOptionsDialog *dlg=new RuleOptionsDialog(rule->getOptionsObject());
	dlg->run();
	delete dlg;

	if (rule->getOptionsObject()->isDefault()) 
	    icn= Resources::global_res->getIconPath("Blank");
	else
	    icn= Resources::global_res->getIconPath("Options");
	g_assert(icn!="");

	itm->setIcon(icn);

	set_data_changed_flag(true);
    }
}

/*******************************************************************/








