/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyListElement.hh,v 1.9 2001/05/26 01:16:15 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef _POLICYLISTELEMENT_HH
#define _POLICYLISTELEMENT_HH

#include <gtk--.h>

#include "FWObject.hh"
#include "Policy.hh"

class PolicyList;
class PolicyListItem;

class PolicyListElement : public  Gtk::VBox {

    friend class PolicyList;

    gint        x,y,width,height;
    gint        row,col;
    bool        active;
    bool        selected;


    gint on_key_press_event(GdkEventKey *ev);

    void obj_drag_data_received ( GdkDragContext     *context,
				  gint                x,
				  gint                y,
				  GtkSelectionData   *data,
				  guint               info,
				  guint32             time);

protected:

    void constructor(gint r,gint c);

    Gtk::Box_Helpers::BoxList::iterator selected_item;
    RuleElement           *rel;
    Rule                  *rule;
    bool                   change_any_to_orig;

    void         on_size_allocate(GtkAllocation *all);

    void set_data_changed_flag(bool f);

public:

    PolicyListElement(gint row,  gint col, RuleElement *re);
    PolicyListElement(gint row,gint col);

    void remove_item_from_policy(PolicyListItem* pi);
//  void add_item_to_policy(string name);
    bool add_item_to_policy(FWObject *obj);
    void toggleNeg();

    int get_row() { return row; }
    int get_col() { return col; }

    int  getX()      { return x;  }
    int  getY()      { return y;  }
    gint getWidth()  {  return width;  }
    gint getHeight() {  return height; }

    void activate();
    void deactivate();

    void select();
    void _select();
    void select(PolicyListItem *pi);
    void select(Gtk::Box_Helpers::BoxList::iterator bl_i);
    void deselect();

    void redraw();

    void request_focus();

    RuleElement* get_rule_element() { return rel; };

    virtual void popup_menu(PolicyListItem *itm);
};

class PolicyListRuleNum : public PolicyListElement {
    int  on_drag_motion( GdkDragContext *context,gint x,gint y,guint m);
    void obj_drag_data_received ( GdkDragContext     *context,
				  gint                x,
				  gint                y,
				  GtkSelectionData   *data,
				  guint               info,
				  guint32             time);
    gint on_motion_notify_event(GdkEventMotion*);

    SigC::Connection  motion_event_handler;
public:
    PolicyListRuleNum(gint row,gint col,gint rule_n,bool neg);
    virtual void popup_menu(PolicyListItem *itm);
};

/*******************************************************************/

class PolicyListRuleAction : public PolicyListElement {
    public:
    PolicyListRuleAction(gint row, gint col, Rule *r);
    virtual void popup_menu(PolicyListItem *itm);
};

/*******************************************************************/

class PolicyListRuleLog : public PolicyListElement {
    public:
    PolicyListRuleLog(gint row, gint col, Rule *r);
    virtual void popup_menu(PolicyListItem *itm);
};

/*******************************************************************/

class PolicyListRuleComment : public PolicyListElement {
    public:
    PolicyListRuleComment(gint row, gint col, Rule *r);
    virtual void popup_menu(PolicyListItem *itm);
};

/*******************************************************************/

class PolicyListRuleDir : public PolicyListElement {
    public:
    PolicyListRuleDir(gint row, gint col, Rule *r);
    virtual void popup_menu(PolicyListItem *itm);
};

/*******************************************************************/

class PolicyListRuleOpt : public PolicyListElement {
    public:
    PolicyListRuleOpt(gint row, gint col, Rule *r);
    virtual void popup_menu(PolicyListItem *itm);
};


#endif









