/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyListItem.cc,v 1.23 2001/07/22 06:13:19 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <gdk_imlib.h>

// #include "ttips.hh"
#include "ObjectQuickView.hh"

#include "glademm_support.hh"
#include "helpers.hh"
#include "FWObjectDatabase.hh"
#include "BuiltinDialog.hh"
#include "gen_popup_menu.hh"

#include "Host.hh"
#include "Network.hh"
#include "Firewall.hh"

#include "PolicyListItem.hh"
#include "PolicyListElement.hh"
#include "PolicyList.hh"

#include "main_window.hh"

#define MAX_COMMENT_FIELD_WIDTH   150

PolicyListObjectItem::PolicyListObjectItem(FWObject *o,
					   bool  neg_flag,
					   bool  change_any_2_orig) :
    PolicyListItem( o->getXPMIconFileName(""),  o->getName(),  neg_flag  )
{
    object=o;

    drag_source_set ( 
	static_cast<GdkModifierType>(GDK_BUTTON1_MASK) ,
	target_table, n_targets, 
	static_cast<GdkDragAction>(GDK_ACTION_COPY) );


    drag_begin.connect(
	slot(this,&PolicyListObjectItem::source_drag_begin));
    drag_data_get.connect(
	slot(this,&PolicyListObjectItem::source_drag_data_get));

    if (change_any_2_orig && lbl_txt=="Any")	setLabel("Original");

}

void PolicyListObjectItem::source_drag_begin(GdkDragContext     *context)
{
    GdkPixmap *pmap;
    GdkBitmap *mask;
    Gdk_Colormap cmap ( get_colormap () );

    string  icn=Resources::global_res->getIconPath("Drag");

    if (gdk_imlib_load_file_to_pixmap((char*)(icn.c_str()), &pmap, &mask)){
	Gdk_Pixmap drag_icon=Gdk_Pixmap(pmap);
	Gdk_Bitmap drag_mask=Gdk_Bitmap(mask);
	drag_source_set_icon(cmap, drag_icon, drag_mask);
	gdk_imlib_free_pixmap(pmap);
    }
}

void PolicyListObjectItem::source_drag_data_get  ( GdkDragContext     *context,
					   GtkSelectionData   *selection_data,
						   guint               info,
						   guint32             time )
{
    PolicyListObjectItem *plitm=this;

    gtk_selection_data_set (selection_data,
			    gdk_atom_intern("PolicyListObjectItem",TRUE),
			    7,
			    (const guchar*)(&plitm),
			    sizeof(PolicyListItem*) );
}

/************************************************************************/

PolicyListRuleNumItem::PolicyListRuleNumItem(const string& label,
					     bool neg_flag) :
    PolicyListItem(Resources::global_res->getIconPath("Blank"),
		   label,neg_flag)
{
    set_name("PolicyListRuleNumItem");

    setOverlap(true);

    drag_source_set ( 
	static_cast<GdkModifierType>(GDK_BUTTON1_MASK) ,
	target_table, n_targets, 
	static_cast<GdkDragAction>(GDK_ACTION_COPY) );
    drag_begin.connect(
	slot(this,&PolicyListRuleNumItem::source_drag_begin));
    drag_data_get.connect(
	slot(this,&PolicyListRuleNumItem::source_drag_data_get));
}

void PolicyListRuleNumItem::source_drag_begin(GdkDragContext     *context)
{
    Gtk::Widget *pl_el = get_parent();     /* PolicyListElement */
    Gtk::Widget *pl = pl_el->get_parent(); /* PolicyList */
    Gtk::Widget *par;



    GdkImlibImage *img;
    GdkPixmap     *pmap;
    GdkBitmap     *mask;
    Gdk_Colormap  cmap ( get_colormap () );

/*
 *     Trick: determine physical size of one policy line in pixels,
 *     then grab portion of the screen in that area and use it as dragging
 *     icon. User is going to see that the whole rule is being dragged 
 */

    gint x,y,w,h;
    gint w1,h1;

    x=((PolicyList*)pl)->getX();
    y=((PolicyListElement*)pl_el)->getY();
    h=((PolicyListElement*)pl_el)->getHeight();
    w=((PolicyList*)pl)->getWidth();

    par=pl->get_parent();

/*
 *  PolicyList may be too large and can be clipped. Check the size of its
 *  parent and if it is smaller, use it.
 */
    gdk_window_get_size( par->gtkobj()->window , &w1, &h1);
    if (w1<w) w=w1;

    x-=2;
//    y-=1;
    h+=4;
    w+=4;

    img=gdk_imlib_create_image_from_drawable(
	par->gtkobj()->window,
	NULL,
	x, y, w, h);

    if (img) {
	unsigned char *pixel, r,g,b, bw;
	int      iw,ih;

	pixel=img->rgb_data; 
	for (iw=0; iw<img->rgb_width; iw++) {
	    for (ih=0; ih<img->rgb_height; ih++) {

		r= *pixel;
		g= *(pixel+1);
		b= *(pixel+2);
	    
		bw= (r+g+b)/3;

		*pixel++ = bw;
		*pixel++ = bw;
		*pixel++ = bw;
	    }
	}

	gdk_imlib_render(img,w,h);

	pmap=gdk_imlib_move_image(img);
	mask=gdk_imlib_move_mask(img);

	Gdk_Pixmap drag_icon=Gdk_Pixmap(pmap);
	Gdk_Bitmap drag_mask=Gdk_Bitmap(mask);

	drag_source_set_icon(cmap, drag_icon, drag_mask);
     
	gdk_imlib_free_pixmap(pmap);
    }
}

void PolicyListRuleNumItem::source_drag_data_get( GdkDragContext     *context,
					   GtkSelectionData   *selection_data,
						   guint               info,
						   guint32             time )
{
    PolicyListRuleNumItem *plitm=this;

    gtk_selection_data_set (selection_data,
			    gdk_atom_intern("PolicyListRuleNumItem",TRUE),
			    6,
			    (const guchar*)(&plitm),
			    sizeof(PolicyListItem*) );
}

/************************************************************************/




GdkPixmap*  PolicyListItem::n_pmap=NULL;
GdkBitmap*  PolicyListItem::n_mask=NULL;
bool        PolicyListItem::neg_icon_loaded=false;


PolicyListItem::PolicyListItem(const string &pixfl, 
			       const string &label,
			       bool  neg_flag)
{
    object=NULL;
    pmap=NULL;
    mask=NULL;
    icon_w=0;
    icon_h=0;
    x=y=0;

    ov=NULL;

    neg=neg_flag;
    lbl_txt=label;
    icn_file=pixfl;

    overlap=false;
    selected=false;
    active=false;
    with_pixmap=false;
    show_icons=(Preferences::global_prefs->getOptBool(
	"/FWBuilderPreferences/UI/ShowIconsInPolicy"));


    border=2;
    spacing=2;

    x=0;
    y=0;
    width=0;
    height=0;

    loadNegIcon();

    icon_size=
	Resources::global_res->getResourceInt("/FWBuilderResources/UI/SmallIconSize");

    set_name("PolicyListItem");
    set_flags( GTK_CAN_FOCUS );

    add_events(GDK_POINTER_MOTION_MASK | 
	       GDK_POINTER_MOTION_HINT_MASK | 
	       GDK_ENTER_NOTIFY_MASK | 
	       GDK_LEAVE_NOTIFY_MASK | 
	       GDK_PROXIMITY_IN_MASK | 
	       GDK_PROXIMITY_OUT_MASK );

    if (icn_file!="") setIcon(icn_file);

    show();

    size_request.connect(SigC::slot(this,&PolicyListItem::on_size_request));
    size_allocate.connect(SigC::slot(this,&PolicyListItem::on_size_allocate));
    button_press_event.connect(
	SigC::slot(this,&PolicyListItem::on_button_press_event));
    button_release_event.connect_after(
	SigC::slot(this,&PolicyListItem::on_button_release_event));

    enter_notify_event.connect( 
	SigC::slot(this,&PolicyListItem::on_enter_notify_event));
    leave_notify_event.connect( 
	SigC::slot(this,&PolicyListItem::on_leave_notify_event));

}

PolicyListItem::~PolicyListItem()
{
    if (pmap)	 gdk_imlib_free_pixmap(pmap);
//    if (n_pmap)	 gdk_imlib_free_pixmap(n_pmap);
}

gint PolicyListItem::on_enter_notify_event(GdkEventCrossing* ev)
{
    if (ov) return(true);  // QuickView is already active

    if (object!=NULL &&
	! Resources::global_res->getObjResourceBool(object,"hidden") ) {

	if ( Preferences::global_prefs->getOptStr(
	    "/FWBuilderPreferences/UI/ObjectQuickView")=="window") {
/* 
 *  show in built-in window
 *
 *  TODO:   implement this!
 */

	}

	if ( Preferences::global_prefs->getOptStr(
	    "/FWBuilderPreferences/UI/ObjectQuickView")=="popup") {

	    int timeout= atoi(
		Preferences::global_prefs->getOpt(
		    "/FWBuilderPreferences/UI/ObjectQuickViewTimeout"));
	
	    quick_view_timer_conn=Gtk::Main::timeout.connect(
		SigC::slot(this,&PolicyListItem::timer_callback),
		timeout*1000 );
	}
    }
    return(true);
}

gint PolicyListItem::on_leave_notify_event(GdkEventCrossing* ev)
{
    quick_view_timer_conn.disconnect();
    if (ov) {
	ov->hide();
	delete ov;
	ov=NULL;
    }
    return(true);
}

gint PolicyListItem::timer_callback()
{
    int x,y,w,h,d;

    Gdk_Window   wnd=get_window();

    wnd.get_geometry(x,y,w,h,d);
    wnd.get_origin(x,y);

    x-=10;
    y+=(h+10);

    ov=new ObjectQuickView( object );
    ov->set_uposition(x,y);

    ov->show();

    return false;
}

gint PolicyListItem::on_button_press_event(GdkEventButton *ev)
{
    PolicyListElement *pe=(PolicyListElement*)get_parent();

/*
 *  Try to bring main window on top
 */

    main_w->get_window().raise();

    if ( ev->type == GDK_BUTTON_PRESS && ev->button==1) 
	pe->select(this);

    return( true );
}

gint PolicyListItem::on_button_release_event(GdkEventButton *ev)
{
    PolicyListElement *pe=(PolicyListElement*)get_parent();

    if ( ev->type == GDK_BUTTON_RELEASE && ev->button==3 ) {
	if (get_state()!=GTK_STATE_SELECTED)  pe->select(this);
	pe->popup_menu(this);
	redraw();

	gtk_signal_emit_stop_by_name( GTK_OBJECT( gtkobj() ),
				      "button_release_event" );

	return(true);
    }
    return( true );
}



void PolicyListItem::select()
{
  activate();
  if (!selected) {
    selected=true;
    set_state(GTK_STATE_SELECTED);
    queue_draw();
  }
}

void PolicyListItem::deselect()
{
  if (selected) {
    selected=false;
    active=true;
    set_state(GTK_STATE_PRELIGHT);
    queue_draw();
  }
}

void PolicyListItem::activate()
{
  if (!active) {
    active=true;
    set_state(GTK_STATE_PRELIGHT);
    queue_draw();
  }
}

void PolicyListItem::deactivate()
{
  if (active) {
    active=false;
    selected=false;
    set_state( GTK_STATE_NORMAL );
    queue_draw();
  }
}

void PolicyListItem::setLabel(string txt)
{
    lbl_txt=txt;
}

string PolicyListItem::getLabel()
{
  return(lbl_txt);
}

void  PolicyListItem::setIcon(string icn)
{
    GdkImlibImage         *im;

    icn_file=icn;

    if (! icn_file.empty() && 
	(im=gdk_imlib_load_image( (char*)(icn_file.c_str())))!=NULL ) {

	icon_w=im->rgb_width;
	icon_h=im->rgb_height;

	if ( gdk_imlib_render( im , icon_w , icon_h )) {

 /* Extract the Image and mask pixmaps from the Image */
	    pmap=gdk_imlib_move_image(im);
 /* The mask will be NULL if the image has no transparency */
	    mask=gdk_imlib_move_mask(im);

	    with_pixmap=true;
	}
    }
}

void PolicyListItem::loadNegIcon()
{
    if ( ! neg_icon_loaded) {

	string     neg_icn=
Resources::global_res->getResourceStr("/FWBuilderResources/Paths/Icndir");
	neg_icn += "/";
	neg_icn += Resources::global_res->getResourceStr("/FWBuilderResources/UI/Icons/neg");

	if ( gdk_imlib_load_file_to_pixmap((char*)(neg_icn.c_str()),
				      &n_pmap, &n_mask)){

	    neg_icon_loaded=true;
	}
    }
}

void PolicyListItem::setNeg(bool neg_flag)
{
    neg=neg_flag;
    Gtk::Main::idle.connect(slot(this,&PolicyListItem::redraw));
}

void PolicyListItem::setOverlap(bool ovr)
{
    overlap=ovr;
    Gtk::Main::idle.connect(slot(this,&PolicyListItem::redraw));
}

void PolicyListItem::setSpacing(gint sp)
{
    spacing=sp;
}

void PolicyListItem::setBorder(gint bdr)
{
    border=bdr;
}


/********************  Drawing methods ********************************** */

void PolicyListItem::realize_impl()
{
    GtkWidget* widget=gtkobj();

    GdkWindowAttr  attributes;
    gint           attributes_mask;
    GTK_WIDGET_SET_FLAGS( widget , GTK_REALIZED );

    attributes.window_type = GDK_WINDOW_CHILD;
    attributes.x = widget->allocation.x;
    attributes.y = widget->allocation.y;
    attributes.width = widget->allocation.width;
    attributes.height= widget->allocation.height;
    attributes.wclass= GDK_INPUT_OUTPUT;
    attributes.visual= gtk_widget_get_visual(widget);
    attributes.colormap= gtk_widget_get_colormap(widget);
    attributes.event_mask=gtk_widget_get_events(widget) | 
	GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK;

    attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

    widget->window = gdk_window_new(gtk_widget_get_parent_window(widget),
				    &attributes, attributes_mask);

    gdk_window_set_user_data( widget->window , widget );


/* Style */

    widget->style = gtk_style_attach( widget->style , widget->window );
    gtk_style_set_background( widget->style, widget->window, GTK_STATE_NORMAL);
    gdk_window_set_background(widget->window, 
			      &widget->style->bg[ get_state() ]);

}

void PolicyListItem::draw_impl(GdkRectangle* p0)
{
    paint(p0);
}

gint PolicyListItem::expose_event_impl(GdkEventExpose* ex)
{
    paint( &ex->area );
    return true;
}

void PolicyListItem::paint(GdkRectangle* p0)
{
/*
    cerr << "Paint: this=" << this 
	 << " state=" << int(get_state() )
	 << " item=" <<  lbl_txt
	 << " x=" << p0->x
	 << " y=" << p0->y
	 << " w=" << p0->width
	 << " h=" << p0->height
	 << endl;
*/

    GtkWidget*   widget=gtkobj();
    Gdk_Window   wnd=get_window();

    gint         icnx,icny,strx,stry;

    Gtk::Style  *st=get_style();

    Gdk_Font     fn=st->get_font();

    Gdk_GC       gc=Gdk_GC(wnd);
    Gdk_GC       wgc;

    if (overlap) {
	icnx=width/2 - icon_w/2;
	strx=width/2 - sw/2;
    } else {
	icnx=0;
	if (with_pixmap) strx=border + icon_w + spacing;
	else             strx=width/2 - sw/2;
    }
    icny=height/2-icon_h/2;
    stry=height/2+sh/2+1;


//    gdk_window_clear_area( wnd, p0->x,p0->y,p0->width,p0->height);

    gdk_gc_set_clip_rectangle( widget->style->black_gc , p0 );

    gdk_draw_rectangle( widget->window,
			widget->style->bg_gc[get_state()],
			true,
			p0->x,p0->y,p0->width,p0->height);


    if (show_icons && with_pixmap)    {
	gdk_gc_set_clip_origin( widget->style->black_gc,
				icnx , icny );
	gdk_gc_set_clip_mask( widget->style->black_gc,
			      mask );
	gdk_draw_pixmap( widget->window, 
			 widget->style->black_gc,
			 pmap , 0,0, icnx,icny, icon_w, icon_h );
	if (neg) {
	    gdk_gc_set_clip_mask( widget->style->black_gc,
				  n_mask );
	    gdk_draw_pixmap( widget->window, 
			     widget->style->black_gc,
			     n_pmap , 0,0, icnx,icny, icon_w, icon_h );
//	    gdk_draw_pixmap( widget->window, 
//			     widget->style->fg_gc[get_state()],
//			     n_pmap , 0,0, icnx,icny, icon_w, icon_h );
	}
    }

//    gdk_gc_set_clip_rectangle(widget->style->fg_gc[get_state()], p0 );
//    gdk_gc_set_clip_rectangle(widget->style->bg_gc[get_state()], p0 );
    gdk_gc_set_clip_rectangle( widget->style->black_gc , p0 );
    gdk_draw_text(  widget->window, 
		    widget->style->font ,
		    widget->style->fg_gc[get_state()],
		    strx, stry,
		    lbl_txt.c_str(),
		    lbl_txt.size() );

    gint asc=widget->style->font->ascent;

    if (neg) 
	gdk_draw_line( widget->window , 
		       widget->style->fg_gc[get_state()],
		       strx,stry-asc/2+1,strx+sw,stry-asc/2+1);
}

gint PolicyListItem::redraw()
{
    queue_draw();
    return false;
}

void PolicyListItem::on_size_request(GtkRequisition* req)
{
    Gtk::Style *st=get_style();
    Gdk_Font    fn=st->get_font();
    gint        resx, resy;

    sw= fn.string_width(lbl_txt);
    sh= fn.string_height(lbl_txt);

    if (! with_pixmap) {
	icon_w=icon_size;
	icon_h=icon_size;
    }

    if (overlap) {
	resx = sw;
	if (with_pixmap && icon_w>sw) resx = icon_w;
    } else {
	if (with_pixmap) resx = icon_w + sw;
	else             resx = sw;
    }

    resy=sh;
    if (with_pixmap && icon_h>sh) resy = icon_h;

    resx+=(border*2);
    resx+=spacing;
    resy+=(border*2);

    req->width   = resx;
    req->height  = resy;
}

void PolicyListItem::on_size_allocate(GtkAllocation* ga)
{
/*
 *  Remember position, width and height allocated for this widget. 
 *  This info is needed in paint
 */
    x=ga->x;
    y=ga->y;
    width=ga->width;
    height=ga->height;
}

/************************************************************************/

PolicyListCommentItem::PolicyListCommentItem(const string& text) :
    PolicyListItem("",text)
{
    lbl_txt=text;
    nlines=1;
    size_request.connect(SigC::slot(this,&PolicyListCommentItem::on_size_request));
}

void PolicyListCommentItem::paint(GdkRectangle* p0)
{
    const char        *str_beginning, *str_end, *cptr, *cptr2;
    int          n,w;

    GtkWidget   *widget=gtkobj();
    Gdk_Window   wnd=get_window();

    gint         strx,stry;

    Gtk::Style  *st=get_style();

    Gdk_Font     fn=st->get_font();

    Gdk_GC       gc=Gdk_GC(wnd);
    Gdk_GC       wgc;

    strx=border;
    stry=sh+2;

    gdk_window_clear_area( wnd, p0->x,p0->y,p0->width,p0->height);
    gdk_gc_set_clip_rectangle( widget->style->black_gc , p0 );

    gdk_draw_rectangle( widget->window,
			widget->style->bg_gc[get_state()],
			true,
			p0->x,p0->y,p0->width,p0->height);


    gdk_gc_set_clip_rectangle(widget->style->fg_gc[get_state()], p0 );

    str_beginning=cptr=lbl_txt.c_str();
    w=0;
    while (true) {
	if ( (cptr2=strchr(cptr,' '))!=NULL ) {

	    n=cptr2-cptr;
	    w+=fn.text_width(cptr,n);

	    if (w < min(width,MAX_COMMENT_FIELD_WIDTH) )     cptr=cptr2+1;
	    else {   // print from str_beginning to cptr, then line feed

		if (cptr==str_beginning) {  //long fragment before first space
		    str_end=cptr2+1;
		} else {
		    str_end=cptr;
		}
		gdk_draw_text(  widget->window, 
				widget->style->font ,
				widget->style->fg_gc[get_state()],
				strx, stry,
				str_beginning, str_end-str_beginning );
		str_beginning=str_end;
		stry+=(sh+2);
		w=0;
	    }
	} else {  // print from str_beginning to the end and stop
	    
	    gdk_draw_text(  widget->window, 
			    widget->style->font ,
			    widget->style->fg_gc[get_state()],
			    strx, stry,
			    str_beginning, strlen(str_beginning) );
	    break;
	}
    }
}

void PolicyListCommentItem::on_size_request(GtkRequisition* req)
{
    const char *str_beginning, *str_end, *cptr, *cptr2;
    int         w,n, maxx, str_width;

    Gtk::Style *st=get_style();
    Gdk_Font    fn=st->get_font();
    gint        resx, resy;

    sw= fn.string_width(lbl_txt);
    sh= fn.string_height( (lbl_txt=="")?"W":lbl_txt );

    str_beginning=cptr=lbl_txt.c_str();
    w=0;
    nlines=0;
    maxx=0;
    while (true) {
	if ( (cptr2=strchr(cptr,' '))!=NULL ) {

	    n=cptr2-cptr;
	    w+=fn.text_width(cptr,n);

	    if (w<MAX_COMMENT_FIELD_WIDTH)     cptr=cptr2+1;
	    else {   // print from str_beginning to cptr, then line feed
		if (cptr==str_beginning) {
		    str_end=cptr2+1;
		} else {
		    str_end=cptr;
		}
		str_width=fn.text_width(str_beginning,str_end-str_beginning+1);
		if (maxx<str_width) maxx=str_width;
		str_beginning=str_end;
		nlines++;
		w=0;
	    }
	} else {  // print from str_beginning to the end and stop
	    str_width=fn.text_width(str_beginning, strlen(str_beginning)+1);
	    if (maxx<str_width) maxx=str_width;
	    nlines++;
	    break;
	}
    }

    resx = maxx;

    resy=(sh+2)*nlines;

    resx+=(border*2);
    resx+=spacing;
    resy+=(border*2);

    req->width   = resx;
    req->height  = resy;

}






