/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyListItem.hh,v 1.8 2001/05/16 08:34:02 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef _POLICYLISTITEM_HH
#define _POLICYLISTITEM_HH

#include <gtk--.h>

#include "FWObject.hh"
#include "Policy.hh"

class PolicyList;
class ObjectQuickView;

class PolicyListItem : public  Gtk::Widget {

    friend class PolicyList;


protected:

    string      obj_quick_view_info;
    gint        x,y,width,height;
    gint        sw, sh;
    FWObject   *object;

    gint        icon_size;
    gint        icon_w, icon_h;
    
    gint        spacing;
    gint        border;

    string      icn_file;
    string      lbl_txt;

    bool        neg;
    bool        active;
    bool        selected;
    bool        overlap;
    bool        with_pixmap;
    bool        show_icons;

    ObjectQuickView *ov;
    Gtk::Connection quick_view_timer_conn;

    GdkPixmap  *pmap;
    GdkBitmap  *mask;

    static GdkPixmap *n_pmap;
    static GdkBitmap *n_mask;
    static bool        neg_icon_loaded;

    void loadNegIcon();

    gint on_enter_notify_event(GdkEventCrossing* ev);
    gint on_leave_notify_event(GdkEventCrossing* ev);

    gint timer_callback();

    gint on_button_press_event(GdkEventButton *ev);
    gint on_button_release_event(GdkEventButton *ev);

    virtual void draw_impl (GdkRectangle* p0);
    virtual gint expose_event_impl(GdkEventExpose* p0);
    virtual void realize_impl();

    void on_size_request(GtkRequisition*);
    void on_size_allocate(GtkAllocation* ga);

    virtual void paint(GdkRectangle* p0);

     
public:

    PolicyListItem(const string &pixmap, 
		   const string &label,
		   bool  neg_flag=false);

    ~PolicyListItem();

    gint redraw();
    
    void activate();
    void deactivate();
    void select();
    void deselect();

    string    getLabel();
    FWObject* getObject() { return object; }

    void   setLabel(string txt);
    void   setIcon(string xpm);

    void setNeg(bool flag);
    void setOverlap(bool flag);

    void setSpacing(gint sp);
    void setBorder(gint bdr);

};

class PolicyListObjectItem : public PolicyListItem {

    friend class PolicyList;

    void source_drag_begin( GdkDragContext  *context);

    void source_drag_data_get  ( GdkDragContext     *context,
				 GtkSelectionData   *selection_data,
				 guint               info,
				 guint32             time );

public:

    PolicyListObjectItem(FWObject *o,
			 bool  neg_flag,
			 bool  change_any_to_orig);


};

class PolicyListRuleNumItem : public PolicyListItem {

    friend class PolicyList;

    void source_drag_begin( GdkDragContext  *context);
    void source_drag_data_get  ( GdkDragContext     *context,
				 GtkSelectionData   *selection_data,
				 guint               info,
				 guint32             time );
public:
    PolicyListRuleNumItem(const string& label,bool  neg_flag=false);
};


class PolicyListCommentItem : public PolicyListItem {
    friend class PolicyList;
    int  nlines;
public:
    PolicyListCommentItem(const string& text);
protected:
    virtual void paint(GdkRectangle* p0);
    void on_size_request(GtkRequisition*);
};


#endif
