
/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: RuleSet.hh,v 1.4 2001/04/25 19:11:35 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef _RULESET_HH
#define _RULESET_HH

#include "FWObject.hh"
#include "Rule.hh"

class PolicyList;
class PolicyListElement;

class RuleSet : public FWObject  {

    friend class PolicyList;
    friend class PolicyListElement;

private:

protected:

    void   renumberRules();

    const char** rule_elements;

public:

    RuleSet();
    virtual ~RuleSet();

    DECLARE_FWOBJECT_SUBTYPE(RuleSet);

    virtual Rule*   createRule();
    virtual void    createElements(Rule* rule);

    virtual DialogPlugin* CreateDialog();
            void          updateMainMenu();

//    virtual FWObject& operator=(const FWObject &);

    Rule* getRuleByNum(int n);

    Rule* insertRuleAtTop();
    Rule* insertRuleBefore(gint rule_n);

    Rule* appendRuleAfter(gint rule_n);
    Rule* appendRuleAtBottom();

    bool  deleteRule(gint rule_n);
    bool  deleteRule(Rule *r);

    bool  moveRuleUp(gint rule_n);
    bool  moveRuleDown(gint rule_n);
    bool  moveRule(gint src_rule_n,gint dst_rule_n);

    bool  disableRule(gint rule_n);
    bool  enableRule(gint rule_n);
    bool  isRuleDisabled(gint rule_n);

    gint  getRuleSetSize();

    void    getTitles(list<const char*>& titles);
    void    getElements(list<const char*>& elements);
  
    PolicyListElement* createPolicyListElement(int row,int col,
					       const char* element_name,
					       Rule *rule);


    virtual gint get_n_columns();


};




#endif

