/* 

                          Firewall Builder

                 Copyright (C) 2001 Vadim Zaliva, Vadim Kurland

  Author:  Vadim Zaliva lord@crocodile.org

  $Id: ThreadTools.cc,v 1.7 2001/06/21 13:23:19 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "ThreadTools.hh"

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/poll.h>

Mutex::Mutex()
{
    mutex = g_mutex_new();
}

Mutex::~Mutex()
{
    g_mutex_free(mutex);
}

void Mutex::lock() const
{
    g_mutex_lock(mutex);
}

void Mutex::unlock() const
{
    g_mutex_unlock(mutex);
}

SyncFlag::SyncFlag(bool v=false)
{
    value = v;
}

bool SyncFlag::peek() const
{
    return value;
}

bool SyncFlag::get() const
{
    bool v;
    lock();
    v = value; 
    unlock();
    return v;
}

void SyncFlag::modify(bool v)
{
    value = v;
}

void SyncFlag::set(bool v)
{
    lock();
    value = v;
    unlock();
}
    
SyncFlag::operator bool() const
{
    return get();
}
    
SyncFlag& SyncFlag::operator=(const SyncFlag &o)
{
    set(o.get());
    return *this;
}
    
SyncFlag& SyncFlag::operator=(bool v)
{
    set(v);
    return *this;
}

TimeoutCounter::TimeoutCounter(unsigned int timeout, const string &name_)
{
    time_t tres;

    name    = name_                ;
    finish  = time(&tres) + timeout ;
}

unsigned int TimeoutCounter::timeLeft() const
{
    time_t tres;
    int res = finish-time(&tres);
    return res<0?0:res;
}

bool TimeoutCounter::isExpired() const
{
    time_t tres;
    return time(&tres) > finish ;
}

void TimeoutCounter::check() const throw(FWException) 
{
    if(isExpired())
    {
        //cerr << "Expired Timeout Counter." << endl;
        throw FWException(name+" timeout");
    }
}

ssize_t TimeoutCounter::read(int fd, void *buf, size_t n) const throw(FWException)
{
    struct pollfd ufds[1];

    ufds[0].fd=fd;
    ufds[0].events=POLLIN|POLLPRI;
    
    int retval=poll(ufds, 1, 1000*timeLeft());

    if(retval==0)
        throw FWException("Timeout "+name);
    else if(retval>0)
        return ::read(fd, buf, n);
    else 
        return -1; //error
}
