/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: XMLTools.hh,v 1.15 2001/05/25 01:45:29 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/**
 * $Id: XMLTools.hh,v 1.15 2001/05/25 01:45:29 lord Exp $
 */

/**
 * This file contains assorted XML handling code
 */

#ifndef __XML_TOOLS_H_FLAG
#define __XML_TOOLS_H_FLAG

#include "Tools.hh"
#include "FWException.hh"

#include <string>

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xmlIO.h>
#include <libxml/parserInternals.h>

#define FROMXMLCAST(x) ((const char *)x)
#define STRTOXMLCAST(x) ((xmlChar *)x.c_str())
#define TOXMLCAST(x) ((xmlChar *)x)

class XMLTools
{
    public:

    static xmlNodePtr getXmlNodeByPath(xmlNodePtr r,const char   *path       );
    static xmlNodePtr getXmlNodeByPath(xmlNodePtr r,const string &path       );

    static xmlNodePtr getXmlChildNode (xmlNodePtr r,const char   *child_name );

    static void initXMLTools();

    /**
     * Loads given file, performing version conversion
     * if neccessary.
     */
    static xmlDocPtr loadFile(const string &file_name, 
                              const string &type_name, 
                              const string &dtd_file) throw(FWException);

    static void setDTD(xmlDocPtr doc, 
                       const string &type_name, 
                       const string &dtd_file) throw(FWException);
    
    /**
     * Saves file with setting DTD.
     */
    static void saveFile(xmlDocPtr doc, 
                         const string &file_name, 
                         const string &type_name,
                         const string &dtd_file) throw(FWException);
    
    static xmlExternalEntityLoader defaultLoader;

    /**
     * Loads given file, without version conversion.
     * @return document pointer
     */
    static xmlDocPtr parseFile(const string &file_name, 
                               bool use_dtd
    ) throw(FWException);
    
    /**
     * Performs XSLT transformation of the file in memory
     * @return new document
     */
    static xmlDocPtr transformDocument(xmlDocPtr doc, 
                                       const string &stylesheet_file,
                                       const char **params
    ) throw(FWException);

    /**
     * Performs XSLT transformation of the file. Results are
     * stored in dst file.
     */
    static void transformDocumentToFile(xmlDocPtr doc, 
                                        const string &stylesheet_file,
                                        const char **params,
                                        const string &dst_file
    ) throw(FWException);
    
    private:
    
    static const string CONVERTERS_DIR;
    
    /**
     * Convert file from older version to current one
     * @return pointer to new document or NULL if doc is unchanged.
     *         if pointer to new document is returned, doc parameter
     *         becomes invalid and should not be used.
     */
    static xmlDocPtr convert(xmlDocPtr doc, 
                             const string &file_name, 
                             const string &type_name) throw(FWException);

    /**
     * Compares 2 product version numbers in dotted notation
     * @return 0 if equal, positive if v1>v2, negative if v1<v2
     */
    static int version_compare(const string &v1, const string &v2);

    /**
     * returns first component of dotted notation.
     */
    static int major_number(const string &v, string &rest);

};

#endif


