/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: fwbuilder.cc,v 1.47 2001/07/17 15:36:44 lord Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#ifdef NEED_GNOMESUPPORT_H
#include <gnome-wrappers/gnomemain.hh>
#else
#include <gtk--/main.h>
#endif

#include <glib.h>
#include <gdk_imlib.h>

#include "FWObject.hh"
#include "FWObjectDatabase.hh"
#include "helpers.hh"
#include "ttips.hh"

#include "main_window.hh"
#include "ObjectTree.hh"

#include "Resources.hh"
#include "Preferences.hh"
#include "MessageDialog.hh"
#include "XMLTools.hh"

// DEBUG
#include "snmp.hh"
// DEBUG
#include "dns.hh"

#include <config.h>

class StaticInitializer
{
    public:
    
    StaticInitializer()
    {
        /* init threads */
        g_thread_init(NULL);
    }
} init;

static char      *filename   = NULL;
main_window      *main_w     = NULL;

struct poptOption options[] = 
{
    { "file", 'f', POPT_ARG_STRING, &filename, 0,
      "File to load on startup", "filename" },
    POPT_AUTOHELP
    { NULL, 0, 0, NULL, 0 }
};

/**
 *   This handler gets called whenever gtk event loop is stopped via 
 *   Gtk::Main::quit()
 */

gint quit_handler()
{
    if (gtk_main_level()==1) {
// this is the  end of program


    }
    return true;
}


int main(int argc, char **argv)
{   
    gdk_threads_enter();

    /* Init XML parser */
    XMLTools::initXMLTools();


    Gtk::Main m(&argc, &argv);

    gnome_init_with_popt_table("fwbuilder", VERSION, argc, argv, options,0,0 );

   
    main_w = new main_window();
    main_w->show_all();

    try  
    {
        new Resources();       // creates singleton Resources::global_res
    } catch(FWException &ex)  
    {
        /*
         *  Error loading resources. MessageDialog needs Resources (for
         *  icons), so we can't use it to show error message.  
         */
	cerr << ex.toString() << endl;
        exit(1);
    }

    new FWObjectClipboard(); // creates singleton 
    new FWObjectDatabase();  // creates db and scratchpad

    try  {
        new Preferences();     // creates singleton Preferences::global_prefs
    } catch(FWException &ex)  {
	MessageDialog::Error(ex.toString());
        exit(1);
    }

    try   {
	if(filename)  {
	    //load file given on command line
	    FWObjectDatabase::db->load(filename);
	} else  {
	    //load default file
	    string dbTemplateFile = 
		TEMPLATE_DIR  "/" + FWObjectDatabase::DB_TEMPLATE_FILE_NAME;
	    FWObjectDatabase::db->load(dbTemplateFile);
	    FWObjectDatabase::db->setFileName("");
	}
    } catch(FWException &ex)  {
	MessageDialog::Error(ex.toString());
	exit(1);
    }
   
    ObjectTree *w=(ObjectTree*)find_widget("ObjectTree",main_w);
    if (w!=NULL)  w->rebuildTree();

    InitializeTooltips();

    Gtk::Main::quit.connect( SigC::slot( &quit_handler )  ) ;


    //DEBUG
/*
    try
    {
        vector<IPNetwork> include;
        SNMPCrawler q(IPAddress("10.3.14.30"),"public",1,1);
        StreamLogger logger(&cerr);
        q.run_impl(&logger);

	map<IPAddress, CrawlerFind>  t;
	CrawlerFind  f;

        cerr << "*** All:" << endl;
        t = q.getAllIPs();
        for(map<IPAddress, CrawlerFind>::iterator j = t.begin(); 
	    j!=t.end(); ++j) {
	    cerr << "--------------------------------------\n";

            cerr << (*j).first.toString() << endl;
	    f=(*j).second;

	    cerr << " descr:    " << f.descr    << endl;
	    cerr << " contact:  " << f.contact  << endl;
	    cerr << " location: " << f.location << endl;
	    cerr << " sysname:  " << f.sysname  << endl;

	    for (vector<Interface*>::iterator k=(*j).second.interfaces.begin();
		 k!=f.interfaces.end(); ++k) 
		cerr << "   interface: " << (*k)->getStr("address") << endl;
        }

        cerr << "*** Networks:" << endl;
        set<IPNetwork> n = q.getNetworks();
        for(set<IPNetwork>::iterator j = n.begin(); j!=n.end(); ++j)
            cerr << string(*j) << endl;
        
            
    } catch(FWException &ex)
    {
        cerr << "EXCEPTION: " << ex.toString();
    }
*/

/*
    try
    {
        cerr << "Starting resolver" << endl;
        HostEnt he=DNS::getHostByAddr(IPAddress("64.1.167.101"));
        cerr << "Name: " << he.name << endl;
        cerr << "Alises:" << endl;
        for(set<string>::iterator j = he.aliases.begin(); j!=he.aliases.end(); ++j)
            cerr << (*j) << endl;
    } catch(FWException &ex)
    {
        cerr << "EXCEPTION: " << ex.toString();
    }
    cerr << "Resolver done" << endl;
*/

//     IPNetwork n(IPAddress("10.3.15.0"), Netmask("255.255.255.0"));
//     cerr << "Belongs?: " << n.belongs(IPAddress("103.14.0.2")) << endl;

    m.run();
    gdk_threads_leave();


    return 0;
}









