/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: main_window.cc,v 1.48 2001/07/07 07:19:16 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <config.h>

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>

#include <gtk--.h>

#include <libgnomeui/gnome-app.h>
#include <libgnomeui/gnome-app-helper.h>
#include <gdk/gdkkeysyms.h>
#include <libgnomeui/gnome-stock.h>
#include <gnome.h>

#include "glademm_support.hh"


#include "main_window.hh"
#include "helpers.hh"

#include "Firewall.hh"
#include "Policy.hh"
#include "NAT.hh"

#include "FWObjectDatabase.hh"
#include "Network.hh"
#include "DialogPlugin.hh"
#include "PrintDialog.hh"
#include "PolicyDialog.hh"
#include "NATDialog.hh"
#include "FirewallDialog.hh"
#include "PolicyList.hh"
#include "ObjectTree.hh"
#include "About.hh"
#include "OptionsDlg.hh"
#include "ExecDialog.hh"
#include "MessageDialog.hh"
#include "FWOptions.hh"

#include "StandardRulesDruid.hh"
#include "DiscoveryDruid.hh"

#include "MiniBrowserDialog.hh"

main_window::main_window() :  Gtk::Window(GTK_WINDOW_TOPLEVEL)
// : App("Firewall Builder","Firewall Builder")
{

    custom_item=NULL;
    policy_in_right_pane=false;
    current_right_pane_dialog=NULL;


    main_menubar1    = manage(new class Gtk::MenuBar());
    hpaned           = manage(new class Gtk::HPaned());
    main_statusbar   = manage(new class Gtk::Statusbar());
   
    Gtk::VBox *vbox1 = manage(new class Gtk::VBox(false, 2));

    hpaned->set_handle_size(10);
    hpaned->set_gutter_size(6);
    hpaned->set_position(-1);

    vbox1->pack_start(*main_menubar1, false, false, 0);
    vbox1->pack_start(*hpaned);
    vbox1->pack_start(*main_statusbar, false, false, 0);

    set_usize(900, 550);

    add(*vbox1);

    hpaned->show();
    main_statusbar->show();
    vbox1->show();




    ctree1=new ObjectTree();

    left_pane_port     = manage( new Gtk::ScrolledWindow() );
    Gtk::Viewport *vp  = manage( new Gtk::Viewport() );
    
    ((Gtk::ScrolledWindow*)left_pane_port)->set_policy(GTK_POLICY_AUTOMATIC, 
						       GTK_POLICY_AUTOMATIC);
    ((Gtk::ScrolledWindow*)left_pane_port)->add(*vp);
    vp->add( *ctree1 );

    left_pane_port->set_usize(150,-1);

    vp=manage( new Gtk::Viewport() );
    right_pane_port=manage( new Gtk::EventBox() );
    vp->add( *right_pane_port );

    hpaned->pack1( *left_pane_port , false, false );
    hpaned->pack2( *vp , true, true);

    main_menubar1->set_shadow_type( GTK_SHADOW_NONE );  // glade bug?

    agr = (Gtk::AccelGroup*)Gtk::AccelGroup::get_default();
    agr->set_default_mod_mask( GdkModifierType(0) );
    add_accel_group( *agr );

    build_menu();  // needs ObjectTree and AccelGroup objects !



    delete_event.connect( SigC::slot(this,&main_window::on_delete_event));

    show_all();
}

void main_window::LoadObjDB(const string &data_file)
{   
    g_assert( FWObjectDatabase::db != NULL );

    try
    {
        FWObjectDatabase::db->load(data_file);
    } catch(FWException &ex)
    {
	MessageDialog::Error("Error loading data file.",ex.toString());
    }
	
    ObjectTree *w=(ObjectTree*)glademm_get_Widget("ObjectTree");
    if (w!=NULL) {
	w->clearSelection();
	w->schedule_rebuild();
    }
}

void main_window::on_new1_activate()
{   
    if(safe_to_close_right_pane()) 
    {
        string dbTemplateFile = TEMPLATE_DIR  "/" + FWObjectDatabase::DB_TEMPLATE_FILE_NAME;
	ClearRightPane(true);
	LoadObjDB(dbTemplateFile);
        FWObjectDatabase::db->setFileName("");
    }
}

void main_window::on_open1_activate()
{   
    if (safe_to_close_right_pane()) {
	ClearRightPane(true);
	LoadObjDB("");
    }
}

void main_window::on_save1_activate()
{   
    g_assert (FWObjectDatabase::db!=NULL);
    try
    {
        FWObjectDatabase::db->save(); 
    } catch(FWException &ex)
    {
	MessageDialog::Error("Error saving data file.",ex.toString());
    }
}

void main_window::on_saveas1_activate()
{   
    g_assert (FWObjectDatabase::db!=NULL);
    try
    {
        FWObjectDatabase::db->saveAs(); 
    } catch(FWException &ex)
    {
	MessageDialog::Error("Error saving data file.",ex.toString());
    }
}

bool main_window::checkAndSaveData()
{
    if ( ! safe_to_close_right_pane()) return false;
    if (FWObjectDatabase::db->isDirty(true) ) {
        try  {
            if (! FWObjectDatabase::db->saveIfModified() ) return false;  // "Cancel"
        } catch(FWException &ex)  {
	    MessageDialog::Error("Error saving data file.",ex.toString());
        }
    }
    return true;
}

/*
 * This event is generated in responce to the window manager's signal
 * sent to the window when user clicks "close" button on window's
 * title bar 
 */
gint main_window::on_delete_event(GdkEventAny* ev)
{
    if ( checkAndSaveData() ) {
	Gtk::Main::quit();
	return false;
    }
    return true;
}

void main_window::on_exit1_activate()
{   
    if ( checkAndSaveData() )
	Gtk::Main::quit();
}

void main_window::on_print_activate()
{   
    if(checkAndSaveData())
    {
        PrintDialog *pd = 
	    new PrintDialog(ctree1->get_current_selection()=="" ?
			    ctree1->get_current_selection().c_str():NULL
        );
        try
        {
            pd->run();
        } catch(FWException &ex)
        {
            MessageDialog::Error("Print error.",ex.toString());
        }
   delete pd;
    }
}

void main_window::on_edit1_activate()
{   
}

void main_window::on_options_activate()
{   
    OptionsDlg *pd=new OptionsDlg();
    pd->run();
    delete pd;
}

void main_window::on_about1_activate()
{   
    About::showAboutDialog();
}

               

void main_window::on_release_notes_activate()
{
    string file=DOCDIR "/ReleaseNotes_" VERSION ".html";

//    string url ="file://"+file;
//    gnome_url_show(url.c_str());

    MiniBrowserDialog *mbr=manage( new MiniBrowserDialog());
    mbr->loadFromFile( file );
    mbr->show();
}

void main_window::on_feedback_activate()
{
    string file=DOCDIR "/feedback.html";

    MiniBrowserDialog *mbr=manage( new MiniBrowserDialog());
    mbr->loadFromFile( file );
    mbr->show();
}

void main_window::showStatusbarMsg(const string& msg)
{
  main_statusbar->push(1,msg);
//  while (Gtk::Main::events_pending())
//      Gtk::Main::iteration(false);
}

void main_window::clearStatusbar()
{
  main_statusbar->pop(1);
//  while (Gtk::Main::events_pending())
//      Gtk::Main::iteration(false);
}

PolicyList* main_window::extractPolicyList()
{
    PolicyList *pl=NULL;

    if (current_right_pane_dialog) {
	DialogPlugin *dp= current_right_pane_dialog->getDialog();
	
	if (dp->get_name()=="policy_dialog")
	    pl=((PolicyDialog*)dp)->getPolicyList();
	
	if (dp->get_name()=="nat_dialog")
	    pl=((NATDialog*)dp)->getPolicyList();

	if (dp->get_name()=="firewall_dialog")
	    pl=((FirewallDialog*)dp)->getPolicyList();
    }
    return pl;
}

/*
 * Submenu Policy
 */
void main_window::on_insert_top()    
{ 
    if (current_right_pane_dialog==NULL) return;
    PolicyList *pl=extractPolicyList();
    if (pl!=NULL) pl->insertRuleAtTop(); 
}

void main_window::on_append_bottom() 
{ 
    if (current_right_pane_dialog==NULL) return;
    PolicyList *pl=extractPolicyList();
    if (pl!=NULL) pl->appendRuleAtBottom();  
}

void main_window::on_add_rule()
{
    if (current_right_pane_dialog==NULL) return;
    PolicyList *pl=extractPolicyList();
    if (pl!=NULL) pl->appendRuleAfter(-1);
}

void main_window::on_remove_rule()   
{ 
    if (current_right_pane_dialog==NULL) return;
    PolicyList *pl=extractPolicyList();
    if (pl!=NULL) pl->delRule(-1);
}

void main_window::on_move_up()       
{ 
    if (current_right_pane_dialog==NULL) return;
    PolicyList *pl=extractPolicyList();
    if (pl!=NULL) pl->moveRuleUp(-1);
}

void main_window::on_move_down()     
{
    if (current_right_pane_dialog==NULL) return;
    PolicyList *pl=extractPolicyList();
    if (pl!=NULL) pl->moveRuleDown(-1);
}

void main_window::on_build_autorules()
{
    if (current_right_pane_dialog==NULL) return;

    if (safe_to_close_right_pane()) {

	FWObject   *obj=nowShowingObject();

	while (obj && ! Firewall::isA(obj)) obj=obj->getParent();
	if (obj) {
//		ClearRightPane(false);

	    StandardRulesDruid *dr=new StandardRulesDruid( obj->getId(),
						   "Standard policy rules",
					   obj->getXPMIconFileName("_dlg") );
	    dr->show();
	}

    }
}

void main_window::on_compile()
{
    if (current_right_pane_dialog==NULL) return;

    if (safe_to_close_right_pane()) {
	FWObject   *obj=nowShowingObject();
	if (obj) {
	    if (Policy::isA(obj)) obj=obj->getParent();
	    if (NAT::isA(obj))    obj=obj->getParent();

	    if (Firewall::isA(obj))   {
		on_save1_activate();
// call compiler now

		Firewall *fw=Firewall::cast(obj);
		FirewallOptions *fwopt=fw->getOptionsObject();
		
		string compiler=fwopt->getStr("compiler");
		if (compiler=="") {
		    compiler=Resources::global_res->getCompiler(
			obj->getStr("platform"));
		}
            
		ExecDialog *cmpl=
		    new ExecDialog(compiler , 
				   fwopt->getStr("cmdline") ,
				   FWObjectDatabase::db->getFileName(),
				   obj->getName() ,
				   Preferences::global_prefs->getWdir()
		    );

		cmpl->execute();
		delete cmpl;
	    }
	}
    }
}

void main_window::on_install()
{
    if (current_right_pane_dialog==NULL) return;

    if (safe_to_close_right_pane()) {
	FWObject   *obj=nowShowingObject();
	if (obj) {
	    if (Policy::isA(obj)) obj=obj->getParent();
	    if (NAT::isA(obj))    obj=obj->getParent();

	    if (Firewall::isA(obj))   {
		on_save1_activate();
// call install script now

		Firewall *fw           = Firewall::cast(obj);
		FirewallOptions *fwopt = fw->getOptionsObject();
		
		string inst_script=fwopt->getStr("inst_script");
		if (inst_script=="") {
// TODO: may be Error dialog "install script is not defined"
		    return ;
		}
            
		ExecDialog *inst=
		    new ExecDialog(inst_script , 
				   fwopt->getStr("inst_cmdline") ,
				   FWObjectDatabase::db->getFileName(),
				   obj->getName() ,
				   Preferences::global_prefs->getWdir()
		    );

		inst->execute();
		delete inst;
	    }
	}
    }
}

void main_window::RefreshRightPane()
{
    PolicyList    *pl=extractPolicyList();
    if (pl) pl->schedule_rebuild(0);
}

string main_window::nowShowingObjectId()
{
    FWObject   *obj=nowShowingObject();
    return (obj)?obj->getId():"";
}

FWObject* main_window::nowShowingObject()
{
    DialogPlugin *dp= current_right_pane_dialog->getDialog();
    return (dp)?dp->getObject():NULL;
}

void main_window::on_tools_scan()
{
    string icn=
	Resources::global_res->getResourceStr("/FWBuilderResources/Paths/Icndir")+"/"+
	Resources::global_res->getResourceStr("/FWBuilderResources/UI/Icons/Discover");

    DiscoveryDruid *dr=new DiscoveryDruid("Objects Discovery",icn);

    dr->show();
}
