/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: main_window2.cc,v 1.12 2001/07/22 06:13:19 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <iostream>
#include <gtk--/base.h>
#include <gtk--/main.h>
#include <gtk--/tree.h>

#include "helpers.hh"

#include "main_window.hh"

#include "FWObjectDatabase.hh"
#include "Host.hh"
#include "Firewall.hh"
#include "Network.hh"

#include "BuiltinDialog.hh"
#include "Preferences.hh"

#include "gen_popup_menu.hh"


#include "MessageDialog.hh"


bool main_window::safe_to_close_right_pane()
{
  if ( current_right_pane_dialog!=NULL &&
       current_right_pane_dialog->IsDataChanged()) {
    // unsaved changes exist
      
      if ( Preferences::global_prefs->getOptBool("/FWBuilderPreferences/UI/Autosave") ) {
	  Gtk::Widget *w=((Gtk::Bin*)right_pane_port)->get_child();
	  if (w) {
	      BuiltinDialog *dlg = dynamic_cast<BuiltinDialog*>(w);
	      if (dlg)
		  dlg->SaveData();
	  }
	  return true;

      } else {

	  MessageDialog::Warning(
"Data in the current dialog
has been modified. Please
save or discard your changes.");

	  return false;
      }
  }
  return true;
}


void main_window::schedule_open_object(string id)
{
    oo_obj=id;
    Gtk::Main::idle.connect(slot(this,&main_window::open_object_when_idle));
}

gint main_window::open_object_when_idle()
{
    if (oo_obj=="") return false;
    OpenObject(oo_obj);
    oo_obj="";
    return false;
}

void main_window::ClearRightPane(bool logo)
{
    if(right_pane_port)
    {
        Gtk::Widget *w=((Gtk::Bin*)right_pane_port)->get_child();
        if (w!=0) {
            //    w->hide();
            ((Gtk::Bin*)right_pane_port)->remove(); 
            /*
             * do not delete it if it is Gtk::Label - we use label as
             * a place holder so we could build dialog in
             * glade. Destructor in main_window_glade deletes this
             * label, so let's leave it alive until then 

	     if ( ! Gtk::Label::isA(w))  delete w;

	     */

	    current_right_pane_dialog=NULL;
        }
    }      

    if (logo) 
    {
        /*
         *   TODO:  show logo in the right pane if there is going to be no dialog
         */
    }

    Gtk::Widget *itm;

    itm=find_widget("duplicate",main_menubar1);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("copy",main_menubar1);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("cut",main_menubar1);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("paste",main_menubar1);
    if (itm) itm->set_sensitive(false);
}

bool main_window::OpenObject (string id)
{
    Gtk::Widget   *right_pane;
    BuiltinDialog *dptr;

    if ( ! safe_to_close_right_pane() ) return(false);

    ClearRightPane(false);

    dptr=NULL;
    dptr = manage(new BuiltinDialog(id));

    if (dptr!=NULL) {
	right_pane = dptr;
    
	//    right_pane_port->remove(); 
	((Gtk::EventBox*)right_pane_port)->add( *dptr );
	dptr->show();
	right_pane_port->show();
	
	current_right_pane_dialog=dptr;
    }

    ctree1->ShowObject(id);

    FWObject      *obj=FWObjectDatabase::db->getById( id , true);
    obj->updateMainMenu();
  
    return(true);
}






