/* 

                          Firewall Builder

                 Copyright (C) 2000,2001 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: iptables.c,v 1.64 2001/07/25 06:30:57 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Puplic License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "iptables.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>

FILE *ofile;
int   dynamic_address=0;

char  global_log_parameters[256];

char  global_log_level[256];
char  global_log_prefix[256];
char  global_log_tcp_seq[256];
char  global_log_tcp_opt[256];
char  global_log_ip_opt[256];
char  global_log_limit[256];
char  global_action_on_reject[256];
int   global_no_optimisation=0;
int   global_no_state=0;
int   global_manage_arp=0;

int   generate_input_output_groups_for_any=1;


char rule_chain[64];
char temp_i_chain[64];
char temp_o_chain[64];
char temp_f_chain[64];

char temp_n_chain[64];

 

void parseOptions(xmlNodePtr options,
		  char* log_level,
		  char* log_prefix,
		  char* log_tcp_seq,
		  char* log_tcp_opt,
		  char* log_ip_opt,
		  char* limit,
		  char* action_on_reject,
		  int * no_optimisation,
		  int * no_state,
		  int * manage_arp)
{
    const char* cptr;
    int         val;
    
    strcpy(log_level,"");
    strcpy(log_prefix,"");
    strcpy(log_tcp_seq,"");
    strcpy(log_tcp_opt,"");
    strcpy(log_ip_opt,"");
    strcpy(limit,"");
    strcpy(action_on_reject,"");

    cptr=getOptionStr(options,"log_level");
    if (cptr && strlen(cptr)) {
	sprintf(log_level," --log-level %s",cptr);
    }
    cptr=getOptionStr(options,"log_prefix");
    if (cptr && strlen(cptr)) {
	sprintf(log_prefix," --log-prefix \"%s\"",cptr);
    }
    cptr=getOptionStr(options,"log_tcp_seq");
    if (cptr && strcasecmp(cptr,"true")==0) {
	strcpy(log_tcp_seq," --log-tcp-sequence ");
    }
    cptr=getOptionStr(options,"log_tcp_opt");
    if (cptr && strcasecmp(cptr,"true")==0) {
	strcpy(log_tcp_opt," --log-tcp-options ");
    }
    cptr=getOptionStr(options,"log_ip_opt");
    if (cptr && strcasecmp(cptr,"true")==0) {
	strcpy(log_ip_opt," --log-ip-options ");
    }
    val=getOptionInt(options,"log_limit_value");

    if (val!=0) {
	sprintf(limit," -m limit --limit %d",val);
	cptr=getOptionStr(options,"log_limit_suffix");
	if (cptr && strlen(cptr)) strcat(limit,cptr);
    }
    val=getOptionBool(options,"no_optimisation");
    if (val) {
	*no_optimisation=val;
    }
    val=getOptionBool(options,"stateless");
    if (val) {
	*no_state=val;
    }
    val=getOptionBool(options,"manage_arp");
    if (val) {
	*manage_arp=val;
    }
    cptr=getOptionStr(options,"action_on_reject");
    if (cptr && strlen(cptr)) {
	if (strstr(cptr,"TCP")) {
	    strcpy(action_on_reject,"-p tcp --reject-with tcp-reset");
	}

	if (strstr(cptr,"ICMP")) {
	    if (strstr(cptr,"unreachable")) {
		strcpy(action_on_reject," --reject-with ");
		if (strstr(cptr,"net"))
		    strcat(action_on_reject,"icmp-net-unreachable");
		if (strstr(cptr,"host"))
		    strcat(action_on_reject,"icmp-host-unreachable");
		if (strstr(cptr,"port"))
		    strcat(action_on_reject,"icmp-port-unreachable");
		if (strstr(cptr,"proto"))
		    strcat(action_on_reject,"icmp-proto-unreachable");
	    }
	    if (strstr(cptr,"prohibited")) {
		strcpy(action_on_reject," --reject-with ");
		if (strstr(cptr,"net"))
		    strcat(action_on_reject,"icmp-net-prohibited");
		if (strstr(cptr,"host"))
		    strcat(action_on_reject,"icmp-host-prohibited");
	    }
	}

    }
}

void parseFirewallOptions(xmlNodePtr options)
{
    parseOptions(options,
		 global_log_level,    
		 global_log_prefix,
		 global_log_tcp_seq,
		 global_log_tcp_opt,
		 global_log_ip_opt,
		 global_log_limit,
		 global_action_on_reject,
		 &global_no_optimisation,
		 &global_no_state,
		 &global_manage_arp);

    generate_input_output_groups_for_any=
	getOptionBool(options,"firewall_is_part_of_any_and_networks");
}


void printARPEntryCommands(xmlNodePtr firewall,
			   xmlNodePtr odst,
			   xmlNodePtr tdst)
{
    char od[80];
    char td[80];
    xmlNodePtr c1;

    if (odst==NULL || tdst==NULL) return;
    
    processSrcAndDst(odst,od);
    processSrcAndDst(tdst,td);


    for(c1=firewall->xmlChildrenNode; c1; c1=c1->next) {
	if ( xmlIsBlankNode(c1) ) continue;
	if (isElement(c1,"Interface")) {
	    if ( isDynamicInterface(c1) ) continue;

	    if (belongsToNet( c1 , od )) 
		fprintf(ofile,"/sbin/arp -Ds %s %s pub\n", od, getName(c1) );

	    if (belongsToNet( c1 , td )) 
		fprintf(ofile,"/sbin/route add %s dev %s\n", od, getName(c1) );

	}
    }

}



void prologue(xmlNodePtr firewall,xmlNodePtr fw_options)
{
    char ofname[256];
    sprintf(ofname,"%s.fw",getName(firewall));

    if ( (ofile=fopen(ofname,"w"))==NULL ) {
	printf("Error: Could not open output file %s for writing\n",ofname);
	exit(1);
    }
    

    if (getOptionBool(fw_options,"debug"))
	fprintf(ofile,"#!/bin/sh -x\n\n");
    else
	fprintf(ofile,"#!/bin/sh\n\n");
	
    if (getOptionBool(fw_options,"load_modules")) {
	fprintf(ofile,"modprobe ip_conntrack\n");
	fprintf(ofile,"modprobe ip_conntrack_ftp\n");
	fprintf(ofile,"modprobe ip_nat_ftp\n");
    }

/*
 *    Turn on packet forwarding
 */
    if (getOptionBool(fw_options,"pkt_fwd")) 
	fprintf(ofile,"echo \"1\" > /proc/sys/net/ipv4/ip_forward\n\n");

/*
 *    Turn on proxy ARP
 */
    if (getOptionBool(fw_options,"proxy_arp")) 
	fprintf(ofile,"echo \"1\" > /proc/sys/net/ipv4/conf/all/proxy_arp\n\n");

    parseFirewallOptions( fw_options );

    fprintf(ofile,"iptables -P OUTPUT  DROP\n");
    fprintf(ofile,"iptables -P INPUT   DROP\n");
    fprintf(ofile,"iptables -P FORWARD DROP\n");

    fprintf(ofile,"\n\n");

/*
 *    flush all the rules which existed before
 */
    fprintf(ofile,
 "iptables -L -n | awk '/Chain/ {printf \"iptables -F %%s\\n\", $2;}'|/bin/sh\n");

    fprintf(ofile,
 "iptables -t nat -L -n | awk '/Chain/ {printf \"iptables -t nat -F %%s\\n\", $2;}'|/bin/sh\n");
    
    fprintf(ofile,"iptables -X \n");
    fprintf(ofile,"iptables -t nat -X \n");
     
    fprintf(ofile,"\n");

/*
 *  Remove all host static routes and "pub" ARP entries if we will be 
 *  creating new ones
 */

    if (global_manage_arp) {

	fprintf(ofile,"
/bin/netstat -rn | /usr/bin/awk \'{ 
    if ($3==\"255.255.255.255\") { printf \"route delete %%s\\n\",$1;}
}\' | /bin/sh

/sbin/arp -an |   /usr/bin/awk \'/PERM/ {
    iface=$NF; addr=$2;
    gsub(\"[()]\",\"\",addr);
    printf \"arp -i %%s -d %%s pub\\n\",iface,addr;
}\' | /bin/sh
");

	fprintf(ofile,"\n");



    }

    
/*    fprintf(ofile,"iptables -N nat\n"); */

    fprintf(ofile,"\n\n\n");
    
    /*
       --state state
              Where  state  is a comma separated list of the con
              nection  states  to  match.   Possible  states  are
              INVALID  meaning that the packet is associated with
              no known connection, ESTABLISHED meaning  that  the
              packet  is  associated  with a connection which has
              seen packets in both directions, NEW  meaning  that
              the  packet has started a new connection, or other
              wise associated with a  connection  which  has  not
              seen  packets in both directions, and RELATED mean
              ing that the packet is starting a  new  connection,
              but is associated with an existing connection, such
              as an FTP data transfer, or an ICMP error.
     */

    fprintf(ofile,
    "iptables -A INPUT     -m state --state ESTABLISHED,RELATED -j ACCEPT\n");
    fprintf(ofile,
    "iptables -A OUTPUT    -m state --state ESTABLISHED,RELATED -j ACCEPT\n");
    fprintf(ofile,
    "iptables -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT\n");

}

void epilogue(xmlNodePtr firewall,xmlNodePtr fw_options)
{
    char ofname[256];
    sprintf(ofname,"%s.fw",getName(firewall));



/*
 *   Catch all rules
 */
    fprintf(ofile,"#\n#  Final rules\n#\n");

    fprintf(ofile,"iptables -A INPUT      -j DROP\n");
    fprintf(ofile,"iptables -A OUTPUT     -j DROP\n");
    fprintf(ofile,"iptables -A FORWARD    -j DROP\n");
/*    fprintf(ofile,"iptables -A nat        -j DROP\n"); */

    fclose(ofile);

    chmod(ofname,S_IXUSR|S_IRUSR|S_IWUSR|S_IRGRP);
}


int processSrcAndDst(xmlNodePtr obj,char *buff)
{
    const char *addr;

    *buff='\0';
    
    addr=getStr(obj,"address");
    if (addr && strlen(addr)==0) return(0);
    
    if (isElement(obj,"Network")) {
	sprintf(buff,"%s/%s",getStr(obj,"address"),getStr(obj,"netmask"));
	if (strcmp(buff,"0.0.0.0/0.0.0.0")==0) strcpy(buff,"0/0");
	return(1);
    } 
    if (isAnyNetwork(obj)) {
	strcpy(buff,"0/0");
	return(1);  
    }
    if (addr)
	strcpy(buff,addr);
    return(1);
}


void processSrc(xmlNodePtr src,triplet *tr)
{
    char str[80];

    if(src==NULL) { tr->p_src=NULL; return; }
    
    processSrcAndDst(src,str);
    tr->p_src=strdup(str);
}

void processDst(xmlNodePtr dst,triplet *tr)
{
    char str[80];

    if(dst==NULL) { tr->p_dst=NULL; return; }
    
    processSrcAndDst(dst,str);
    tr->p_dst=strdup(str);
}



void processSrv(xmlNodePtr srv,triplet *tr)
{
    char sres[128], dres[128];
    int  sps,spe,dps,dpe;
    int  syn_flag, ack_flag, fin_flag, rst_flag, urg_flag, psh_flag;
    int  z;
    
    sps=spe=dps=dpe=0;


    if (isElement(srv,"TCPService") || isElement(srv,"UDPService") ) {
    
	strcpy(sres,"");
	strcpy(dres,"");
    
	sps=getInt(srv,"src_range_start");
	spe=getInt(srv,"src_range_end");
	dps=getInt(srv,"dst_range_start");
	dpe=getInt(srv,"dst_range_end");

	if ( spe!=0 ) {
	    if (sps==spe)  sprintf(sres,"%d",sps);
	    else           sprintf(sres,"%d:%d",sps,spe);
	}
	else {
	    if (sps!=0)    sprintf(sres,"%d",sps);
	}    

	if ( dpe!=0 ) {
	    if (dps==dpe)  sprintf(dres,"%d",dps);
	    else           sprintf(dres,"%d:%d",dps,dpe);
	}
	else {
	    if (dps!=0)    sprintf(dres,"%d",dps);
	}    
    

	tr->p_sprt=strdup(sres);
	tr->p_dprt=strdup(dres);

	if (isElement(srv, "UDPService")) {
	    tr->p_proto      = "udp";
	}
    
	if (isElement(srv, "TCPService")) {
	    tr->p_proto      = "tcp";
	    syn_flag=getBool(srv,"syn_flag");
	    ack_flag=getBool(srv,"ack_flag");
	    fin_flag=getBool(srv,"fin_flag");
	    rst_flag=getBool(srv,"rst_flag");
	    psh_flag=getBool(srv,"psh_flag");
	    urg_flag=getBool(srv,"urg_flag");
	    if (syn_flag ||  ack_flag ||  fin_flag ||
		rst_flag ||  psh_flag ||  urg_flag) {
		if (syn_flag && (!ack_flag && !fin_flag &&
				 !rst_flag && !psh_flag && !urg_flag) )
		    addOptionCode(tr,"--syn");
		else {
		    z=0;
		    addOptionCode(tr,"--tcp-flags ALL ");
		    if (syn_flag) {
			addOptionCode(tr,"SYN");
			z=1;
		    }
		    if (ack_flag) {
			if (z++) addOptionCode(tr,",");
			addOptionCode(tr,"ACK");
		    }
		    if (fin_flag) {
			if (z++) addOptionCode(tr,",");
			addOptionCode(tr,"FIN");
		    }
		    if (rst_flag) {
			if (z++) addOptionCode(tr,",");
			addOptionCode(tr,"RST");
		    }
		    if (psh_flag) {
			if (z++) addOptionCode(tr,",");
			addOptionCode(tr,"PSH");
		    }
		    if (urg_flag) {
			if (z++) addOptionCode(tr,",");
			addOptionCode(tr,"URG");
		    }
		}
	    }
	}
    }

    if (isElement(srv,"IPService") ){
	if (getInt(srv,"protocol_num")==0) tr->p_proto=NULL;
	else  tr->p_proto = strdup( getStr(srv,"protocol_num") );

	if (getBool(srv,"fragm")) {
	    addOptionCode(tr,"-f");
	    tr->state=0;
	}
	if (getBool(srv,"short_fragm")) {
	    addOptionCode(tr,"-f");
	    tr->state=0;
	}
    }

    if (isElement(srv,"ICMPService") ) {
	tr->p_proto      = "icmp";
	tr->p_icmp_type  = strdup( getStr(srv,"type") );
	tr->p_icmp_code  = strdup( getStr(srv,"type") );
    }


}


