/* 

                          Firewall Builder

                 Copyright (C) 2000,2001 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: nat.c,v 1.12 2001/07/25 06:32:15 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Puplic License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "iptables.h"

#include <stdlib.h>
#include <string.h>

static int         snat, dnat,redir;
static int         ofs,ofd,tfs,tfd;

static GSList     *ready_nat_subrules;


void NatRulePrologue(xmlNodePtr firewall,
		     xmlNodePtr rule,
		     int subrules_n)
{
    xmlNodePtr osrc,odst,osrv, tsrc,tdst,tsrv;
    int           num=getInt(rule,"position");
    const char* comm =getStr(rule,"comment");

    snat=dnat=redir=0;
    
    osrc=getFirstChildXmlNode(rule,"OSrc");
    odst=getFirstChildXmlNode(rule,"ODst");
    osrv=getFirstChildXmlNode(rule,"OSrv");

    tsrc=getFirstChildXmlNode(rule,"TSrc");
    tdst=getFirstChildXmlNode(rule,"TDst");
    tsrv=getFirstChildXmlNode(rule,"TSrv");


    fprintf(ofile,"#\n");
    fprintf(ofile,"#   NAT Rule #%d\n" , num );
    fprintf(ofile,"#\n");
    if (comm) {
	fprintf(ofile,"#    %s\n",comm);
	fprintf(ofile,"#\n");
    }

    if ( getBool(tsrc,"neg") || getBool(tdst,"neg") || getBool(tsrv,"neg") ) {
	fprintf(stderr,"Negation is not supported in 'translated source' and 'translated destination' in rule #%d\n",num);
	exit(1);
    }

    
    if (getBool(osrc,"neg") || getBool(odst,"neg") || getBool(osrv,"neg") ) {
        sprintf(temp_n_chain,"T_N_RULE_%d"    ,num);
	fprintf(ofile,"iptables -t nat -N %s\n\n",temp_n_chain);
    }
    else 
	strcpy(temp_n_chain,"");

}

void NatRuleEpilogue(xmlNodePtr firewall,
		     xmlNodePtr rule,
		     elementaryNatRule *last_erule)
{
    xmlNodePtr osrc,odst,osrv, tsrc,tdst,tsrv;
    xmlNodePtr c1;
    const char *ifaddr;
    elementaryNatRule *trule;

    osrc=getFirstChildXmlNode(rule,"OSrc");
    odst=getFirstChildXmlNode(rule,"ODst");
    osrv=getFirstChildXmlNode(rule,"OSrv");

    tsrc=getFirstChildXmlNode(rule,"TSrc");
    tdst=getFirstChildXmlNode(rule,"TDst");
    tsrv=getFirstChildXmlNode(rule,"TSrv");


    if (getBool(osrc,"neg") || getBool(odst,"neg") || getBool(osrv,"neg") ) {

	for(c1=firewall->xmlChildrenNode; c1; c1=c1->next) {
	    if ( xmlIsBlankNode(c1) ) continue;
	    if (isElement(c1,"Interface")) {
		if ( ! isExternalInterface(c1) ) continue; 

		trule=createElementaryNatRule();

		copyElementaryNatRule(trule,last_erule);
		if (getBool(osrc,"neg"))  trule->o->p_src=NULL;
		if (getBool(odst,"neg"))  trule->o->p_dst=NULL;

		trule->p_group = strdup(temp_n_chain);

		ifaddr=getStr(c1,"address");
		if (strcmp(ifaddr,"127.0.0.1")==SAME) continue;

		trule->p_iface = getName(c1);

		if (ofd && dnat) {
		    if ( isDynamicInterface(c1) ) {
			trule->p_action="DNAT";
			trule->odst=NULL;
		    } else {
			trule->p_action="DNAT";
			trule->odst=c1;
		    }
		    
		    parseNatRule( rule,trule);
		}
		if (tfs && snat) {
 		    if ( isDynamicInterface(c1) ) {
			trule->p_action="MASQUERADE";
			trule->tsrc=NULL;
		    } else {
			trule->p_action="SNAT";
			trule->tsrc=c1;
		    }
		    trule->odst=NULL;
		    
		    parseNatRule( rule,trule);
		}
	    }
	} 


    }
}


void addARPEntries(xmlNodePtr firewall)
{
    GSList            *list_ptr1;
    elementaryNatRule *erule;
    
    for (list_ptr1=ready_nat_subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {
	
	erule=(elementaryNatRule*)(list_ptr1->data);
	
	if (strcmp(erule->p_action,"DNAT")==SAME && 
	    cmpObjects(erule->odst , firewall)==SAME )

	    printARPEntryCommands(firewall, erule->odst, erule->tdst);
    }
}

void elementaryNatRulePrologue(xmlNodePtr firewall,
			       xmlNodePtr rule,
			       elementaryNatRule *erule)
{
    elementaryNatRule *trule;


    erule->p_group = "POSTROUTING";

    
    if (erule->osrc_neg || erule->odst_neg || erule->osrv_neg ) {
	if (snat) {
	    trule=createElementaryNatRule();
	    trule->p_group = "POSTROUTING";
	    trule->p_action  = strdup(temp_n_chain);
	    storeNatRule(trule);
	    if ( cmpObjects(erule->osrc , firewall)==SAME ) {
		trule=createElementaryNatRule();
		trule->p_group = "OUTPUT";
		trule->p_action  = strdup(temp_n_chain);
		storeNatRule(trule);
	    }
	}
	if (dnat) {
	    trule=createElementaryNatRule();
	    trule->p_group = "PREROUTING";
	    trule->p_action  = strdup(temp_n_chain);
	    storeNatRule(trule);
	}
	erule->p_group = strdup(temp_n_chain);
	erule->p_action  = "RETURN";

    } else {
	if (snat) {
	    erule->p_group=
		(cmpObjects(erule->osrc , firewall)==SAME)?"OUTPUT":"POSTROUTING";
	}
	if (dnat || redir) {
	    erule->p_group="PREROUTING";
/*	    erule->p_action  = "DNAT"; */
	}
    }

}

void elementaryNatRuleEpilogue(xmlNodePtr firewall,
			       xmlNodePtr rule,
			       elementaryNatRule *erule)
{

}

void parseNatRule(xmlNodePtr rule,
		  elementaryNatRule *erule)
{

    processSrc(erule->osrc,erule->o);
    processDst(erule->odst,erule->o);
    
    processSrc(erule->tsrc,erule->t);
    processDst(erule->tdst,erule->t);

    if (erule->osrv==NULL || isAny(erule->osrv) ) {
	storeNatRule(erule);
	return;
    }
    
    processSrv(erule->osrv,erule->o);
    processSrv(erule->tsrv,erule->t);

    /*
     *  TODO: sanity checking
     */

    storeNatRule(erule);
}


void processNATRule(xmlNodePtr firewall,
		    xmlNodePtr rule,
		    const char *comment,
		    GSList    *subrules)
{
    elementaryNatRule *erule, *trule, *last_erule;
    GSList            *list_ptr1;
    const char        *ifaddr;
    xmlNodePtr         c1;
    int                srn;
    const char        *snat_addr;

    
    snat=dnat=0;
    ready_nat_subrules=NULL;
    
    NatRulePrologue(firewall, rule, g_slist_length(subrules) );

    srn=0;
    for (list_ptr1=subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {

	erule=(elementaryNatRule*)(list_ptr1->data);
	
	if (erule->num==-1) continue;

	erule->subrule_no=srn;


	ofs=ofd=tfs=tfd=0;
	if ( cmpObjects(erule->osrc , firewall)==SAME ) ofs=1;
	if ( cmpObjects(erule->odst , firewall)==SAME ) ofd=1;
	if ( cmpObjects(erule->tsrc , firewall)==SAME ) tfs=1;
	if ( cmpObjects(erule->tdst , firewall)==SAME ) tfd=1;

	if ( isAny(erule->tdst) ) {
	    snat=1;
	    snat_addr = getStr(erule->tsrc,"address");
	    if (snat_addr && strcmp(snat_addr,"0.0.0.0")==SAME)
		erule->p_action="ACCEPT";  /* special case - no translation */
	    else
		erule->p_action = "SNAT";
	} else
    
	    if ( tfd && isAny(erule->tsrc) ) {
		redir=1;
		erule->p_action  = "REDIRECT";
	    } else
    
		if ( ! isAny(erule->odst) &&
		     isAny(erule->tsrc) &&
		     ! isAny(erule->tdst) ) {
		    dnat=1;
		    erule->p_action  = "DNAT";
		}

	if (snat==0 && dnat==0 && redir==0) {
	    fprintf(stderr,"Unsupported translation in rule #%d\n",erule->num);
	    exit(1);
	}
    

	
	if (ofs || ofd || tfs) {
    
	    for(c1=firewall->xmlChildrenNode; c1; c1=c1->next) {
		if ( xmlIsBlankNode(c1) ) continue;
		if (isElement(c1,"Interface")) {
		    if ( ! isExternalInterface(c1) ) continue; 

		    ifaddr=getStr(c1,"address");
		    if (strcmp(ifaddr,"127.0.0.1")==SAME) continue;

		    trule=createElementaryNatRule();
		    copyElementaryNatRule(trule,erule);

		    if (ofd && dnat) {
			if ( isDynamicInterface(c1) ) 
			    trule->odst=NULL;
			else
			    trule->odst=c1;

			trule->p_iface = getName(c1);

			elementaryNatRulePrologue(firewall,rule,trule);
			parseNatRule(rule,trule);
			elementaryNatRuleEpilogue(firewall,rule,trule);
		    }
		    if (tfs && snat) {
			if ( isDynamicInterface(c1) ) {
			    trule->tsrc=NULL;
			    trule->p_action = "MASQUERADE";
			} else
			    trule->tsrc=c1;

			trule->p_iface = getName(c1);

			elementaryNatRulePrologue(firewall,rule,trule);
			parseNatRule(rule,trule);
			elementaryNatRuleEpilogue(firewall,rule,trule);
		    }
		    
		    last_erule=trule;
		}
	    } 

	} else {
	
	    if (tfd && redir) {
		trule=createElementaryNatRule();
		copyElementaryNatRule(trule,erule);
		trule->tdst=NULL;
		
		elementaryNatRulePrologue(firewall,rule,trule);
		parseNatRule(rule,trule);
		elementaryNatRuleEpilogue(firewall,rule,trule);
		
		last_erule=trule;

	    } else {
		elementaryNatRulePrologue(firewall,rule,erule);
		parseNatRule(rule,erule);
		elementaryNatRuleEpilogue(firewall,rule,erule);
		if (global_manage_arp && dnat)
		    printARPEntryCommands(firewall, erule->odst, erule->tdst);
		last_erule=erule;
	    }
	}

	srn++;
    }

    NatRuleEpilogue(firewall,rule,last_erule);

    optimiseNatRules();
    
/*    if (global_manage_arp) 
	addARPEntries(firewall);
*/ 
    printNatRule();
    
    return;
}

void storeNatRule(elementaryNatRule *erule)
{
    if (strcmp(erule->p_action,"RETURN")==SAME ) 	{
	erule->p_iface=NULL;
	erule->t->p_dst=NULL;
	erule->t->p_src=NULL;
    }
    ready_nat_subrules=g_slist_append(ready_nat_subrules , erule);
}

void optimiseNatRules()
{
    elementaryNatRule     *nrptr1, *nrptr2;
    GSList                *list_ptr1, *list_ptr2;

 again:
    for (list_ptr1=ready_nat_subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {

	nrptr1=(elementaryNatRule*)(list_ptr1->data);
	if (nrptr1->num==-1) continue;
	    
	for (list_ptr2=g_slist_next(list_ptr1);
	     list_ptr2!=NULL;
	     list_ptr2=g_slist_next(list_ptr2)) {

	    nrptr2=(elementaryNatRule*)(list_ptr2->data);

	    if (cmpPNatRules( nrptr1,nrptr2 )==SAME) {
		ready_nat_subrules=g_slist_remove(ready_nat_subrules, nrptr2);
		goto again;
	    }
	}
    }
}

void printNatRule()
{
    elementaryNatRule     *erule;
    GSList                *list_ptr1;


    
    for (list_ptr1=ready_nat_subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {
	
	erule=(elementaryNatRule*)(list_ptr1->data);
	    
	fprintf(ofile,"iptables -t nat -A %s ",erule->p_group);

	if (erule->p_iface) {
	    if (strcmp(erule->p_action,"SNAT")==SAME ||
		strcmp(erule->p_action,"MASQUERADE")==SAME)
		fprintf(ofile," -o %s",erule->p_iface);
	    if (strcmp(erule->p_action,"DNAT")==SAME )
		fprintf(ofile," -i %s",erule->p_iface);
	}

	if ( erule->o->p_proto ) {
	    fprintf(ofile," -p %s",erule->o->p_proto);
	} else
	    if ( erule->t->p_proto ) {
		fprintf(ofile," -p %s",erule->t->p_proto);
	    }
    
	if (erule->o->p_src!=NULL) fprintf(ofile," -s %s",erule->o->p_src);

	if ( erule->o->p_sprt && strlen(erule->o->p_sprt)!=0)
	    fprintf(ofile," --source-port %s",erule->o->p_sprt);
	
	if (erule->o->p_dst!=NULL) fprintf(ofile," -d %s",erule->o->p_dst);
    
	if (erule->o->p_dprt && strlen(erule->o->p_dprt)!=0)
	    fprintf(ofile," --destination-port %s",erule->o->p_dprt);
	
	if ( erule->o->p_icmp_type )
	    fprintf(ofile," --icmp-type %s", erule->o->p_icmp_type);
	
	if (erule->p_action!=NULL)	fprintf(ofile," -j %s",erule->p_action);

	if (strcmp(erule->p_action,"SNAT")==SAME && erule->t->p_src!=NULL) {
	    fprintf(ofile," --to %s", erule->t->p_src);
	}

	if (strcmp(erule->p_action,"DNAT")==SAME && erule->t->p_dst!=NULL) {
	    fprintf(ofile," --to %s", erule->t->p_dst);
	}

	if (strcmp(erule->p_action,"REDIRECT")==SAME && erule->t->p_dprt!=NULL) {
	    fprintf(ofile," --to-ports %s", erule->t->p_dprt);
	}

	fprintf(ofile,"\n");
    }

}

