/* 

                          Firewall Builder

                 Copyright (C) 2000,2001 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: policy.c,v 1.20 2001/07/25 01:30:32 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Puplic License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "iptables.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
 
static GSList     *ready_subrules;

/*
 *  Rule options:
 */
char  log_level[256];
char  log_prefix[256];
char  log_tcp_seq[256];
char  log_tcp_opt[256];
char  log_ip_opt[256];
char  limit[256];
char  action_on_reject[256];
int   no_optimisation;
int   no_state;
int   manage_arp;    /* this does not apply as it may be only global
			option, but we have it here to satisfy list of
			arguments for parseOptions function */

void rulePrologue(xmlNodePtr firewall,
		  xmlNodePtr iface,
		  xmlNodePtr rule,
		  int subrules_n)
{
    xmlNodePtr src,dst,srv;
    xmlNodePtr  options;

    int           num=getInt(rule,"position");
    const char* comm =getStr(rule,"comment");
    

    strcpy(log_level,"");
    strcpy(log_prefix,"");
    strcpy(log_tcp_seq,"");
    strcpy(log_tcp_opt,"");
    strcpy(log_ip_opt,"");
    strcpy(limit,"");
    strcpy(action_on_reject,"");

    no_optimisation=0;
    no_state=0;

    options=getFirstChildXmlNode(rule,"PolicyRuleOptions");
    
    if ( options!=NULL )
	parseOptions( options,
		      log_level,
		      log_prefix,
		      log_tcp_seq,
		      log_tcp_opt,
		      log_ip_opt,
		      limit,
		      action_on_reject,
		      &no_optimisation,
		      &no_state,
		      &manage_arp);

    
    if (iface)  sprintf(rule_chain,"IRULE_%d_%s"  ,num ,getName(iface));
    else        sprintf(rule_chain,"RULE_%d"      ,num);


    fprintf(ofile,"#\n");
    fprintf(ofile,"#   %s Rule #%d\n", (iface!=NULL)?"Interface":"" , num );
    fprintf(ofile,"#\n");
    if (comm) {
	fprintf(ofile,"#    %s\n",comm);
	fprintf(ofile,"#\n");
    }

    fprintf(ofile,"iptables -N %s\n",rule_chain);

    src=getFirstChildXmlNode(rule,"Src");
    dst=getFirstChildXmlNode(rule,"Dst");
    srv=getFirstChildXmlNode(rule,"Srv");

    if (getBool(src,"neg") || getBool(dst,"neg") || getBool(srv,"neg") ) {

	if (iface)  {
	    sprintf(temp_i_chain,"TI_IRULE_%d_%s",num ,getName(iface));
	    sprintf(temp_o_chain,"TO_IRULE_%d_%s",num ,getName(iface));
	    sprintf(temp_f_chain,"TF_IRULE_%d_%s",num ,getName(iface));
	}
	else {
	    sprintf(temp_i_chain,"TI_RULE_%d"    ,num);
	    sprintf(temp_o_chain,"TO_RULE_%d"    ,num);
	    sprintf(temp_f_chain,"TF_RULE_%d"    ,num);
	}

	fprintf(ofile,"iptables -N %s\n",temp_i_chain);
	fprintf(ofile,"iptables -N %s\n",temp_o_chain);
	fprintf(ofile,"iptables -N %s\n",temp_f_chain);
	fprintf(ofile,"\n");
	
    }
    else {
	strcpy(temp_i_chain,"");
	strcpy(temp_o_chain,"");
	strcpy(temp_f_chain,"");
    }
}

void ruleEpilogue(xmlNodePtr firewall,
		  xmlNodePtr iface,
		  xmlNodePtr rule,
		  GSList     *subrules)
{
    GSList   *list_ptr1;
    elementaryRule *erule, *trule, *t2rule;

    xmlNodePtr src,dst,srv;
    char  tmpstr[1024], tmp2str[1024];
    char *cptr;
    char  action_chain[64];
    char  log_options[4096];

    const char  *action;
    int          num, log;


    src=getFirstChildXmlNode(rule,"Src");
    dst=getFirstChildXmlNode(rule,"Dst");
    srv=getFirstChildXmlNode(rule,"Srv");

    if (getBool(src,"neg") || getBool(dst,"neg") || getBool(srv,"neg")) {

	for (list_ptr1=subrules;
	     list_ptr1!=NULL;
	     list_ptr1=g_slist_next(list_ptr1)) {

	    erule=(elementaryRule*)(list_ptr1->data);

	    if (erule->num==-1)  continue;
	    if (erule->final==1) continue;
	    
	    trule=createElementaryPolicyRule();
	    copyElementaryPolicyRule(trule,erule);

	    if (trule->src_neg)  {
		trule->src=NULL;
		trule->t->p_src="0/0";
	    }
	    if (trule->dst_neg)  {
		trule->dst=NULL;
		trule->t->p_dst="0/0";
	    }
	    if (trule->srv_neg)  {
		trule->srv=NULL;
		trule->t->p_sprt=NULL;
		trule->t->p_dprt=NULL;
		trule->t->p_proto=NULL;
		trule->t->p_icmp_type=NULL;
		trule->t->p_icmp_code=NULL;
	    }

	    
	    trule->p_action  = copyStr(rule_chain);

	    if ( strcasecmp(erule->master_group,"INPUT")==SAME)
		trule->p_group   = copyStr(temp_i_chain);
	    if ( strcasecmp(erule->master_group,"OUTPUT")==SAME)
		trule->p_group   = copyStr(temp_o_chain);
	    if ( strcasecmp(erule->master_group,"FORWARD")==SAME)
		trule->p_group   = copyStr(temp_f_chain);

	    trule->t->state  = 0;
	    
	    if ( iface && (erule->p_direction==NULL || 
		 strcasecmp(erule->p_direction,"both")==SAME ) ) {

		t2rule=createElementaryPolicyRule();
		copyElementaryPolicyRule(t2rule,trule);
		
		trule->p_direction="inbound";
		trule->final     = 1;
		processElementaryPolicyRule(firewall,iface,rule,trule);

		t2rule->p_direction="outbound";
		t2rule->final     = 1;
		processElementaryPolicyRule(firewall,iface,rule,t2rule);

	    } else {
		trule->final     = 1;
		processElementaryPolicyRule(firewall,iface,rule,trule);
	    }
	}
    }

    action=getStr(rule,"action");
    log=getBool(rule,"log");
    num=getInt(rule,"position");

    strcpy(action_chain,"");
    strcpy(log_options,"");
    
    if (strcasecmp(action,"accept")==0)  strcat(action_chain,"ACCEPT");
    if (strcasecmp(action,"reject")==0) {
	strcat(action_chain,"REJECT");
	if (strlen(action_on_reject)) {
	    strcat(action_chain," ");
	    strcat(action_chain,action_on_reject);
	} else
	    if (strlen(global_action_on_reject)) {
		strcat(action_chain," ");
		strcat(action_chain,global_action_on_reject);
	    }
    }
    if (strcasecmp(action,"deny")==0)  strcat(action_chain,"DROP");
	
    if (log) {

	strcpy(tmpstr,(strlen(log_level)!=0)?log_level:global_log_level);
	strcat(tmpstr,(strlen(log_prefix)!=0)?log_prefix:global_log_prefix);
	strcat(tmpstr,(strlen(log_tcp_seq)!=0)?log_tcp_seq:global_log_tcp_seq);
	strcat(tmpstr,(strlen(log_tcp_opt)!=0)?log_tcp_opt:global_log_tcp_opt);
	strcat(tmpstr,(strlen(log_ip_opt)!=0)?log_ip_opt:global_log_ip_opt);
	strcat(tmpstr,(strlen(limit)!=0)?limit:global_log_limit);

	if ( (cptr=strstr(tmpstr,"%N"))!=NULL) {
	    strncpy(tmp2str, tmpstr, cptr-tmpstr);
	    tmp2str[cptr-tmpstr]='\0';
	    sprintf(tmp2str+strlen(tmp2str),"%d ",num);
	    strcat(tmp2str, cptr+2 );
	    strcpy(tmpstr,tmp2str);
	}
	
	if ( (cptr=strstr(tmpstr,"%A"))!=NULL) {
	    strncpy(tmp2str, tmpstr, cptr-tmpstr);
	    tmp2str[cptr-tmpstr]='\0';
	    strcat(tmp2str , action );
	    strcat(tmp2str, " ");
	    strcat(tmp2str, cptr+2 );
	    strcpy(tmpstr,tmp2str);
	}
	
	strcpy(log_options, global_log_parameters);
	strcat(log_options, " ");
	strcat(log_options, tmpstr);
	
	trule=createElementaryPolicyRule();
	trule->p_group      = copyStr(rule_chain);
	trule->p_action       = "LOG";
	trule->t->p_options = copyStr(log_options);
	if (no_state) trule->t->state= 0;
	storePolicyRule(trule);

    }
    
    trule=createElementaryPolicyRule();
    trule->p_group      = copyStr(rule_chain);
    trule->p_action       = copyStr(action_chain);
    trule->t->p_options = copyStr(limit);
    if (no_state) trule->t->state= 0;
    storePolicyRule(trule);
}



void processElementaryPolicyRule(xmlNodePtr firewall,
				 xmlNodePtr iface,
				 xmlNodePtr rule,
				 elementaryRule *erule)
{
    xmlNodePtr      c1;
    elementaryRule *trule;
    const char     *ifaddr;
    int             fs,fd;

    
    if (iface!=NULL && erule->p_iface==NULL) erule->p_iface=getName(iface);

    
    if (erule->final) {
	parsePolicyRule( rule,erule);
	return;
    }


    if ( iface && ( erule->p_direction==NULL ||
	 strcasecmp(erule->p_direction,"both")==SAME ) ) {

	trule=createElementaryPolicyRule();
	copyElementaryPolicyRule( trule , erule );
	trule->p_direction="inbound";
	if (no_state) trule->t->state= 0;
	processElementaryPolicyRule(firewall,iface,rule,trule);
    
	trule=createElementaryPolicyRule();
	copyElementaryPolicyRule( trule , erule );
	trule->p_direction="outbound";
	if (no_state) trule->t->state= 0;
	processElementaryPolicyRule(firewall,iface,rule,trule);

	return;
    }

    
    if ( (erule->p_direction==NULL ||
	  strcasecmp(erule->p_direction,"outbound")==0) &&
	 (generate_input_output_groups_for_any &&
	  erule->src && isAny(erule->src) ) ) {
	
	trule=createElementaryPolicyRule();
	copyElementaryPolicyRule( trule , erule );
	trule->src=firewall;
	if (no_state) trule->t->state= 0;
	
	processElementaryPolicyRule(firewall,iface,rule,trule);
    }

    if ( (erule->p_direction==NULL ||
	  strcasecmp(erule->p_direction,"inbound")==0) &&
	 (generate_input_output_groups_for_any &&
	  erule->dst && isAny(erule->dst) ) ) {
	
	trule=createElementaryPolicyRule();
	copyElementaryPolicyRule( trule , erule );
	trule->dst=firewall;
	if (no_state) trule->t->state= 0;

	processElementaryPolicyRule(firewall,iface,rule,trule);
    }

    if ( ( erule->p_direction==NULL ||
	 strcasecmp(erule->p_direction,"both")==SAME ) &&
	 cmpObjects(erule->src , firewall)==SAME &&  
	 cmpObjects(erule->dst , firewall)==SAME ) {

	trule=createElementaryPolicyRule();
	copyElementaryPolicyRule( trule , erule );
	trule->p_direction="inbound";
	if (no_state) trule->t->state= 0;
	processElementaryPolicyRule(firewall,iface,rule,trule);
    
	trule=createElementaryPolicyRule();
	copyElementaryPolicyRule( trule , erule );
	trule->p_direction="outbound";
	if (no_state) trule->t->state= 0;
	processElementaryPolicyRule(firewall,iface,rule,trule);

	return;

    }

/* decide on the chain this rule should go to and on the action chain  */

    erule->p_group="FORWARD";
    
    if ( (erule->p_direction==NULL ||
	  strcasecmp(erule->p_direction,"inbound")==0 ) &&
	 cmpObjects(erule->dst , firewall)==SAME )  erule->p_group="INPUT";

    if ( (erule->p_direction==NULL ||
	  strcasecmp(erule->p_direction,"outbound")==0 ) &&
	 cmpObjects(erule->src , firewall)==SAME )  erule->p_group="OUTPUT";
 
    erule->master_group = copyStr( erule->p_group );
    
    if (erule->src_neg || erule->dst_neg || erule->srv_neg) {

	if ( strcasecmp(erule->p_group,"INPUT")==SAME) {
	    trule=createElementaryPolicyRule();
	    copyElementaryPolicyRule( trule , erule );

	    trule->src=NULL;
	    trule->dst=NULL;
	    trule->srv=NULL;
	    trule->p_action  = copyStr(temp_i_chain);
	    trule->t->state  = 0;
	    trule->final     = 1;
	    processElementaryPolicyRule(firewall,iface,rule,trule);
	}

	if ( strcasecmp(erule->p_group,"OUTPUT")==SAME) {
	    trule=createElementaryPolicyRule();
	    copyElementaryPolicyRule( trule , erule );

	    trule->src=NULL;
	    trule->dst=NULL;
	    trule->srv=NULL;
	    trule->p_action  = copyStr(temp_o_chain);
	    trule->t->state  = 0;
	    trule->final     = 1;
	    processElementaryPolicyRule(firewall,iface,rule,trule);
	}

	if ( strcasecmp(erule->p_group,"FORWARD")==SAME) {
	    trule=createElementaryPolicyRule();
	    copyElementaryPolicyRule( trule , erule );

	    trule->src=NULL;
	    trule->dst=NULL;
	    trule->srv=NULL;
	    trule->p_action  = copyStr(temp_f_chain);
	    trule->t->state  = 0;
	    trule->final     = 1;
	    processElementaryPolicyRule(firewall,iface,rule,trule);
	}

	erule->p_group      = copyStr( trule->p_action );
	erule->p_action     = "RETURN";

    } else {

	erule->p_action  = copyStr(rule_chain);

    }


    
/*
 *  The follwoing fragment handles the case when firewall object is source
 *  in the rule which goes into INPUT chain, or it is destination in
 *  the rule which goes into OUTPUT chain. We will go over all interfaces
 *  of the firewall object and generate multiple rules using address of
 *  each interface instead.
 *
 *  Exceptions: if chain is INPUT and firewall object is in
 *  destination, then we do not need to pick every interface. The same
 *  goes to the case with chain OUTPUT and firewall object in source
 */
    
    fs=fd=0;

    if ( cmpObjects(erule->src , firewall)==SAME ) fs=1;
    if ( cmpObjects(erule->dst , firewall)==SAME ) fd=1;

    if (fs==0 && fd==0) {
	parsePolicyRule( rule,erule);
	return;
    }

    
    if (fs==fd) {
	if ( strcasecmp(erule->p_group,"INPUT")==SAME  ) fd=0;
	if ( strcasecmp(erule->p_group,"OUTPUT")==SAME ) fs=0;
    }

    if ( (strcasecmp(erule->p_group,"INPUT")==SAME && fd ) ||
	 (strcasecmp(erule->p_group,"OUTPUT")==SAME && fs ) ) {

	parsePolicyRule( rule,erule);
	return;
    }

    if (fs || fd ) {
	for(c1=firewall->xmlChildrenNode; c1; c1=c1->next) {
	    if ( xmlIsBlankNode(c1) ) continue;
	    if (isElement(c1,"Interface")) {
		if ( isDynamicInterface(c1) ) continue;

		trule=createElementaryPolicyRule();
		copyElementaryPolicyRule(trule , erule);
		if (no_state) trule->t->state= 0;

		ifaddr=getStr(c1,"address");
		if (strcmp(ifaddr,"127.0.0.1")==SAME) continue;
		if (fs) {
		    trule->src=c1;
		    parsePolicyRule( rule,trule);
		}
		if (fd) {
		    trule->dst=c1;
		    parsePolicyRule( rule,trule);
		}

	    }
	}


    } else 	
	parsePolicyRule( rule,erule);
    
}



void processPolicyRule(xmlNodePtr firewall,
		       xmlNodePtr iface,
		       xmlNodePtr rule,
		       const char *comment,
		       GSList     *subrules)
{
    GSList   *list_ptr1;
    elementaryRule *erule;
    int       i,o,f, srn;


    i=o=f=0;


/*    dumpPolicyRule(stderr,doc,rule); */

    ready_subrules=NULL;
    
    rulePrologue(firewall, iface, rule , g_slist_length(subrules) );

    srn=0;
    for (list_ptr1=subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {

	erule=(elementaryRule*)(list_ptr1->data);

	if (erule->num==-1) continue;

	erule->subrule_no=srn;
	if (no_state) erule->t->state= 0;

	processElementaryPolicyRule(firewall,iface,rule,erule);
	
	srn++;
    }
    
    ruleEpilogue(firewall, iface, rule , ready_subrules);

    if (! global_no_optimisation) optimisePolicyRules();

    printPolicyRule();
}


void parsePolicyRule(xmlNodePtr rule,elementaryRule *erule)
{
    xmlNodePtr  src,dst,srv;

    src=erule->src;
    dst=erule->dst;
    srv=erule->srv;


    if (erule->t->p_options) {
	free(erule->t->p_options);
	erule->t->p_options=NULL;
    }

    
    if (src!=NULL) processSrc(src,erule->t);

    if (dst!=NULL) processDst(dst,erule->t);

    if (srv==NULL || isAny(srv) ) {
	storePolicyRule(erule);
	return;
    }

    processSrv(srv,erule->t);

    storePolicyRule(erule);
}


void storePolicyRule(elementaryRule *erule)
{
    ready_subrules=g_slist_append(ready_subrules , erule);

}

/*
 *  returns 1 if rule r1 "shades" rule r2, that is r1's destination
 *  is broader than r2's, or r1's source is broader than r2's
 *  Otherwise returns 0;
 */
int checkRuleShading(elementaryRule *r1,elementaryRule *r2)
{
    triplet t1, t2;

    if (cmpParameter(r1->p_group,r2->p_group)==SAME &&
	cmpParameter(r1->p_action,r2->p_action)==SAME &&
/*	cmpParameter(r1->p_direction,r2->p_direction)==SAME && */
	cmpParameter(r1->p_iface,r2->p_iface)==SAME ) {

	copyTriplet( &t1 , r1->t );
	copyTriplet( &t2 , r2->t );

	if (t1.p_src==NULL) t1.p_src="0/0";
	if (t1.p_dst==NULL) t1.p_dst="0/0";
	if (t2.p_src==NULL) t2.p_src="0/0";
	if (t2.p_dst==NULL) t2.p_dst="0/0";
				
	if ( cmpParameter(t1.p_src ,t2.p_src)==SAME &&
	     cmpParameter(t1.p_sprt,t2.p_sprt)==SAME &&
	     cmpParameter(t1.p_dprt,t2.p_dprt)==SAME ) {
/* compare dst */

	    if (strcmp(t1.p_dst,"0/0")==SAME &&
		strcmp(t2.p_dst,"0/0")!=SAME )  return 1;

	    return 0;
	}
	
	if ( cmpParameter(t1.p_dst, t2.p_dst)==SAME &&
	     cmpParameter(t1.p_sprt,t2.p_sprt)==SAME &&
	     cmpParameter(t1.p_dprt,t2.p_dprt)==SAME ) {
/* compare src */
	    
	    if (strcmp(t1.p_src,"0/0")==SAME &&
		strcmp(t2.p_src,"0/0")!=SAME )  return 1;

	    return 0;
	}

    }
    return 0;
}

void optimisePolicyRules()
{

    elementaryRule     *prptr1, *prptr2;
    GSList             *list_ptr1, *list_ptr2;

/*
 *  Phase 1: no need in destination in INPUT chain and source in
 *  OUTPUT chain
 */
    for (list_ptr1=ready_subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {

	prptr1=(elementaryRule*)(list_ptr1->data);
	if (prptr1->num==-1) continue;
	    

	if (prptr1->master_group &&
	    strcasecmp(prptr1->master_group,"INPUT")==SAME) {
		
	    prptr1->dst=NULL;
	    prptr1->t->p_dst=NULL;
	}
	    
	if (prptr1->master_group &&
	    strcasecmp(prptr1->master_group,"OUTPUT")==SAME) {
		
	    prptr1->src=NULL;
	    prptr1->t->p_src=NULL;
	}
    }
    
/*
 *  Phase 2: check for identical rules and rule shading
 */
 again:
    for (list_ptr1=ready_subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {

	prptr1=(elementaryRule*)(list_ptr1->data);
	if (prptr1->num==-1) continue;
	    
	for (list_ptr2=g_slist_next(list_ptr1);
	     list_ptr2!=NULL;
	     list_ptr2=g_slist_next(list_ptr2)) {

	    prptr2=(elementaryRule*)(list_ptr2->data);

	    if (cmpPPolicyRules( prptr1,prptr2 )==SAME) {
		ready_subrules=g_slist_remove(ready_subrules, prptr2);
		goto again;
	    }

	    if (checkRuleShading(prptr1,prptr2)==1) {
		ready_subrules=g_slist_remove(ready_subrules, prptr2);
		goto again;
	    }

	    if (checkRuleShading(prptr2,prptr1)==1) {
		ready_subrules=g_slist_remove(ready_subrules, prptr1);
		goto again;
	    }
	}
    }



}

void printPolicyRule()
{
    elementaryRule     *erule;
    GSList             *list_ptr1;
    
    for (list_ptr1=ready_subrules;
	 list_ptr1!=NULL;
	 list_ptr1=g_slist_next(list_ptr1)) {
	
	erule=(elementaryRule*)(list_ptr1->data);
    
	fprintf(ofile,"iptables -A %s ",erule->p_group);

	if (erule->p_direction!=NULL && erule->p_iface!=NULL) {
	    if ( strcasecmp(erule->p_direction,"inbound")==0)
		fprintf(ofile,"-i %s ",erule->p_iface);
	    if ( strcasecmp(erule->p_direction,"outbound")==0)
		fprintf(ofile,"-o %s ",erule->p_iface);
	}
	
	if ( erule->t->p_proto ) {
	    fprintf(ofile," -p %s",erule->t->p_proto);
	}
    
	if ( erule->t->state ) 
	    fprintf(ofile," -m state --state NEW ");

	if (strcmp( erule->p_group,"OUTPUT")!=SAME ) {
	    if (erule->t->p_src!=NULL)	fprintf(ofile," -s %s",erule->t->p_src);
	}
	if ( erule->t->p_sprt && strlen(erule->t->p_sprt)!=0)
	    fprintf(ofile," --source-port %s",erule->t->p_sprt);
    
	if (strcmp( erule->p_group,"INPUT")!=SAME ) {
	    if (erule->t->p_dst!=NULL)	fprintf(ofile," -d %s",erule->t->p_dst);
	}
	if ( erule->t->p_dprt && strlen(erule->t->p_dprt)!=0)
	    fprintf(ofile," --destination-port %s",erule->t->p_dprt);

	if ( erule->t->p_icmp_type )
	    fprintf(ofile," --icmp-type %s", erule->t->p_icmp_type);

	if (erule->p_action!=NULL) {
	    fprintf(ofile," -j %s",erule->p_action);
	}
	
	if (erule->t->p_options!=NULL)  {
	    fprintf(ofile," %s",erule->t->p_options);
	}
	fprintf(ofile,"\n");
    }
}



